#ifndef BatchLab_BaseFitter_h
#define BatchLab_BaseFitter_h

// Inheritance :
#include <AIDA/IFitter.h>

#include <AIDA/IConstants.h> //For AIDA_VERSION

namespace AIDA {class IFunctionFactory;}
namespace AIDA {class ITupleFactory;}
namespace AIDA {class IAnnotation;}

namespace Slash {namespace Core {class IWriter;}}

#include <BatchLab/Core/FitParameterSettings.h>

namespace BatchLab {

class BaseFitter : public virtual AIDA::IFitter {
public: //AIDA::IFitter
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,const std::string&,
                                const std::vector<double>&);

  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,const std::string&,
                                const std::vector<double>&);

  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,const std::string&,
                                const std::vector<double>&);
#else
  virtual AIDA::IFitResult* fit(AIDA::IFitData& fitData,
                                AIDA::IFunction& func,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IFitData& fitData,
                                const std::string& model,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IFitData& fitData,
                                const std::string& model,
                                const std::vector<double>& initialParams,
                                const std::string& range = ""); //AIDA-v3r3p0

  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram& hist,
                                AIDA::IFunction& func,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram& fitData,
                                const std::string& model,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram& fitData,
                                const std::string& model,
                                const std::vector<double>& initialParams,
                                const std::string& range = ""); //AIDA-v3r3p0

  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet& hist,
                                AIDA::IFunction& func,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet& fitData,
                                const std::string& model,
                                const std::string& range = ""); //AIDA-v3r3p0
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet& fitData,
                                const std::string& model,
                                const std::vector<double>& initialParams,
                                const std::string& range = ""); //AIDA-v3r3p0
#endif
  virtual bool setEngine(const std::string&);
  virtual std::string engineName();
  virtual AIDA::IFitParameterSettings& fitParameterSettings(const std::string&);
  virtual std::vector<std::string> listParameterSettings();
  virtual void resetParameterSettings();
  virtual bool setConstraint(const std::string&);
  virtual std::vector<std::string> constraints();
  virtual void resetConstraints();
  virtual bool setFitMethod(const std::string&);
  virtual std::string fitMethodName();
  virtual AIDA::IDataPointSet* createScan1D(AIDA::IFitData&,
                                            AIDA::IFunction&,
                                            const std::string&,
                                            int,double,double);
  virtual AIDA::IDataPointSet* createContour(AIDA::IFitData&,
                                             AIDA::IFitResult&,
                                             const std::string&,
                                             const std::string&,int,double);
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual void setUseFunctionGradient(bool);
#else
  virtual bool setUseFunctionGradient(bool);
#endif
  virtual bool useFunctionGradient();
public: //BatchLab::BaseFitter :
  virtual bool baseFit(AIDA::IFunction&,
                       const std::vector< std::vector<double> >&,
                       const std::vector<double>&,
                       const std::vector<double>&,
                       double&,std::vector<double>&) = 0;
public:
  BaseFitter(Slash::Core::IWriter&,
             AIDA::IFunctionFactory&,
             AIDA::ITupleFactory&,
             const std::string&);
  virtual ~BaseFitter();
  virtual void* cast(const std::string&) const;
public:
  static void setFitInfos(AIDA::IAnnotation&,AIDA::IFitResult&);
private:
  static double userFunction(double,const std::vector<double>&,void*);
protected:
  Slash::Core::IWriter& fPrinter;
private:
  AIDA::IFunctionFactory& fFunctionFactory;
  AIDA::ITupleFactory& fTupleFactory;
  std::string fEngineName;
  std::vector<std::string> fParameterSettings;
  std::vector<std::string> fConstraints;
  std::string fFitMethodName;
  int fCounter;
  std::vector<FitParameterSettings> fFitParameterSettings;
};

}

#endif
