#ifndef BatchLab_HistogramAllocator_h
#define BatchLab_HistogramAllocator_h

// Inheritance :
#include <Slash/Core/IManager.h>

#include <vector>

namespace Slash {namespace Core {class IWriter;}}

namespace HCL {
  class Histogram;
}

namespace BatchLab {

class Histogram1D;
class Histogram2D;
class Histogram3D;
class Profile1D;
class Profile2D;
class Cloud1D;
class Cloud2D;
class Cloud3D;

class HistogramAllocator : public virtual Slash::Core::IManager {
public: 
  HistogramAllocator(Slash::Core::IWriter&);
  virtual ~HistogramAllocator();
public: //Slash::Core::IManager
  virtual std::string name() const;
  virtual void* cast(const std::string&) const;
public:
  // new_ methods having same signature than constructors :
  virtual BatchLab::Histogram1D* new_Histogram1D
            (Slash::Core::IWriter&,const std::string&,HCL::Histogram&);
  virtual BatchLab::Histogram1D* new_Histogram1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double);
  virtual BatchLab::Histogram1D* new_Histogram1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&);
  virtual BatchLab::Histogram1D* new_Histogram1D(const Histogram1D&);

  virtual BatchLab::Histogram2D* new_Histogram2D
            (Slash::Core::IWriter&,const std::string&,HCL::Histogram&);
  virtual BatchLab::Histogram2D* new_Histogram2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double,
             int,double,double);
  virtual BatchLab::Histogram2D* new_Histogram2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&,
             const std::vector<double>&);
  virtual BatchLab::Histogram2D* new_Histogram2D(const Histogram2D&);

  virtual BatchLab::Histogram3D* new_Histogram3D
            (Slash::Core::IWriter&,const std::string&,HCL::Histogram&);
  virtual BatchLab::Histogram3D* new_Histogram3D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double,
             int,double,double,
             int,double,double);
  virtual BatchLab::Histogram3D* new_Histogram3D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&,
             const std::vector<double>&,
             const std::vector<double>&);
  virtual BatchLab::Histogram3D* new_Histogram3D(const Histogram3D&);

  virtual BatchLab::Profile1D* new_Profile1D
            (Slash::Core::IWriter&,const std::string&,HCL::Histogram&);
  virtual BatchLab::Profile1D* new_Profile1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double);
  virtual BatchLab::Profile1D* new_Profile1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double,
             double,double);
  virtual BatchLab::Profile1D* new_Profile1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&);
  virtual BatchLab::Profile1D* new_Profile1D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&,
             double,double);
  virtual BatchLab::Profile1D* new_Profile1D(const BatchLab::Profile1D&);

  virtual BatchLab::Profile2D* new_Profile2D
            (Slash::Core::IWriter&,const std::string&,HCL::Histogram&);
  virtual BatchLab::Profile2D* new_Profile2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double,
             int,double,double);
  virtual BatchLab::Profile2D* new_Profile2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             int,double,double,
             int,double,double,
             double,double);
  virtual BatchLab::Profile2D* new_Profile2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&,
             const std::vector<double>&);
  virtual BatchLab::Profile2D* new_Profile2D
            (Slash::Core::IWriter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&,
            const std::vector<double>&,
             double,double);
  virtual BatchLab::Profile2D* new_Profile2D(const BatchLab::Profile2D&);

  virtual BatchLab::Cloud1D* new_Cloud1D
            (Slash::Core::IWriter&,
             const std::string&,const std::string&,int=-1);
  virtual BatchLab::Cloud1D* new_Cloud1D(const Cloud1D&);

  virtual BatchLab::Cloud2D* new_Cloud2D
            (Slash::Core::IWriter&,
             const std::string&,const std::string&,int=-1);
  virtual BatchLab::Cloud2D* new_Cloud2D(const Cloud2D&);

  virtual BatchLab::Cloud3D* new_Cloud3D
            (Slash::Core::IWriter&,
             const std::string&,const std::string&,int=-1);
  virtual BatchLab::Cloud3D* new_Cloud3D(const Cloud3D&);
private:
  Slash::Core::IWriter& fPrinter;
  std::string fName;
};

}

#endif
