
#include <Slash/Tools/smanip.h>
#include <Slash/Tools/args.h>

#include <Lib/Printer.h>
#include <Lib/Debug.h>

#include <Lib/ItemML.h>
#include <Lib/BaseML.h>

#define ACTION_COUNT "count"
#define ID_COUNT 1

class AIDA_ML : public Lib::BaseML {
public:
  AIDA_ML(Lib::ItemMLFactory& aFactory,
          int aVerbose,
          const std::string& aAction)
  :Lib::BaseML(aFactory,aVerbose)
  ,fAction(0)
  //
  ,f_objects(0)
  ,f_histogram1d(0)
  ,f_histogram2d(0)
  ,f_histogram3d(0)
  ,f_profile1d(0)
  ,f_profile2d(0)
  ,f_cloud1d(0)
  ,f_cloud2d(0)
  ,f_cloud3d(0)
  ,f_tuple(0)
  ,f_dataPointSet(0)
  ,f_function(0)
  {
    if(aAction==ACTION_COUNT) fAction = ID_COUNT;
  }
public:
  virtual bool visitEndElement(Lib::ItemML& aItemML,bool& aKeep) {
    if(fAction==ID_COUNT) {
      const std::string& tagName = aItemML.tagName();
      if(tagName=="aida") {
        aKeep = false;
      } else if(tagName=="histogram1d") {
        f_histogram1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="histogram2d") {
        f_histogram2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="histogram3d") {
        f_histogram3d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="profile1d") {
        f_profile1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="profile2d") {
        f_profile2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud1d") {
        f_cloud1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud2d") {
        f_cloud2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud3d") {
        f_cloud3d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="tuple") {
        f_tuple++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="dataPointSet") {
        f_dataPointSet++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="function") {
        f_function++;
        f_objects++;
        aKeep = false;
      } else {
        aKeep = true;
      }
    } else {
      aKeep = false;
    }
    return true;
  }
  void dump(std::ostream& aOut) const { 
    if(fAction==ID_COUNT) {
      aOut << " AIDA objects " << f_objects << std::endl;
      aOut << " histogram1d = " << f_histogram1d << std::endl;
      aOut << " histogram2d = " << f_histogram2d << std::endl;
      aOut << " histogram3d = " << f_histogram3d << std::endl;
      aOut << " profile1d = " << f_profile1d << std::endl;
      aOut << " profile2d = " << f_profile2d << std::endl;
      aOut << " cloud1d = " << f_cloud1d << std::endl;
      aOut << " cloud2d = " << f_cloud2d << std::endl;
      aOut << " cloud3d = " << f_cloud3d << std::endl;
      aOut << " tuple = " << f_tuple << std::endl;
      aOut << " dataPointSet = " << f_dataPointSet << std::endl;
      aOut << " function = " << f_function << std::endl;
    }
  }
private:
  int fAction;
  //
  int f_objects;
  int f_histogram1d;
  int f_histogram2d;
  int f_histogram3d;
  int f_profile1d;
  int f_profile2d;
  int f_cloud1d;
  int f_cloud2d;
  int f_cloud3d;
  int f_tuple;
  int f_dataPointSet;
  int f_function;
};

#include <iostream>

int main(int aArgc,char** aArgv) {

 {Slash::args args(aArgc,aArgv);

  if(args.is_arg("-h")) {
    std::cout << "OS> BatchLab_xml.exe [options] <file>" << std::endl;
    std::cout << " -h : help" << std::endl;
    std::cout << " -action=<int> : action" << std::endl;
    return 0;
  }

  int verboseLevel = 0;
  std::string sverbose;
  args.find("-verbose",verboseLevel);

  std::string file;
  if(!args.file(file)) {
    std::cout << "no XML aida file given." << std::endl;
    return 0;
  }

  std::string action;
  if(!args.find("-action",action)) action = "count";

  Lib::ItemMLFactory factory;
  AIDA_ML aidaML(factory,verboseLevel,action);
  std::vector<std::string> tags;
  tags.push_back("aida");
  tags.push_back("histogram1d");
  tags.push_back("histogram2d");
  tags.push_back("histogram3d");
  tags.push_back("profile1d");
  tags.push_back("profile2d");
  tags.push_back("axis");
  tags.push_back("statistics");
  tags.push_back("data1d");
  tags.push_back("data2d");
  tags.push_back("data3d");
  tags.push_back("tuple");
  tags.push_back("columns");
  tags.push_back("rows");
  tags.push_back("row");
  tags.push_back("entryTuple");
  tags.push_back("cloud1d");
  tags.push_back("cloud2d");
  tags.push_back("cloud3d");
  tags.push_back("entries1d");
  tags.push_back("entries2d");
  tags.push_back("entries3d");
  tags.push_back("dataPointSet");
  tags.push_back("dataPoint");
  tags.push_back("function");
  tags.push_back("arguments");
  tags.push_back("argument");
  tags.push_back("parameters");
  tags.push_back("annotation");
  aidaML.setTags(tags);
  Lib::Printer printer;
  if(!aidaML.loadFile(file,false,printer)) {
    std::cout << "Lib_xml : " 
        << " problem when loading file " << Slash::sout(file) << std::endl;
  } else { 
    aidaML.dump(std::cout);
  }


 }Lib::Debug::balance(std::cout);

  return 0;
}
