// this :
#include <BatchLab/Core/BaseObject.h>

#include <Slash/Core/IRelationManager.h>
#include <Slash/Core/IRelationVisitor.h>
#include <Slash/Core/IRelation.h>
#include <Slash/Thread/IMutex.h>

#include <Slash/Tools/Relation.h>

#include <Lib/Debug.h>
#include <Lib/Cast.h>
#include <Lib/Mutex.h>
#include <Lib/Out.h>

#include <Slash/Store/IFolder.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::BaseObject(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:fPrinter(aPrinter)
,fName(aName)
,fFolder(0)
,fRelationManager(0)
,fVerboseLevel(0)
,fMutex(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::BaseObject(
 const BaseObject& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,fPrinter(aFrom.fPrinter)
,fName(aFrom.fName)
,fFolder(0)
,fRelationManager(0)
,fMutex(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::~BaseObject(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fMutex) delete fMutex;
  if(fFolder) fFolder->remove(this);
  if(fRelationManager) {    
    Slash::RV_Or_Remove visitor(this,this);
    fRelationManager->remove(visitor,true);
  }
  Lib::Debug::decrement("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseObject::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IManagedObject)
  else if_Lib_SCast(Slash::Store::IObject) 
  else if_Lib_SCast(BatchLab::BaseObject)
  else return 0;
}

//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::emitUpdate(
)
//////////////////////////////////////////////////////////////////////////////
// To update plotters.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fRelationManager) {
    Slash::RV_First_Schedule visitor(this);
    fRelationManager->visit(visitor);
  }
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setFolder(
 Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFolder = aFolder;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setRelationManager(
 Slash::Core::IRelationManager* aRelationManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fRelationManager = aRelationManager;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseObject::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fName = aName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& BatchLab::BaseObject::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::BaseObject::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}

bool BatchLab::BaseObject::setThreadSafe(bool a_safe){
  if(a_safe) {
    if(fMutex) return true; //done.
    Lib::Mutex* mutex = new Lib::Mutex();
    if(!mutex) return false;
    if(!mutex->initialize()) {
      //Lib::StreamBuffer sbuf(fPrinter);
      //std::ostream out(&sbuf);
      Lib::Out out(fPrinter);
      out << "BatchLab::BaseObject::setThreadSafe :"
          << " failed."
          << Lib::endl;
      delete mutex;
      return false;
    }
    fMutex = mutex;
    return true;
  } else {
    delete fMutex;
    fMutex = 0;
    return true;
  }
}

bool BatchLab::BaseObject::hasMutex() const {return fMutex?true:false;}

bool BatchLab::BaseObject::lock() const {
  if(!fMutex) return true; //we consider that thread safety not required.
  return fMutex->lock();
}

bool BatchLab::BaseObject::unlock() const {
  if(!fMutex) return true; //we consider that thread safety not required.
  return fMutex->unlock();
}

