// this :
#include <BatchLab/Core/Calculator.h>

#include <AIDA/ITuple.h>

#include <Lib/Out.h>
#include <Lib/Processor.h>
#include <Lib/Debug.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <BatchLab/Core/BaseTuple.h>
 
//to find AIDA::IFunction.
#include <Slash/Store/ITree.h>
#include <AIDA/ITree.h>
#include <AIDA/IFunction.h>
#include <Lib/ExpFunc.h>
#include <Lib/valuemanip.h>

#define NotFound (-1)

//////////////////////////////////////////////////////////////////////////////
BatchLab::Calculator::Calculator(
 Slash::Core::IWriter& aPrinter
,const std::string& aScript
,AIDA::ITree& aTree
)
:fPrinter(aPrinter)
,fTree(aTree)
,fTuple(0)
,fScript(aScript)
,fBaseTuple(0)
,fProcessor(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::Calculator");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Calculator::~Calculator(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fProcessor;
  Lib::Debug::decrement("BatchLab::Calculator");
}

//private:
BatchLab::Calculator::Calculator(const Calculator& aFrom)
:fPrinter(aFrom.fPrinter)
,fTree(aFrom.fTree)
,fTuple(0)
,fScript(aFrom.fScript)
,fBaseTuple(0)
,fProcessor(0)
{
  Lib::Debug::increment("BatchLab::Calculator");
}
//private:
BatchLab::Calculator& BatchLab::Calculator::operator=(const Calculator&) {return *this;}


class ExpFunc_AIDA_FUNC : public Lib::ExpFunc {
public:
  virtual bool eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
    if(a_args.size()!=fNumArgs) return false;
    if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
    } else {
      std::vector<double> ds(fNumArgs);
      std::vector<Lib::Value>::const_iterator it;
      std::vector<double>::iterator it2 = ds.begin();
      for(it=a_args.begin();it!=a_args.end();++it,++it2) {
        if(!Lib::valuemanip::to_double(*it,*it2)) {
          aError = 
            "Lib::Expression::execute : can't convert Variable to double.";
          a_return.setNone();
          return false;
        }
      }
      a_return.set(fFunction.value(ds));
      return true;
    }
    return false;
  }
  virtual Lib::IExpFunc* copy() const { return new ExpFunc_AIDA_FUNC(*this);}
public:
  ExpFunc_AIDA_FUNC(const std::string& aName,AIDA::IFunction& aFunction)
  :ExpFunc(aName,aFunction.dimension()),fFunction(aFunction){}
  virtual ~ExpFunc_AIDA_FUNC(){}
private:
  ExpFunc_AIDA_FUNC(const ExpFunc_AIDA_FUNC& aFrom)
  : Lib::ExpFunc(aFrom),fFunction(aFrom.fFunction){}
  ExpFunc_AIDA_FUNC& operator=(const ExpFunc_AIDA_FUNC& aFrom){ Lib::ExpFunc::operator=(aFrom);return *this;}
private:
  AIDA::IFunction& fFunction;
};

//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Calculator::initialize(
 AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTuple = 0;
  delete fProcessor;
  fProcessor = 0;
  fNames.clear();
  fIndices.clear();
  fBaseTuple = 0;
  fBaseCols.clear();
  fVals.clear();

  fProcessor = new Lib::Processor(fScript,fPrinter);

  // Get cut variables :
  if(!fProcessor->findVariables(fNames)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Calculator::initialize : "
        << " findVariables failed." 
        << Lib::endl;
    fNames.clear();
    delete fProcessor;
    fProcessor = 0;
    return false;    
  }

  // Get functions names :
 {std::vector<std::string> funcs;
  if(!fProcessor->findFunctions(funcs)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Calculator::initialize : "
        << " findFunctions failed." 
        << Lib::endl;
    fNames.clear();
    delete fProcessor;
    fProcessor = 0;
    return false;    
  }
 {std::vector<std::string>::const_iterator it;
  for(it=funcs.begin();it!=funcs.end();++it) {
    if(fProcessor->findFunction(*it,false)) continue;
    AIDA::IManagedObject* bo = fTree.find(*it);
    //Slash::Store::IObject* bo = stree->findObject(*it);
    if(!bo) {
      Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize : "
          << " object " << Lib::sout(*it) 
          << " not found in tree with store " << Lib::sout(fTree.storeName())
          << Lib::endl;
      fNames.clear();
      delete fProcessor;
      fProcessor = 0;
      return false;    
    }
    AIDA::IFunction* func = Lib_Cast(*bo,AIDA::IFunction);
    if(!func) {
      Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize : "
          << " object " << Lib::sout(*it) 
          << " not an AIDA::IFunction."
          << Lib::endl;
      fNames.clear();
      delete fProcessor;
      fProcessor = 0;
      return false;    
    }
    fProcessor->addFunction(new ExpFunc_AIDA_FUNC(*it,*func));
  }}
  }

  if(!fProcessor->compile(fNames)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Calculator::initialize : "
        << " compilation failed." 
        << Lib::endl;
    fNames.clear();
    delete fProcessor;
    fProcessor = 0;
    return false;    
  }

  unsigned int number = fNames.size(); //Could be zero.
  fIndices.resize(number);
  for(unsigned int count=0;count<number;count++) {
    //printf("debug : variable \"%s\" %lu\n",fNames[count].c_str(),&aTuple);
    int index = aTuple.findColumn(fNames[count]);
    if(index==NotFound) {
      Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize : "
          << Lib::sout(fNames[count]) << " not a column of tuple with title " 
          << Lib::sout(aTuple.title()) 
          << Lib::endl;
      delete fProcessor;
      fProcessor = 0;
      fNames.clear();
      fIndices.clear();
      fBaseTuple = 0;
      fBaseCols.clear();
      fVals.clear();
      // Dump column names :
     {Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize :"
          << " columns are :"
          << Lib::endl;
      for(unsigned int count=0;count<number;count++) {
        out << Lib::sout(aTuple.columnName(count)) << Lib::endl;
      }}
      return false;
    } 
    fIndices[count] = index;
  }
  fTuple = &aTuple;

  fVals.resize(number);

  // To optimize the evaluateDouble :
  if(!fTuple->cast("BatchLab::ChainedTuple")) {
    fBaseTuple = Lib_ConstCast(*fTuple,BatchLab::BaseTuple);
    if(fBaseTuple) {
      fBaseCols.resize(number);
      for(unsigned int count=0;count<number;count++) {
        fBaseCols[count] = fBaseTuple->findColumn1(fIndices[count]);
      }
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Calculator::getVariables(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fProcessor) return false;

  unsigned int number = fNames.size(); //Could be zero.

  if(fBaseTuple) {  
    // Faster :

    int irow = fBaseTuple->rowIndex();

    for(unsigned int count=0;count<number;count++) {
      BaseColumn* column = fBaseCols[count];
      if(!column->value(irow,fVals[count])) {
        Lib::Out out(fPrinter);
        out << "BatchLab::Calculator::evaluateDouble : "
            << " can't get Variable value for column " 
            << Lib::sout(column->name()) << "."
            << Lib::endl;
        return false;
      }
    }

  } else {

    // Through AIDA::ITuple only but more slower :

    for(unsigned int count=0;count<number;count++) {
      int icol = fIndices[count];
      std::string type = fTuple->columnType(icol);
      if(type=="float") {
        fVals[count].set(fTuple->getFloat(icol));
      } else if(type=="double") {
        fVals[count].set(fTuple->getDouble(icol));
      } else if(type=="char") {
        fVals[count].set(fTuple->getChar(icol));
      } else if(type=="short") {
        fVals[count].set(fTuple->getShort(icol));
      } else if(type=="int") {
        fVals[count].set(fTuple->getInt(icol));
      } else if(type=="boolean") {
        fVals[count].set(fTuple->getBoolean(icol));
      } else {
        Lib::Out out(fPrinter);
        out << "BatchLab::Calculator::evaluateDouble : "
            << " make no sense on column type " << type << "."
            << Lib::endl;
      }
    }
  }

  return true;
}
