// this :
#include <BatchLab/Core/DataPointSet.h>

#include <Lib/Cast.h>
#include <Lib/Sizes.h>

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

// Empty cstor used by IO readers.

BatchLab::DataPointSet::DataPointSet(Slash::Core::IWriter& aPrinter,const std::string& aName)
:BaseObject(aPrinter,aName)
,fDimension(0)
{}

BatchLab::DataPointSet::DataPointSet(Slash::Core::IWriter& aPrinter,const std::string& aName,const std::string& aTitle,int aDimension)
:BaseObject(aPrinter,aName)
,fDimension(aDimension)
{
  setTitle(aTitle);
}

BatchLab::DataPointSet::DataPointSet(const DataPointSet& aFrom)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IDataPointSet(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Store::IArray(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
,fTitle(aFrom.fTitle)
,fDimension(aFrom.fDimension)
,fPoints(aFrom.fPoints)
{}

BatchLab::DataPointSet::~DataPointSet(){}

BatchLab::Annotation& BatchLab::DataPointSet::getAnnotation(){return fAnnotation;}

void BatchLab::DataPointSet::setAnnotation(const BatchLab::Annotation& aFrom){fAnnotation = aFrom;}

Slash::Store::IObject* BatchLab::DataPointSet::copy() const{
  return new BatchLab::DataPointSet(*this);
}

std::string BatchLab::DataPointSet::storeClassName() const{return "BatchLab::DataPointSet";}

std::string BatchLab::DataPointSet::type() const {return "AIDA::IDataPointSet";}

void* BatchLab::DataPointSet::cast(const std::string& aClass) const {
  if_Lib_SCast(AIDA::IDataPointSet)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::DataPointSet)
  else return BaseObject::cast(aClass);
}

AIDA::IAnnotation& BatchLab::DataPointSet::annotation() {return fAnnotation;}
const AIDA::IAnnotation& BatchLab::DataPointSet::annotation() const {return fAnnotation;}
int BatchLab::DataPointSet::dimension() const {return fDimension;}
std::vector<BatchLab::DataPoint>& BatchLab::DataPointSet::points(){return fPoints;}

SLASH_STORE_SOURCE(BatchLab::DataPointSet)
bool BatchLab::DataPointSet::visit(Slash::Store::IConstVisitor& aVisitor) const{
  SLASH_STORE_BEGIN(BatchLab::DataPointSet)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fDimension",fDimension)) return false;
  if(!aVisitor.visit_double("fPoints",*this)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}

bool BatchLab::DataPointSet::read(Slash::Store::IVisitor& aVisitor){
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fDimension)) return false;
  if(!aVisitor.visit_double(*this)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}

//////////////////////////////////////////////////////////////////////////////
/// lock/unlocked ins ////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::setTitle(const std::string& aTitle) { 
  lock();
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  unlock();
  return true;
}

void BatchLab::DataPointSet::clear() {
  lock();
  fPoints.clear();
  unlock();
}

AIDA::IDataPoint* BatchLab::DataPointSet::addPoint() { 
  lock();
  fPoints.push_back(DataPoint(fDimension));
  const DataPoint& dataPoint = fPoints.back();
  AIDA::IDataPoint* pt = (AIDA::IDataPoint*)&dataPoint;
  unlock();
  emitUpdate();
  return pt;
}

bool BatchLab::DataPointSet::addPoint(const AIDA::IDataPoint& aPoint) { 
  lock();
  fPoints.push_back(DataPoint(aPoint));
  unlock();
  emitUpdate();
  return true;
}

bool BatchLab::DataPointSet::removePoint(int aIndex) { 
  lock();
  bool stat = false;
  if( (aIndex>=0) && (aIndex<(int)fPoints.size()) ){
    std::vector<DataPoint>::iterator it = fPoints.begin();
    for(int i=0;i<aIndex;i++) it++;
    fPoints.erase(it);
    stat = true;
  }
  unlock();
  if(stat) emitUpdate();
  return stat;
}

bool BatchLab::DataPointSet::scale(double aScaleFactor) { 
  lock();
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setValue(m->value() * aScaleFactor);
      m->setErrorPlus(m->errorPlus() * aScaleFactor);
      m->setErrorMinus(m->errorMinus() * aScaleFactor);
    }
  }
  unlock();
  if(fPoints.size()) emitUpdate();
  return true;
}

bool BatchLab::DataPointSet::scaleValues(double aScaleFactor) { 
  lock();
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setValue(m->value() * aScaleFactor);
    }
  }
  unlock();
  if(fPoints.size()) emitUpdate();
  return true;
}

bool BatchLab::DataPointSet::scaleErrors(double aScaleFactor) { 
  lock();
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setErrorPlus(m->errorPlus() * aScaleFactor);
      m->setErrorMinus(m->errorMinus() * aScaleFactor);
    }
  }
  unlock();
  if(fPoints.size()) emitUpdate();
  return true;
}

bool BatchLab::DataPointSet::setCoordinate(int aCoordinate,const std::vector<double>& aValues,const std::vector<double>& aErrors){
  lock();
  bool stat = false;
  unsigned int number = aValues.size();
  if(   (aCoordinate>=0) 
     && (aCoordinate<fDimension) 
     && number 
     && (aErrors.size()==number) ){
    if(fPoints.empty()){
      for(unsigned int index=0;index<number;index++) addPoint();
    }
    if(fPoints.size()==number) {
      std::vector<DataPoint>::const_iterator it;
      unsigned int index = 0;
      for(it=fPoints.begin();it!=fPoints.end();++it) {
        AIDA::IMeasurement* m = 
          (AIDA::IMeasurement*)(*it).coordinate(aCoordinate);
        m->setValue(aValues[index]);
        m->setErrorPlus(aErrors[index]);
        m->setErrorMinus(aErrors[index]);
        index++;
      }
      stat = true;
    }
  }
  unlock();
  if(stat) emitUpdate();
  return stat;
}

bool BatchLab::DataPointSet::setCoordinate(int aCoordinate,const std::vector<double>& aValues,const std::vector<double>& aErrorsPlus,const std::vector<double>& aErrorsMinus){
  lock();
  bool stat = false;
  unsigned int number = aValues.size();
  if(   (aCoordinate>=0) 
     && (aCoordinate<fDimension) 
     && number 
     && (aErrorsPlus.size()==number) 
     && (aErrorsMinus.size()==number) ){
    if(fPoints.empty()){
      fPoints.resize(number,DataPoint(fDimension));
    }
    if(fPoints.size()==number) {
      std::vector<DataPoint>::const_iterator it;
      unsigned int index = 0;
      for(it=fPoints.begin();it!=fPoints.end();++it) {
        AIDA::IMeasurement* m = 
          (AIDA::IMeasurement*)(*it).coordinate(aCoordinate);
        m->setValue(aValues[index]);
        m->setErrorPlus(aErrorsPlus[index]);
        m->setErrorMinus(aErrorsMinus[index]);
        index++;
      }
      stat = true;
    }
  }
  unlock();
  if(stat) emitUpdate();
  return stat;
}

bool BatchLab::DataPointSet::setPoints(unsigned int aNumber,const double* aData){
  // Expected in aData :
  //   aData[0 + 3 * icoord + 3 * dimension * ipoint] = value
  //   aData[1 + 3 * icoord + 3 * dimension * ipoint] = error minus
  //   aData[2 + 3 * icoord + 3 * dimension * ipoint] = error plus
  // with :
  //   icoord in [0,dimension[
  //   ipoint in [0,aNumber[
  lock();
  bool stat = false;
  if(aNumber) {
    if(fPoints.empty()) fPoints.resize(aNumber,DataPoint(fDimension));
    if(fPoints.size()==aNumber) {
      std::vector<DataPoint>::const_iterator it;
      double* data = (double*)aData;
      for(it=fPoints.begin();it!=fPoints.end();++it) {
        for(int icoord=0;icoord<fDimension;icoord++) {
          AIDA::IMeasurement* m = 
            (AIDA::IMeasurement*)(*it).coordinate(icoord);
          m->setValue(*data);data++;
          m->setErrorMinus(*data);data++;
          m->setErrorPlus(*data);data++;
        }
      }
      stat = true;
    }
  }
  unlock();
  if(stat) emitUpdate();
  return stat;
}

bool BatchLab::DataPointSet::read(const Slash::Store::IArray::Orders& aOrders,char*& aPos,char* aEnd){
  lock();
  bool stat = false;
  unsigned int array_sz = 1;
  unsigned int dimn = aOrders.size();
  for(unsigned int dimi=0;dimi<dimn;dimi++) array_sz *= aOrders[dimi];
  array_sz *= LIB_SIZE_DOUBLE;
  if((aPos+array_sz)>aEnd) {
    fDimension = 0;
    fPoints.clear();
  } else {
    fDimension = aOrders[1];
    unsigned int number = aOrders[0];
    fPoints.resize(number,DataPoint(fDimension));
    if(array_sz) {
      double* data = (double*)aPos;
      std::vector<DataPoint>::const_iterator it;
      for(it=fPoints.begin();it!=fPoints.end();++it) {
        for(int icoord=0;icoord<fDimension;icoord++) {
          AIDA::IMeasurement* m = 
            (AIDA::IMeasurement*)(*it).coordinate(icoord);
          m->setValue(*data);data++;
          m->setErrorMinus(*data);data++;
          m->setErrorPlus(*data);data++;
        }
      }
      aPos += array_sz;
    }
    stat = true;
  }
  unlock();
  return stat;
}

//////////////////////////////////////////////////////////////////////////////
/// lock/unlocked outs ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Slash::Store::IArray::Orders BatchLab::DataPointSet::orders() const {
  lock();
  std::vector<unsigned int> dims(3);
  dims[0] = fPoints.size();
  dims[1] = fDimension;
  dims[2] = 3;
  unlock();
  return dims;
}

std::string BatchLab::DataPointSet::title() const {
  lock();
  std::string s = fTitle;
  unlock();
  return s;
}

int BatchLab::DataPointSet::size() const { 
  lock();
  int n = fPoints.size();
  unlock();
  return n;
}

AIDA::IDataPoint* BatchLab::DataPointSet::point(int aIndex) { 
  lock();
  AIDA::IDataPoint* p = 0;
  if((aIndex>=0) && (aIndex<(int)fPoints.size()) ) {
    p = &(fPoints[aIndex]);
  }
  unlock();
  return p;
}

const AIDA::IDataPoint* BatchLab::DataPointSet::point(int aIndex) const { 
  lock();
  const AIDA::IDataPoint* p = 0;
  if((aIndex>=0) && (aIndex<(int)fPoints.size()) ) {
    p = &(fPoints[aIndex]);
  }
  unlock();
  return p;
}

double BatchLab::DataPointSet::lowerExtent(int aCoordinate) const { 
  lock();
  double value = 0;//FIXME nan.
  if((aCoordinate>=0) && (aCoordinate<fDimension) && (fPoints.size()>0) ){
    std::vector<DataPoint>::const_iterator it = fPoints.begin();
    const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
    value = m->value();
    ++it;
    for(;it!=fPoints.end();++it) {
      const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
      value = MINIMUM(value,m->value());
    }
  }
  unlock();
  return value;
}

double BatchLab::DataPointSet::upperExtent(int aCoordinate) const { 
  lock();
  double value = 0;//FIXME nan.
  if((aCoordinate>=0) && (aCoordinate<fDimension) && (fPoints.size()>0) ){
    std::vector<DataPoint>::const_iterator it = fPoints.begin();
    const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
    value = m->value();
    ++it;
    for(;it!=fPoints.end();++it) {
      const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
      value = MAXIMUM(value,m->value());
    }
  }
  unlock();
  return value;
}

bool BatchLab::DataPointSet::write(char*& aPos,char* aEnd) const{
  lock();
  bool stat = false;
  unsigned int number = fPoints.size();
  unsigned int dim = fDimension;
  unsigned int array_sz = number*dim*3*LIB_SIZE_DOUBLE;
  if((aPos+array_sz)>aEnd) {
  } else {
    if(array_sz) {
      double* data = (double*)aPos;
      for(unsigned int index=0;index<number;index++) {
        const DataPoint& point = fPoints[index];
        for(unsigned int icoord=0;icoord<dim;icoord++) {
          const AIDA::IMeasurement* measurement = point.coordinate(icoord);
          *data = measurement->value();data++;
          *data = measurement->errorMinus();data++;
          *data = measurement->errorPlus();data++;
        }
      }
      aPos += array_sz;
    }
    stat = true;
  }
  unlock();
  return stat;
}
