// this :
#include <BatchLab/Core/Folder.h>

#include <Slash/Store/IObjectIterator.h>
#include <Slash/Store/IObject.h>
#include <Slash/Store/ITree.h>
#include <Slash/Store/IFolderIterator.h>
#include <BatchLab/Core/LinkFolder.h>

#include <Lib/Debug.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::Folder::Folder(
 Slash::Core::IWriter& aPrinter
,Slash::Store::IFolder* aParent
,const std::string& aName
,Slash::Store::ITree& aTree
)
:BaseFolder(aPrinter,aParent,aName,aTree)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Folder::~Folder(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Folder::clear();
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Folder::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::Store::IFolder)
  else if_Lib_SCast(BatchLab::BaseFolder)
  else if_Lib_SCast(BatchLab::Folder)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Folder::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  while(!fObjects.empty()) {
    Slash::Store::IObject* baseObject = *(fObjects.begin());
    // In case the clear is called from ~Folder.
    baseObject->setFolder(0);
    remove(baseObject);
    delete baseObject;
  }

  std::list<Slash::Store::IFolder*>::iterator it;
  // Delete sub folders :
  for(it=fFolders.begin();it!=fFolders.end();it=fFolders.erase(it)) {
    Slash::Store::IFolder* folder = *it;
    BatchLab::LinkFolder* link = Lib_Cast(*folder,BatchLab::LinkFolder);
    if(link) link->setLinkedFolder(0);
    delete *it;
  }
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Folder::add(
 Slash::Store::IObject* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fObjects.push_back(aObject);
  fTree.emitUpdate(); //Notify the UI.
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Folder::remove(
 Slash::Store::IObject* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fObjects.remove(aObject);
  fTree.emitUpdate(); //Notify the UI.
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Folder::add(
 Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFolders.push_back(aFolder);
  fTree.emitUpdate(); //Notify the UI.
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Folder::remove(
 Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFolders.remove(aFolder);
  fTree.emitUpdate(); //Notify the UI.
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Folder::link(
 Slash::Store::IFolder* aFolder
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : check that aFolder is not a parent of this.
  if(aFolder==this) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Folder::link :" 
        << " can't link a folder on itself."
        << Lib::endl;
    return false;
  }
  if(aFolder->linkingFolder()) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Folder::link :" 
        << " can't link an already linked folder."
        << Lib::endl;
    return false;
  }
  if(findFolder(aName)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Folder::link :" 
        << " a folder with name " << Lib::sout(aName)
        << " already exists."
        << Lib::endl;
    return false;
  }
  //FIXME : have to install a notification so that
  //  deleting aFolder will notify the mouting point and "this".
  fFolders.push_back(new LinkFolder(fPrinter,this,aName,fTree,aFolder));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Folder::unlink(
 Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BatchLab::LinkFolder* link = Lib_Cast(*aFolder,BatchLab::LinkFolder);
  if(!link) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Folder::unlink :" 
        << " folder " << Lib::sout(aFolder->path()) 
        << " is not a link."
        << Lib::endl;
    return false;
  }
  fFolders.remove(link);
  delete link;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IFolder* BatchLab::Folder::mkdir(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName.empty()) return 0;
  if(findFolder(aName)) return 0;
  if(findObject(aName)) return 0;
  Slash::Store::IFolder* folder = new Folder(fPrinter,this,aName,fTree);
  fFolders.push_back(folder);
  return folder;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
namespace BatchLab {
class Folder_Objects_Iterator : public virtual Slash::Store::IObjectIterator {
public: //Slash::Data::Slash::Store::IObjectIterator
  virtual Slash::Store::IObject* object() {
    if(fIterator==fList.end()) return 0;
    return *fIterator;
  }
  virtual void next() { ++fIterator; }
public:
  Folder_Objects_Iterator(const std::list<Slash::Store::IObject*>& aList)
    :fList(aList) {
    fIterator = fList.begin();
    Lib::Debug::increment("BatchLab::Folder_Objects_Iterator");
  }
  virtual ~Folder_Objects_Iterator() {
    Lib::Debug::decrement("BatchLab::Folder_Objects_Iterator");
  }
private:
  const std::list<Slash::Store::IObject*>& fList;
  std::list<Slash::Store::IObject*>::const_iterator fIterator;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObjectIterator* BatchLab::Folder::objects(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new Folder_Objects_Iterator(fObjects);
}
namespace BatchLab {
class Folder_Folders_Iterator : public virtual Slash::Store::IFolderIterator {
public: //Slash::Data::Slash::Store::IObjectIterator
  virtual Slash::Store::IFolder* folder() {
    if(fIterator==fList.end()) return 0;
    return *fIterator;
  }
  virtual void next() { ++fIterator; }
public:
  Folder_Folders_Iterator(const std::list<Slash::Store::IFolder*>& aList):fList(aList) {
    fIterator = fList.begin();
    Lib::Debug::increment("BatchLab::Folder_Folders_Iterator");
  }
  virtual ~Folder_Folders_Iterator() {
    Lib::Debug::decrement("BatchLab::Folder_Folders_Iterator");
  }
private:
  const std::list<Slash::Store::IFolder*>& fList;
  std::list<Slash::Store::IFolder*>::const_iterator fIterator;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IFolderIterator* BatchLab::Folder::folders(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new Folder_Folders_Iterator(fFolders);
}
