// this :
#include <BatchLab/Core/Main.h>

#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>
#include <Lib/System.h>
#include <Lib/smanip.h>
#include <Lib/vmanip.h>
#include <Lib/Cast.h>

bool belong(const std::vector<std::string>& aArgs,const std::string& aWhat) {
  std::string senv;
  if(Lib::System::getenv("OSC_ARGS",senv)) {
    std::vector<std::string> args(aArgs);
    Lib::append<std::string>(args,Lib::smanip::words(senv," "));
    return Lib::smanip::belong(args,aWhat);
  } else {
    return Lib::smanip::belong(aArgs,aWhat);
  }
}

BatchLab::Main::Main(const std::vector<std::string>& aArgs)
: Lib::Session(*((Lib::Printer*)this),
               belong(aArgs,"-balance"),
               belong(aArgs,"-debug_check_class")) //acrobatic !
, AnalysisFactory(*((Lib::Session*)this)) //acrobatic !
{
  Lib::Debug::increment("BatchLab::Main");

  Lib::Arguments args(aArgs);
 {std::string senv;
  if(Lib::System::getenv("OSC_ARGS",senv)) {
    args.add(Lib::smanip::words(senv," "));
  }}

  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose.empty()) {
      Lib::Session::setVerboseLevel(1);
    }
  }

  if(fVerboseLevel) {    
    Lib::Out out(*this);
    out << "BatchLab::Main::Main :"
        << " " << this 
        << Lib::endl;
  }

  Lib::Session::setArguments(args.args());

  // declare ourself in the Session.
  // WARNING : we are a Session and a Manager !
  addManager(this);
}

BatchLab::Main::~Main() {
  if(fVerboseLevel) {
    Lib::Out out(*this);
    out << "BatchLab::Main::~Main :"
        << " " << this 
        << Lib::endl;
  }

  // take care of ourself being a manager :
  Lib::Session::removeManager(this);

  Lib::Debug::decrement("BatchLab::Main");
}

//private:
BatchLab::Main::Main(const Main& aFrom)
: Lib::Session(const_cast<Main*>(&aFrom)->printer())
, AnalysisFactory(*static_cast<Lib::Session*>( const_cast<Main*>(&aFrom)  ))
{}

BatchLab::Main& BatchLab::Main::operator=(const Main&){return *this;}

void* BatchLab::Main::cast(const std::string& aClass) const {
  // inheritance :
  if_Lib_SCast(Lib::Printer)
  else if(void* p = Lib::Session::cast(aClass)) return p;
  else if(void* p = BatchLab::AnalysisFactory::cast(aClass)) return p;
  // this :
  else if_Lib_SCast(BatchLab::Main)
  else return 0;
}
