
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/Cloner.h>
#include <Lib/System.h>
#include <Lib/calc.h>

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::Arguments args(aArgc,aArgv);

  if(args.isAnArgument("-h")) {
    out << "Lib_calc :" << Lib::endl;
    out << "Usage :" << Lib::endl;
    out << "  OS> Lib_calc -h" << Lib::endl;
    out << "  OS> Lib_calc -formula=<formula> [variable values]" << Lib::endl;
    out << "For example :" << Lib::endl;
    out << " UNIX> Lib_calc '-formula=sqrt(2*x+2*y)' 2 3" << Lib::endl;
    return Lib::System::exitSuccess();
  }

  std::string formula;
  if(!args.find("-formula",formula)) {
    out << "Lib_calc : -formula argument missing." << Lib::endl;
    return Lib::System::exitFailure();
  }

  args.removeFirst();
  args.remove("-formula");
  std::vector<std::string> vals = args.tovector();
  std::string value;

  if(!Lib::calc(printer,formula,vals,value)) {
    out << "Lib_calc : failed." << Lib::endl;
    return Lib::System::exitFailure();
  }

  out << value << Lib::endl;

  return Lib::System::exitSuccess();
}

// embed code :
#include <Printer.cxx>
#include <Out.cxx>
#include <sout.cxx>
#include <Arguments.cxx>
#include <smanip.cxx>
#include <cmanip.cxx>
#include <System.cxx>
#include <Debug.cxx>

#include <Functions.cxx>
#include <SpaceAlgebra.cxx>
#include <A3s.cxx>
#include <mmanip.cxx>
#include <nmanip.cxx>
#include <Value.cxx>
#include <valuemanip.cxx>
#include <ExpFunc.cxx>
#include <Expression.cxx>
#include <Processor.cxx>
//Do the below to avoid to have YYSTYPE doubly defined.
#define Lib_ProcessorTokens_h
#include <ProcessorLexer.cxx>

#include <calc.cxx>
