
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>

#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::Arguments args(aArgc,aArgv);

  std::string cmd = 
    "  OS> Lib_cvs_co <file with packages>";

  std::string sfile;
  std::string s;
  if(!args.last(sfile,s)||(s.size())) {
    out << cmd << Lib::endl;
    return EXIT_FAILURE;
  }

  std::vector<std::string> text;
  if(!Lib::fmanip::read(sfile,text)) {
    out << "Can't read \"" << sfile << "\"" << Lib::endl;
    return false;
  }

  for(unsigned int index=0;index<text.size();index++) {
    std::string line = text[index];
    std::vector<std::string> words;
    Lib::smanip::words(line," ",words);
    if( (words.size()!=4) && (words.size()!=5) ) {
      out << "Bad word number in \"" << line << "\"" << Lib::endl;
      return EXIT_FAILURE;
    }
    std::string package = words[1];
    std::string version = words[2];
    std::string group = words.size()==5 ? words[3] : std::string("");

    out << "Get \"" << package 
        << "\" version \"" << version 
        << "\" in \"" << group << "\"." << Lib::endl;

    std::string cocmd = "cvs co -d " + version + " -r " + version + " ";
    if(group.empty()) cocmd += package;
    else cocmd += group + "/" + package;

    std::string currentDirectory;
    Lib::dirmanip::directory(currentDirectory);

    if(group.size()) {
      if(!Lib::dirmanip::go(group)) {
        out << "Can't do in  \"" << group << "\"" << Lib::endl;
        return EXIT_FAILURE;
      }
    }

    if(!Lib::dirmanip::go(package)) {
      out << "Can't do in  \"" << package << "\"" << Lib::endl;
      return EXIT_FAILURE;
    }

    if(Lib::dirmanip::isAnEntry(".",version)) {
      out << "Version \"" << version 
          << "\" for package \"" << package 
          << "\" already exists." << Lib::endl;
    } else {
      out << "system : \"" << cocmd  << "\"." << Lib::endl;
      ::system((char*)cocmd.c_str());
    }

    if(!Lib::dirmanip::go(currentDirectory)) {
      out << "Can't do in  \"" << currentDirectory << "\"" << Lib::endl;
      return EXIT_FAILURE;
    }

  }

  return EXIT_SUCCESS;
}


