
#include <Slash/Tools/args.h>
#include <Slash/Tools/directory.h>

#include <Lib/Debug.h>
#include <Lib/Upgrader.h>

#include <cstdlib>
#include <iostream>

int main (int aArgc,char** aArgv){
  int verboseLevel = 0;

 {// balance block :

  Slash::args args(aArgc,aArgv);
  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose.empty()) verboseLevel = 1;
  }

  std::vector<std::string> options;
  options.push_back("-h");
  options.push_back("-verbose");
  options.push_back("-orders");
  options.push_back("-filter");

  if(!args.known_options(options)) {
    std::cout << "Lib_upgrade :"
        << " Some options are unknown." << std::endl;
    std::cout << "To have help, type :" << std::endl;
    std::cout << " OS> Lib_upgrade -h" << std::endl;
    return EXIT_SUCCESS;
  }

  if(args.is_arg("-h")) {
    std::cout << "Lib_upgrade :" << std::endl;
    std::cout << "Usage :" << std::endl;
    std::cout << " OS> Lib_upgrade <options> -orders=<file> <path>" << std::endl;
    std::cout << "Options :" << std::endl;
    std::cout << " -verbose" << std::endl;
    std::cout << " -filter=<string>" << std::endl;
    return EXIT_SUCCESS;
  }

  std::string orders;
  if(!args.find("-orders",orders)) {
    std::cout << "Lib_upgrade :"
        << " no order file given. Take $HOME/Lib.upgrade." 
        << std::endl;
    std::string home = Slash::directory::home();
    orders = home+ "/Lib.upgrade";
  }

  std::string filter;
  args.find("-filter",filter);

  std::string path;
  if(!args.file(path)) path = ".";

  Lib::Upgrader upgrader(std::cout,verboseLevel,orders,filter);
  upgrader.upgrade(path);

  } //end balance block.

  Lib::Debug::balance(std::cout);

  if(verboseLevel) {
    std::cout << "Lib_upgrade :"
              << " Exiting..." << std::endl;
  }

  return EXIT_SUCCESS;
}

// embed code :
#include <Upgrader.cxx>
#include <Debug.cxx>

#include <Printer.cxx>
#include <Out.cxx>
#include <System.cxx>
#include <smanip.cxx>
#include <sout.cxx>
#include <cmanip.cxx>
#include <fmanip.cxx>
#include <dirmanip.cxx>
