#ifndef Lib_BaseSocket_h
#define Lib_BaseSocket_h

#include <string>
#include <vector>
#include <ostream>

#include <Slash/Core/typedefs.h>

namespace Lib {

class BaseSocket { 
public:
  virtual bool accept() = 0;
  virtual bool connect() = 0;
  virtual void disconnect() = 0;
public:        
  BaseSocket(std::ostream&,int = 0);  
  BaseSocket(const BaseSocket&);  
  virtual ~BaseSocket();
  BaseSocket& operator=(const BaseSocket&);  
public:        
  bool isConnected() const;
  int socket() const;
public:
  bool send_buffer(const char*,Slash::uint64); 
  bool wait_buffer(char*,Slash::uint64); 
  bool send_uint64(Slash::uint64); 
  bool wait_uint64(Slash::uint64&); 
  bool send_string(const std::string&); 
  bool wait_string(std::string&); 
  bool send_text(const std::vector<std::string>&,bool=false); 
  bool wait_text(std::vector<std::string>&,bool=false); 
  bool send_file(const std::string&); 
  bool wait_file(const std::string&); 
  bool is_there_input(bool&);

  //avoid long, void* here.
  bool send_uchar(unsigned char); 
  bool wait_uchar(unsigned char&); 
  bool send_ushort(unsigned short); 
  bool wait_ushort(unsigned short&); 
  bool send_short(short); 
  bool wait_short(short&); 
  bool send_uint(unsigned int); 
  bool wait_uint(unsigned int&); 
  bool send_int(int); 
  bool wait_int(int&); 
  bool send_float(float); 
  bool wait_float(float&); 
  bool send_bool(bool); 
  bool wait_bool(bool&); 
protected:        
  void closeSocket();
protected:        
  std::ostream& m_out;
  int fVerboseLevel;
  int fSocket;
  bool fIsLE;
};

}

#endif

