// this :
#include <Lib/AccessorManager.h>

#include <Slash/Data/IIterator.h>
#include <Slash/Data/IVisualizer.h>
#include <Slash/Core/IValue.h>
#include <Slash/Data/IHandlersIterator.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/Property.h>
#include <Lib/Value.h>
#include <Lib/Handler.h>
#include <Lib/Processor.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>
#include <Lib/Debug.h>
#include <Lib/valuemanip.h>

// STL :
#include <algorithm>

#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::AccessorManager::AccessorManager(
 Slash::Core::IWriter& aPrinter
)
:fPrinter(aPrinter)
,fName("AccessorManager")
,fVerboseLevel(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::AccessorManager");
}
//////////////////////////////////////////////////////////////////////////////
Lib::AccessorManager::~AccessorManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  eraseHandlers();
 {std::list<Slash::Data::IAccessor*>::iterator it;
  for(it=fAccessors.begin();it!=fAccessors.end();it=fAccessors.erase(it)) {
    if(fVerboseLevel) {
      Lib::Out out(fPrinter);
      out << "Lib::AccessorManager::~AccessorManager :"
          << " delete IAccessor " << Lib::sout((*it)->name()) << "." 
          << Lib::endl;
    }
    delete *it;
  }
 }
 {std::list<Slash::Core::IAction*>::iterator it;
  for(it=fActions.begin();it!=fActions.end();
      it=fActions.erase(it)) delete *it;}
  Lib::Debug::decrement("Lib::AccessorManager");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int Lib::AccessorManager::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::AccessorManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::AccessorManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::Data::IProcessor)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::AccessorManager::printer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::addAccessor(
 Slash::Data::IAccessor* aType
)
//////////////////////////////////////////////////////////////////////////////
// Will be deleted by the AccessorManager destructor.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aType) return;
  fAccessors.push_back(aType);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::removeAccessor(
 Slash::Data::IAccessor* aType
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aType) return;
  fAccessors.remove(aType);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::removeAccessor(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IAccessor* type = findAccessor(aName);
  if(!type) return;
  fAccessors.remove(type);
  delete type;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IAccessor* Lib::AccessorManager::findAccessor(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Data::IAccessor*>::iterator it;
  for(it=fAccessors.begin();it!=fAccessors.end();++it) {
    if(aName==(*it)->name()) return *it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<Slash::Data::IAccessor*> Lib::AccessorManager::accessors(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Data::IAccessor*> v(fAccessors.size());
  std::list<Slash::Data::IAccessor*>::const_iterator it;
  unsigned int index = 0;
  for(it=fAccessors.begin();it!=fAccessors.end();++it,index++) {
    v[index] = *it;
  }
  return v;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::AccessorManager::accessorNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vs;
  std::list<Slash::Data::IAccessor*>::const_iterator it;
  for(it=fAccessors.begin();it!=fAccessors.end();++it) {
    vs.push_back((*it)->name());
  }
  return vs;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::AccessorManager::actionNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vs;
  std::list<Slash::Core::IAction*>::const_iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    vs.push_back((*it)->name());
  }
  return vs;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::addHandler(
 Slash::Data::IAccessor* aType
,Slash::Data::IAccessor::Data aData
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHandlers.push_back(Handler(aType,aData,aTag));
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::AccessorManager::numberOfHandlers(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHandlers.size();
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::addOutHandler(
 Slash::Data::IAccessor* aType
,Slash::Data::IAccessor::Data aData
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fOutHandlers.push_back(Handler(aType,aData,aTag));
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::eraseHandlers(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHandlers.clear();
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::eraseOutHandlers(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fOutHandlers.clear();
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::swapHandlers(
)
//////////////////////////////////////////////////////////////////////////////
// Copy fOutHandlers into fHandlers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHandlers = fOutHandlers;
  fOutHandlers.clear();
}
namespace Lib {
class HandlersIterator : public virtual Slash::Data::IHandlersIterator {
public: //IHandlersIteractor
  virtual const Slash::Data::IHandler* handler() const {
    if(fIterator==fList.end()) return 0;
    return &(*fIterator);
  }
  virtual void next() { ++fIterator;}
public:
  HandlersIterator(const std::list<Handler>& aList)
  :fList(aList),fIterator(aList.begin()) {
    Lib::Debug::increment("Lib::HandlersIterator");
  }
  virtual ~HandlersIterator() {
    Lib::Debug::decrement("Lib::HandlersIterator");
  }
private:
  const std::list<Handler>& fList;
  std::list<Handler>::const_iterator fIterator;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IHandlersIterator* Lib::AccessorManager::handlersIterator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new HandlersIterator(fHandlers);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IHandlersIterator* Lib::AccessorManager::outHandlersIterator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new HandlersIterator(fOutHandlers);
}
//////////////////////////////////////////////////////////////////////////////
std::vector<Slash::Data::IAccessor*> Lib::AccessorManager::handlersAccessors(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Data::IAccessor*> list;
  Slash::Data::IHandlersIterator* hit = handlersIterator();
  for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
    if(std::find(list.begin(),list.end(),h->type())==list.end())
      list.push_back(h->type());
  }
  delete hit;
  return list;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::AccessorManager::type_id_tag(
 const std::string& aString
,Slash::Data::IAccessor*& a_type
,void*& a_id
,void*& a_tag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> fields;
  smanip::words(aString,"/",fields);
  if(fields.size()<=1) {
    //fAccessorManager.out().println
    //("Filter property \"%s\" :",fProperty.c_str());
    //fAccessorManager.out().println
    //(" \"%s\" : bad name format (should be class/id).",
    //words[index].c_str());
    a_type = 0;
    a_id = 0;
    a_tag = 0;
    return false;
  }
  
  std::string stype = fields[0];
  a_type = findAccessor(stype);
  if(!a_type) {
    Out out(fPrinter);
    out << "Lib::AccessorManager::::type_id_tag :"
        << " accessor " << Lib::sout(stype) << " not found." 
        << endl;
    a_type = 0;
    a_id = 0;
    a_tag = 0;
    return false;
  } 
  
  std::string sid = fields[1];
  if(!smanip::topointer(sid,a_id)) {
    Out out(fPrinter);
    out << "Lib::AccessorManager::::type_id_tag :"
        << " " << sid << " not an id." 
        << endl;
    a_type = 0;
    a_id = 0;
    a_tag = 0;
    return false;
  }

  a_tag = 0;
  if(fields.size()>=3) {
    std::string stag = fields[2];
    if(!smanip::topointer(stag,a_tag)) {
      Out out(fPrinter);
      out << "Lib::AccessorManager::::type_id_tag :"
          << " " << stag << " not a void*." 
          << endl;
      a_type = 0;
      a_id = 0;
      a_tag = 0;
      return false;
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
//// Actions /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::addAction(
 Slash::Core::IAction* aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aAction) return;
  fActions.push_back(aAction);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IAction* Lib::AccessorManager::findAction(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::IAction*>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if(aName==(*it)->name()) return *it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::AccessorManager::removeAction(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IAction* action = findAction(aName);
  if(!action) return;
  fActions.remove(action);
  delete action;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::AccessorManager::execute(
 const std::string& aName
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IAction* action = findAction(aName);
  if(!action) {
    Out out(fPrinter);
    out << "Lib::AccessorManager::execute :" 
        << " Action " << Lib::sout(aName) << " not found." << endl;
    return "";
  }
  return action->execute(aArgs);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::CollectAction::CollectAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("collect")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::CollectAction::~CollectAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::CollectAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::CollectAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::CollectAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::CollectAction::execute(
 const std::vector<std::string>& aArgs
) //FIXME : throw
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<2) return "";
  std::string what = aArgs[0];
  const std::string& cuts = aArgs[1];

  // Cuts
  // Get cut variables :
  Lib::Processor cut(cuts,fAccessorManager.printer());
  std::vector<std::string> cvar;
  if(!cut.findVariables(cvar)) { //FIXME : throw
    Out out(fAccessorManager.printer());
    out << "Lib::CollectAction::execute :" 
        << " in collect " << Lib::sout(what)
        << " can't find variables in cut " << Lib::sout(cuts)
        << endl;
    return "";
  }
  if(!cut.compile(cvar)) { //FIXME : throw
    Out out(fAccessorManager.printer());
    out << "Lib::CollectAction::execute :" 
        << " in collect " << Lib::sout(what)
        << " compilation of cut " << Lib::sout(cuts) << " failed."
        << endl;
    return "";
  }
  unsigned int cvarn = cvar.size();

  if(what=="-u") { // First remove duplicates :  

    fAccessorManager.eraseOutHandlers();
    Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      //FIXME : have a quick search :
      bool found = false;
      Slash::Data::IHandlersIterator* hit2 = 
        fAccessorManager.outHandlersIterator();
      for(;const Slash::Data::IHandler* h2 = hit2->handler();hit2->next()) {
        if(h2->isEqual(*h)) {
          found = true;
          break;
        }
      }
      delete hit2;
      if(!found) {
        fAccessorManager.addOutHandler(h->type(),h->object(),h->tag());
      }
    }
    fAccessorManager.swapHandlers();
    delete hit;
  }

  if((what=="-")||(what=="-u")) {
    
    fAccessorManager.eraseOutHandlers();
    Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      // Get variables values :
      std::vector<Lib::Value> cvals;
      bool status = true;
      for(unsigned int i=0;i<cvarn;i++) {
	Slash::Core::IValue* value = 
          h->type()->findValue(h->object(),cvar[i],h->tag());
        if(!value) {
          Out out(fAccessorManager.printer());
          out << "Lib::CollectAction::execute :" 
              << " Property not found : " << cvar[i] << endl;
          status = false;
          break;
        }
        //FIXME : work directly on the vals[]
        Lib::Value var;
        if(!Lib::valuemanip::set(var,*value)) {
          Out out(fAccessorManager.printer());
          out << "Lib::CollectAction::execute :" 
              << " convertion of IValue " << Lib::sout(value->stype()) 
              << " to a Lib::Value failed."
              << endl;
          status = false;
          delete value;
          break;
        }
        delete value;
        cvals.push_back(var);
      }
      if(!status) {
        fAccessorManager.eraseHandlers();
        Out out(fAccessorManager.printer());
        out << "Lib::CollectAction::execute :" 
            << " Can't init cut." << endl;
        fAccessorManager.eraseOutHandlers();
        break;
      }
      bool takeit;
      std::string serror;
      if(!cut.cut(cvals,takeit,serror)) {
        fAccessorManager.eraseHandlers();
        Out out(fAccessorManager.printer());
        out << "Lib::CollectAction::execute :" 
            << " cut failed."
            << endl;
        out << serror << endl;
        fAccessorManager.eraseOutHandlers();
        break;
      }
      if(takeit==false) continue;

      fAccessorManager.addOutHandler(h->type(),(void*)h->object(),h->tag());
    }
    fAccessorManager.swapHandlers();
    delete hit;

  } else {
    fAccessorManager.eraseHandlers();

    std::vector<std::string> args;
   {std::string name;
    if(!Lib::smanip::nameArgs(what,name,args)) {
      Out out(fAccessorManager.printer());
      out << "Lib::CollectAction::execute :" 
          << " bad syntax in " << Lib::sout(what) << "." << endl;
      return "";
    }
    what = name;}

    Slash::Data::IAccessor* type = fAccessorManager.findAccessor(what);
    if(!type) {
      Out out(fAccessorManager.printer());
      out << "Lib::CollectAction::execute :" 
          << " Unknown type " << Lib::sout(what) << "." << endl;
      return "";
    }
  
    if(fAccessorManager.verboseLevel()) {
      Out out(fAccessorManager.printer());
      out << "Lib::CollectAction::execute :" 
          << " type " << Lib::sout(what) << " found." << endl;
    }
  
    type->setIteratorArguments(args);

    Slash::Data::IIterator* iterator = type->iterator();
    if(!iterator) {
      Out out(fAccessorManager.printer());
      out << "Lib::CollectAction::execute :" 
          << " No iterator found for type " << Lib::sout(what) << "." << endl;
      return "";
    }
  
    if(fAccessorManager.verboseLevel()) {
      Out out(fAccessorManager.printer());
      out << "Lib::CollectAction::execute :" 
          << " iterator found for type " << Lib::sout(what) << "." << endl;
      out << "Lib::CollectAction::execute :"
          << " Cuts " << Lib::sout(cuts) << "." << endl;
    }
  
    for(Slash::Data::IAccessor::Data id = 0;
          (id = iterator->object());iterator->next()) {
      // Get variables values :
      std::vector<Lib::Value> cvals;
      bool status = true;
      for(unsigned int i=0;i<cvarn;i++) {
	Slash::Core::IValue* value = 
          type->findValue(id,cvar[i],iterator->tag());
        if(!value) {
          Out out(fAccessorManager.printer());
          out << "Lib::CollectAction::execute :" 
              << " Property not found : " << cvar[i] << endl;
          status = false;
          break;
        }
        //FIXME : work directly on the vals[]
        Lib::Value var;
        if(!Lib::valuemanip::set(var,*value)) {
          Out out(fAccessorManager.printer());
          out << "Lib::CollectAction::execute :" 
              << " convertion of IValue " << Lib::sout(value->stype()) 
              << " to a Lib::Variable failed."
              << endl;
          status = false;
          delete value;
          break;
        }
        delete value;
        cvals.push_back(var);
      }
      if(!status) {
        fAccessorManager.eraseHandlers();
        Out out(fAccessorManager.printer());
        out << "Lib::CollectAction::execute :" 
            << " Can't init cut." << endl;
        break;
      }
      bool takeit;
      std::string serror;
      if(!cut.cut(cvals,takeit,serror)) {
        fAccessorManager.eraseHandlers();
        Out out(fAccessorManager.printer());
        out << "Lib::CollectAction::execute :" 
            << " cut failed."
            << endl;
        out << serror << endl;
        break;
      }
      if(takeit==false) continue;
      fAccessorManager.addHandler(type,id,iterator->tag());
    }
    delete iterator;
  }

  if(fAccessorManager.verboseLevel()) {
    Out out(fAccessorManager.printer());
    out << "Lib::CollectAction::execute :"
        << " collected " << fAccessorManager.numberOfHandlers() << endl;
  }

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::FilterAction::FilterAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("filter")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::FilterAction::~FilterAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::FilterAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::FilterAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::FilterAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::FilterAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<1) return "";
  const std::string& property = aArgs[0];
 
  std::string filter = (aArgs.size()>=2 ? aArgs[1] : std::string(""));

  fAccessorManager.eraseOutHandlers();

  Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
  for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
    Slash::Core::IValue* value = 
      h->type()->findValue(h->object(),property,h->tag());
    if(!value) {
      Out out(fAccessorManager.printer());
      out << "Lib::FilterAction::execute :"
          << " Property not found : " << property << endl;
      fAccessorManager.eraseOutHandlers();
      break;
    }
    //FIXME : work directly on the value !
    Lib::Value var;
    if(!Lib::valuemanip::set(var,*value)) {
      Out out(fAccessorManager.printer());
      out << "Lib::FilterAction::execute :"
          << " convertion of IValue " << Lib::sout(value->stype()) 
          << " to a Lib::Variable failed."
          << endl;
      fAccessorManager.eraseOutHandlers();
      delete value;
      break;
    }
    delete value;
    std::string ss = Lib::valuemanip::toString(var);
    if(filter.size()) {
      if(!smanip::match(ss,filter)) continue;
    }

    std::vector<std::string> words;
    smanip::words(ss,"\n",words);
    unsigned int wordn = words.size();
    for(unsigned int index=0;index<wordn;index++) {

      Slash::Data::IAccessor* type = 0;
      void* id = 0;
      void* tag = 0;
      if(!fAccessorManager.type_id_tag(words[index],type,id,tag)) {
        fAccessorManager.eraseOutHandlers();
        break;
      }
      
      fAccessorManager.addOutHandler(type,id,tag);

    }
  }
  fAccessorManager.swapHandlers();
  delete hit;  

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::DumpAction::DumpAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("dump")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::DumpAction::~DumpAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::DumpAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::DumpAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::DumpAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::DumpAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string format = aArgs.size()>=1 ? aArgs[0] : std::string("");
  if(format.empty()) format = "table";

  Out out(fAccessorManager.printer());

  std::vector<Slash::Data::IAccessor*> types = 
    fAccessorManager.handlersAccessors();
  std::vector<Slash::Data::IAccessor*>::iterator it;
  for(it=types.begin();it!=types.end();++it) {
    Slash::Data::IAccessor* type = *it;

    unsigned int propn = type->numberOfProperties();

    // Count handlers of given type :
   {int count = 0;
   {Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      if( h->type() == type ) count++;
    }
    delete hit;}

    out << "Lib::DumpAction::execute :" 
        << " " << type->name() 
        << " informations (" << count << " objects) :" << endl;}

    if(format=="table") {
      // Dump header :
      for(unsigned int index=0;index<propn;index++) {
        out << " " << type->property(index)->header();
      }
      out << endl;
    }
    
   {Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      Slash::Data::IAccessor::Data id = h->object();
      if(h->type()==type) {
        if(format=="table") {
          bool stop = false;
          for(unsigned int propi=0;propi<propn;propi++) {
	    Slash::Core::IValue* value = 
              type->findValue(id,type->property(propi)->name(),h->tag());
            if(!value) {
              out << "Lib::DumpAction::execute : "
                  << " Property not found : " 
                  << type->property(propi)->name()
                  << endl;
              stop = true;
              break;
            }
            //FIXME : work directly on the value !
            Lib::Value var;
            if(!Lib::valuemanip::set(var,*value)) {
              out << "Lib::DumpAction::execute : "
                  << " convertion of IValue " << Lib::sout(value->stype()) 
                  << " to a Lib::Variable failed."
                  << endl;
              stop = true;
              delete value;
              break;
            }
            delete value;
            std::string s;
            if(var.type()==Slash::Core::IValue::ARRAY_DOUBLE) {
              s = "vector";
            } else {
              s = Lib::valuemanip::toString(var);
            }
            type->property(propi)->dress(s);
            out << " " << s;
          }
          if(stop) continue;
          out << endl;
        } else {
          bool stop = false;
          for(unsigned int propi=0;propi<propn;propi++) {
	    Slash::Core::IValue* value = 
              type->findValue(id,type->property(propi)->name(),h->tag());
            if(!value) {
              out << "Lib::DumpAction::execute : "
                  << " Property not found : " 
                  << type->property(propi)->name()
                  << endl;
              stop = true;
              break;
            }
            //FIXME : work directly on the value !
            Lib::Value var;
            if(!Lib::valuemanip::set(var,*value)) {
              out << "Lib::DumpAction::execute : "
                  << " convertion of IValue " << Lib::sout(value->stype()) 
                  << " to a Lib::Value failed."
                  << endl;
              stop = true;
              delete value;
              break;
            }
            delete value;
            std::string s = Lib::valuemanip::toString(var);
            out << type->property(propi)->name() << " :" << s << endl;
          }
          if(stop) continue;
          out << endl;
        }
      }
    }
    delete hit;}
  }

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::NumberAction::NumberAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("number")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::NumberAction::~NumberAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::NumberAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::NumberAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::NumberAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::NumberAction::execute(
 const std::vector<std::string>&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = fAccessorManager.numberOfHandlers();
  Out out(fAccessorManager.printer());
  out << "Lib::NumberAction::execute :"
      << " " << number << " objects." << endl;
  std::string sreturn;
  smanip::printf(sreturn,32,"%d",number);
  return sreturn;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::GetValuesAction::GetValuesAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("values")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::GetValuesAction::~GetValuesAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::GetValuesAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::GetValuesAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::GetValuesAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::GetValuesAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<1) return "";
  const std::string& property = aArgs[0];

  std::vector<std::string> values;

  Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
  for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
    Slash::Core::IValue* value = 
      h->type()->findValue(h->object(),property,h->tag());
    if(!value) {
      Out out(fAccessorManager.printer());
      out << "Lib::GetValuesAction::execute :"
          << " Property not found : " << property << endl;
      values.clear();
      break;
    }
    //FIXME : work directly on the value !
    Lib::Value var;
    if(!Lib::valuemanip::set(var,*value)) {
      Out out(fAccessorManager.printer());
      out << "Lib::GetValuesAction::execute :"
          << " convertion of IValue " << Lib::sout(value->stype()) 
          << " to a Lib::Value failed."
          << endl;
      values.clear();
      delete value;
      break;
    }
    delete value;
    values.push_back(Lib::valuemanip::toString(var));
  }
  delete hit;
  return smanip::tostring(values,"\n");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::DestroyAction::DestroyAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("destroy")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::DestroyAction::~DestroyAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::DestroyAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::DestroyAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::DestroyAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::DestroyAction::execute(
 const std::vector<std::string>&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
  for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
    h->type()->destroy(h->object(),h->tag());
  }
  delete hit;
  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::SetAction::SetAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("set")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::SetAction::~SetAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::SetAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::SetAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::SetAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::SetAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<2) return "";
  const std::string& what = aArgs[0];
  const std::string& value = aArgs[1];

  Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
  for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
    h->type()->set(h->object(),what,value,h->tag());
  }
  delete hit;

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::CalculateAction::CalculateAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("calculate")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::CalculateAction::~CalculateAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::CalculateAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::CalculateAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::CalculateAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::CalculateAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<2) return "";

  int dimension = aArgs.size()-1;
  std::vector<Lib::Processor> processors;
  std::vector< std::vector<std::string> > variables(dimension);
  bool status = true;
 {for(int dim=0;dim<dimension;dim++) {
    processors.push_back
      (Lib::Processor(aArgs[dim+1],fAccessorManager.printer()));
    if(!processors[dim].findVariables(variables[dim])) {
      Out out(fAccessorManager.printer());
      out << "Lib::CalculateAction::execute :"
          << " can't find variables in " << Lib::sout(aArgs[dim+1]) << endl;
      status = false;
    }
    if(variables[dim].size()==0) {
      Out out(fAccessorManager.printer());
      out << "Lib::CalculateAction::execute :"
          << " No variables found in " << aArgs[dim+1] << endl;
      status = false;
    } else {
      if(!processors[dim].compile(variables[dim])) {
        Out out(fAccessorManager.printer());
        out << "Lib::CalculateAction::execute :"
            << " compilation of " << Lib::sout(aArgs[dim+1]) 
            << " failed." << endl;
        status = false;
      }
    }
  }}

  if(status) {
    Slash::Data::IHandlersIterator* hit = fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      std::vector<Slash::Core::IValue*> values;
      bool stop = false;
      for(int dim=0;dim<dimension;dim++) {
        unsigned int xn = variables[dim].size();
        std::vector<Lib::Value> vals(xn);
        for(unsigned int xi=0;xi<xn;xi++) {
	  Slash::Core::IValue* value = 
            h->type()->findValue(h->object(),variables[dim][xi],h->tag());
          if(!value) {
            Out out(fAccessorManager.printer());
            out << "Lib::CalculateAction::execute :"
                << " Property " << Lib::sout(variables[dim][xi])
                << " not found." << endl;
            stop = true;
            break;
          }
          if(!Lib::valuemanip::set(vals[xi],*value)) { 
            Out out(fAccessorManager.printer());
            out << "Lib::CalculateAction::execute :"
                << " convertion of IValue " << Lib::sout(value->stype()) 
                << " to a Lib::Variable failed."
                << endl;
            stop = true;
            delete value;
            break;
          }
          delete value;
        }
        if(stop) break;

        Lib::Value var;
        std::string serror;
        if(!processors[dim].execute(vals,var,serror)) {
          Out out(fAccessorManager.printer());
          out << "Lib::CalculateAction::execute :"
              << " execution failed"
              << endl;
          out << serror << endl;
          stop = true;
          break;
        }
        values.push_back(new Lib::Value(var));
      }
      if(!stop) {
        h->type()->calculate(h->object(),aArgs[0],values,h->tag());
      }
     {std::vector<Slash::Core::IValue*>::iterator it;
      for(it=values.begin();it!=values.end();++it) delete (*it);}
    }
    delete hit;
  }

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::VisualizeAction::VisualizeAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("visualize")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::VisualizeAction::~VisualizeAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::VisualizeAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::VisualizeAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::VisualizeAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::VisualizeAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Data::IAccessor*> types = 
    fAccessorManager.handlersAccessors();
  std::vector<Slash::Data::IAccessor*>::iterator it;
  for(it=types.begin();it!=types.end();++it) {
    Slash::Data::IAccessor* type = *it;
    Slash::Data::IVisualizer* visType = 
      Lib_Cast(*type,Slash::Data::IVisualizer);
    if(!visType) {
      Out out(fAccessorManager.printer());
      out << "Lib::AccessorManager::execute :"
          << " type " << Lib::sout(type->name()) 
          << " is not a Slash::Data::IVisualizer." 
          << Lib::endl;
      continue;
    }

    type->setExecuteArguments(aArgs);

    visType->beginVisualize();

    Slash::Data::IHandlersIterator* hit = 
      fAccessorManager.handlersIterator();
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      if( h->type() == type ) {
        visType->visualize(h->object(),h->tag());
      }
    }
    visType->endVisualize();

    delete hit;
  }
  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::InitAction::InitAction(
 Slash::Data::IProcessor& aAccessorManager
)
:fAccessorManager(aAccessorManager)
,fName("init")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::InitAction::~InitAction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::InitAction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::InitAction)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::InitAction::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::InitAction::execute(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //used in OnX::NetUI::steer ONX_PROTOCOL_DATA

  if(aArgs.size()<1) return "";
  const std::string& values = aArgs[0];

  fAccessorManager.eraseHandlers();

  std::vector<std::string> words = Lib::smanip::words(values,"\n");
  for(std::vector<std::string>::iterator it = words.begin();
    it!=words.end();++it) {

    Slash::Data::IAccessor* type = 0;
    void* id = 0;
    void* tag = 0;
    if(!fAccessorManager.type_id_tag(*it,type,id,tag)) {
      fAccessorManager.eraseHandlers();
      break;
    }

    fAccessorManager.addHandler(type,id,tag);
  }

  return "";
}
