
// this :
#include <Lib/Arguments.h>

#include <Lib/smanip.h>

#include <string.h> //strcpy

Lib::Arguments::Arguments(){}
Lib::Arguments::Arguments(int aArgc,char** aArgv){
  std::vector<std::string> args;
  for(int index=0;index<aArgc;index++) args.push_back(aArgv[index]);
  add(args);
}

Lib::Arguments::Arguments(const std::vector<std::string>& aArgs){add(aArgs);}

Lib::Arguments::Arguments(const std::string& aArgs,const std::string& aSep,bool aStrip){
  std::vector<std::string> args;
  smanip::words(aArgs,aSep,args);  
  add(args,aStrip);
}

Lib::Arguments::Arguments(const std::vector<Argument>& aArgs):fArgs(aArgs){}

Lib::Arguments::Arguments(const Arguments& aFrom):fArgs(aFrom.fArgs){}

Lib::Arguments::~Arguments(){}

unsigned int Lib::Arguments::number() const {return fArgs.size();}

const std::vector<Lib::Arguments::Argument>& Lib::Arguments::args() const {return fArgs;}

bool Lib::Arguments::isAnArgument(const std::string& aString) const {
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    if((*it).first==aString) return true;
  }
  return false;
}

bool Lib::Arguments::add(const std::string& aKey,const std::string& aValue,bool a_override /*true*/){
  if(a_override) {
    for(std::vector<Argument>::iterator it = fArgs.begin();
        it!=fArgs.end();++it) {
      if((*it).first==aKey) {
        (*it).second = aValue;
        return true;
      }
    }
  }
  if(aKey.empty()) return false;
  fArgs.push_back(Argument(aKey,aValue)); 
  return true;
}

int Lib::Arguments::remove(const std::string& aKey){
  unsigned int nbeg = fArgs.size();
  for(std::vector<Argument>::iterator it = fArgs.begin();it!=fArgs.end();) {
    if(aKey==(*it).first) {
      it = fArgs.erase(it);
    } else {
      ++it;
    }
  }
  return nbeg - fArgs.size();
}

void Lib::Arguments::removeFirst(){fArgs.erase(fArgs.begin());}

std::string Lib::Arguments::tostring(const std::string& aSeparator) const {return Lib::smanip::tostring(tovector()," ");}

std::vector<std::string> Lib::Arguments::tovector() const {
  // Return a vector of string <name=value>
  std::vector<std::string> vec;
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    std::string s;
    if((*it).second.empty()) {
      s = (*it).first;
    } else {
      s = (*it).first;
      s += "=";
      s += (*it).second;
    }
    vec.push_back(s);
  }
  return vec;
}

bool Lib::Arguments::find(const std::string& aString,std::string& aValue) const {
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    if((*it).first==aString) {
      aValue = (*it).second;
      return true;
    }
  }
  aValue = "";
  return false;
}

bool Lib::Arguments::find(const std::string& aArg,bool& aValue) const {
  std::string svalue;
  if(!find(aArg,svalue)) {
    aValue = false;
    return false;
  }
  return Lib::smanip::tobool(svalue,aValue);
}

bool Lib::Arguments::find(const std::string& aArg,int& aValue) const {
  std::string svalue;
  if(!find(aArg,svalue)) {
    aValue = 0;
    return false;
  }
  return Lib::smanip::toint(svalue,aValue);
}

bool Lib::Arguments::find(const std::string& aArg,unsigned int& aValue) const {
  std::string svalue;
  if(!find(aArg,svalue)) {
    aValue = 0;
    return false;
  }
  return Lib::smanip::touint(svalue,aValue);
}

bool Lib::Arguments::find(const std::string& aArg,double& aValue) const {
  std::string svalue;
  if(!find(aArg,svalue)) {
    aValue = 0;
    return false;
  }
  return Lib::smanip::todouble(svalue,aValue);
}

std::vector<std::string> Lib::Arguments::find(const std::string& aKey) const {
  std::vector<std::string> vals;
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    if((*it).first==aKey) vals.push_back((*it).second);
  }
  return vals;
}

bool Lib::Arguments::match(const std::string& aPattern,std::string& aKey,std::string& aValue) const {
  // aString may contains wildcards.
  aKey = "";
  aValue = "";
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    if(Lib::smanip::match((*it).first,aPattern)) {
      aKey = (*it).first;
      aValue = (*it).second;
      return true;
    }
  }
  return false;
}

bool Lib::Arguments::first(std::string& aName,std::string& aValue) const {
  aName = "";
  aValue = "";
  if(fArgs.empty()) return false;
  aName = fArgs.front().first;
  aValue = fArgs.front().second;
  return true;
}

bool Lib::Arguments::last(std::string& aName,std::string& aValue) const {
  aName = "";
  aValue = "";
  if(fArgs.empty()) return false;
  aName = fArgs.back().first;
  aValue = fArgs.back().second;
  return true;
}

bool Lib::Arguments::file(std::string& aFile) const {
  std::string slast;
  std::string s;
  if((fArgs.size()>1) //first arg is the program name !
     && last(slast,s) 
     && (slast.find('-')!=0) 
     && (s.empty()) ) {
    aFile = slast; //Last argument is not an option.
    return true;
  } else {
    aFile = "";
    return false;
  }
}

std::vector<std::string> Lib::Arguments::files(bool a_skip_first) const {
  // Get the serie of trailing args not beginning with '-'
  // and without a value (not of the form [-]xxx=yyy). 
  // Note that an argument like that in between arguments
  // not like that is NOT taken into account. 
  std::vector<std::string> files;
  if(fArgs.empty()) return files;
  std::vector<Argument>::const_iterator it = fArgs.begin();
  if(a_skip_first) it++;
  for(;it!=fArgs.end();++it) {
    if( ((*it).first.find('-')==0) || (*it).second.size() ) {
      files.clear();
    } else {
      files.push_back((*it).first);
    }
  }
  return files;
}

void Lib::Arguments::files_at_end(bool a_skip_first) {
  // reorder to have "file" arguments at end.
  if(fArgs.empty()) return;
  std::vector<Argument> args;
  if(a_skip_first) args.push_back(*(fArgs.begin()));
  //first pass :
 {std::vector<Argument>::const_iterator it = fArgs.begin();
  if(a_skip_first) it++;
  for(;it!=fArgs.end();++it) {
    if( ((*it).first.find('-')==0) || (*it).second.size() ) {
      args.push_back(*it);
    }
  }}
  //second pass :
 {std::vector<Argument>::const_iterator it = fArgs.begin();
  if(a_skip_first) it++;
  for(;it!=fArgs.end();++it) {
    if( ((*it).first.find('-')==0) || (*it).second.size() ) {
    } else {
      args.push_back(*it);
    }
  }}
  fArgs = args;
}

bool Lib::Arguments::knownOptions(const std::vector<std::string>& aOptions) const {
  for(std::vector<Argument>::const_iterator it = fArgs.begin();
      it!=fArgs.end();++it) {
    if((*it).first.find('-')==0) { //find '-' at first pos.
      bool found = false;
      for(std::vector<std::string>::const_iterator it2 = aOptions.begin();
        it2!=aOptions.end();++it2) {
        if((*it).first==(*it2)) {
          found = true;
          break;        
        }
      }
      if(!found) return false;
    }
  }
  return true;
}

void Lib::Arguments::add(const std::vector<std::string>& aArgs,bool aStrip) {
  for(std::vector<std::string>::const_iterator it = aArgs.begin();
      it!=aArgs.end();++it) {
    std::vector<std::string> words;
    smanip::words((*it),"=",words);
    if(words.size()==1) {
      if(aStrip) {
        fArgs.push_back(Argument(Lib::smanip::strp(words[0]),"")); 
      } else {
        fArgs.push_back(Argument(words[0],"")); 
      }
    } else if(words.size()>=2) {
      if(aStrip) {
        fArgs.push_back(Argument(Lib::smanip::strp(words[0]),
                                 Lib::smanip::strp(words[1]))); 
      } else {
        fArgs.push_back(Argument(words[0],words[1])); 
      }
    }
  }
}

#include <string.h>

void Lib::Arguments::argcv(int& aArgc,char**& aArgv) const {
  // If using with :
  //   int argc;
  //   char** argv;
  //   args.argcv(argc,argv);  
  // delete with :
  //  for(int index=0;index<argc;index++) delete [] argv[index];
  //  delete [] argv;
  aArgc = (int)fArgs.size();
  if(aArgc<=0) {
    aArgc = 0;
    aArgv = 0;
    return;
  }
  typedef char* cstring;
  aArgv = new cstring[aArgc];
  for(int index=0;index<aArgc;index++) {
    std::string arg = fArgs[index].first;
    if(fArgs[index].second.size()) {
      arg += "=";
      arg += fArgs[index].second;
    }
    aArgv[index] = new char[arg.size()+1];
    ::strcpy(aArgv[index],arg.c_str());
  }
}
