// From Geant4/source/visualization/FukuiRenderer/G4FRBaseSocket code.
// Itself coming from dawn-3.85a.
//  tag : geant4-07-00-cand-00

// this :
#include <Lib/BaseSocket.h>

#include <Slash/Tools/platform.h>
#include <Slash/Tools/file.h>
#include <Slash/Tools/buffer.h>

#ifdef WIN32
#include <io.h>
#include <winsock2.h>
typedef int ssize_t; //FIXME : correct ?
#else
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>  //read,close
#include <string.h> //strerror
#endif

#define MINIMUM(a,b) ((a)<(b)?a:b)

//#define BLOCK 1 //to test things.

//#define BLOCK 1024
//#define BLOCK 2048
//#define BLOCK 4096
//#define BLOCK 8192
//#define BLOCK 16384
//
#define BLOCK 32768
//#define BLOCK 65536
//#define BLOCK 262144

Lib::BaseSocket::BaseSocket(std::ostream& a_out,int aVerboseLevel)
:m_out(a_out)
,fVerboseLevel(aVerboseLevel)
,fSocket(-1)           
,fIsLE(true)
{ 
  fIsLE = Slash::is_little_endian();
}

Lib::BaseSocket::BaseSocket(const BaseSocket& aFrom)
:m_out(aFrom.m_out)
,fVerboseLevel(aFrom.fVerboseLevel)
,fSocket(aFrom.fSocket)           
,fIsLE(aFrom.fIsLE)
{}

Lib::BaseSocket::~BaseSocket(){closeSocket();}

Lib::BaseSocket& Lib::BaseSocket::operator=(const BaseSocket& aFrom){ 
  fVerboseLevel = aFrom.fVerboseLevel;
  fSocket = aFrom.fSocket;
  fIsLE = aFrom.fIsLE;
  return *this;
}

bool Lib::BaseSocket::isConnected() const {return (fSocket==(-1)?false:true);}

int Lib::BaseSocket::socket() const {return fSocket;}

void Lib::BaseSocket::closeSocket(){
  if(fSocket!=(-1)) {
#ifdef WIN32
    if(::shutdown(fSocket,SD_BOTH)<0) { 
      if(errno==WSAENOTCONN) { 
#else
    if(::shutdown(fSocket,SHUT_RDWR)<0) { 
      if(errno==ENOTCONN) { 
#endif
        // ok, the "other side" may have already done a shutdown.
      } else {
        m_out << "Lib::BaseSocket::closeSocket :"
              << " for socket : " << fSocket
              << ", shutdown : errno " << errno
              << ", error : " << std::string(strerror(errno))
              << std::endl;
      }
    }
    ::close(fSocket);
    fSocket = -1;
  }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::send_buffer(const char* aBuffer,Slash::uint64 aLength) {
  if(!aLength) return true;
  if(fSocket==(-1)) return false;

  char* buf = (char*)aBuffer;
  Slash::uint64 sent = 0;

  while(true) {

    Slash::uint64 to_send = MINIMUM(BLOCK,aLength-sent);
    if(!to_send) break;
    
    ssize_t num_char = ::send(fSocket,(char*)buf,(size_t)to_send,0);
    if(num_char<0) {
      m_out << "Lib::BaseSocket::send_buffer :"
            << " send : " << std::string(strerror(errno))
            << std::endl;
      disconnect();
      return false;
    }

    //NOTE : num_char not necessary equal to to_send.
    buf += num_char;
    sent += num_char;
  }

  return true;
}

bool Lib::BaseSocket::wait_buffer(
 char* aBuffer  //assumed allocated !
,Slash::uint64 aLength //given !
){
  if(!aLength) return true;
  if(fSocket==(-1)) return false;

  char* buf = aBuffer;
  Slash::uint64 got = 0;

  while(true) {
    Slash::uint64 to_get = MINIMUM(BLOCK,aLength-got);
    if(!to_get) break;

    ssize_t num_char = ::recv(fSocket,buf,to_get,0);
    if(num_char<0) {
      m_out << "Lib::BaseSocket::wait_buffer :"
            << " recv(2) : " << std::string(strerror(errno))
            << std::endl;
      disconnect();
      return false;
    }

    //NOTE : num_char not necessary equal to to_get.
    buf += num_char;
    got += num_char;
  }
  return true;
}

bool Lib::BaseSocket::send_uint64(Slash::uint64 aValue) {
  char buffer[8];
  Slash::write_buffer<Slash::uint64>(aValue,buffer);
  return send_buffer(buffer,8);
}

bool Lib::BaseSocket::wait_uint64(Slash::uint64& aValue) {
  char buffer[8];
  if(!wait_buffer(buffer,8)) return false;
  Slash::read_buffer<Slash::uint64>(buffer,aValue);
  return true;
}

// for SDL WILD :
bool Lib::BaseSocket::send_uchar(unsigned char aValue) {
  char buffer[1];
  ::memcpy(buffer,&aValue,1);
  return send_buffer(buffer,1);
}

bool Lib::BaseSocket::wait_uchar(unsigned char& aValue) {
  char buffer[1];
  if(!wait_buffer(buffer,1)) return false;
  ::memcpy(&aValue,buffer,1);
  return true;
}

#define SEND(a_type)			\
  char buffer[sizeof(a_type)];		\
  Slash::write_buffer<a_type>(aValue,buffer);\
  return send_buffer(buffer,sizeof(a_type));

#define WAIT(a_type)			\
  char buffer[sizeof(a_type)];\
  if(!wait_buffer(buffer,sizeof(a_type))) return false;\
  Slash::read_buffer<a_type>(buffer,aValue);\
  return true;

bool Lib::BaseSocket::send_ushort(unsigned short aValue) { SEND(unsigned short) }
bool Lib::BaseSocket::wait_ushort(unsigned short& aValue) { WAIT(unsigned short) }
bool Lib::BaseSocket::send_short(short aValue) { SEND(short) }
bool Lib::BaseSocket::wait_short(short& aValue) { WAIT(short) }

bool Lib::BaseSocket::send_uint(unsigned int aValue) { SEND(unsigned int) }
bool Lib::BaseSocket::wait_uint(unsigned int& aValue) { WAIT(unsigned int) }
bool Lib::BaseSocket::send_int(int aValue) { SEND(int) }
bool Lib::BaseSocket::wait_int(int& aValue) { WAIT(int) }
bool Lib::BaseSocket::send_float(float aValue) { SEND(float) }
bool Lib::BaseSocket::wait_float(float& aValue) { WAIT(float) }

bool Lib::BaseSocket::send_bool(bool aValue) {
  unsigned char c = aValue?1:0;
  return send_uchar(c);
}

bool Lib::BaseSocket::wait_bool(bool& aValue) {
  unsigned char c;
  if(!wait_uchar(c)) {aValue = false;return false;}
  aValue = (c==1?true:false);
  return true;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

bool Lib::BaseSocket::send_string(const std::string& aString) {
  if(!send_uint64(aString.size())) return false;
  return send_buffer(aString.c_str(),aString.size());
}

bool Lib::BaseSocket::wait_string(std::string& aString) {
  Slash::uint64 length;
  if(!wait_uint64(length)) {
    aString.clear();
    return false;
  }
  aString.resize(length);
  if(!wait_buffer((char*)aString.c_str(),length)) {
    aString.clear();
    return false;
  }
  return true;
}

bool Lib::BaseSocket::send_text(const std::vector<std::string>& aText,bool aDump) { 
  unsigned int number = aText.size();
  if(!send_uint64(number)) return false;
  for(unsigned int index=0;index<number;index++) {
    if(!send_string(aText[index])) return false;
  }
  return true;
}

bool Lib::BaseSocket::wait_text(std::vector<std::string>& aText,bool aDump) { 
  Slash::uint64 number;
  if(!wait_uint64(number)) {
    aText.clear();
    return false;
  }
  for(unsigned int index=0;index<number;index++) {
    std::string s;
    if(!wait_string(s)) {
      aText.clear();
      return false;
    }
    aText.push_back(s);
  }
  return true;
}

bool Lib::BaseSocket::send_file(const std::string& aFile) {
  char* buffer;
  long length;
  if(!Slash::file::read_bytes(aFile,buffer,length)) {
    m_out << "Lib::BaseSocket::send_file : "
          << " can't read file \"" << aFile << "\"."
          << std::endl;
    return false;
  }
  if(!send_uint64((Slash::uint64)length)) return false;
  bool stat = send_buffer(buffer,length);
  delete [] buffer;
  return stat;
}

bool Lib::BaseSocket::wait_file(const std::string& aFile) {
  Slash::uint64 length;
  if(!wait_uint64(length)) return false;
  char* buffer = 0;
  if(length) buffer = new char[length];
  if(!wait_buffer(buffer,length)) {
    delete [] buffer;
    return false;
  }
  bool stat = Slash::file::write_bytes(aFile,buffer,(size_t)length);
  if(!stat) {
    m_out << "Lib::BaseSocket::send_file : "
          << " can't write file \"" << aFile << "\"."
          << std::endl;
  }
  delete [] buffer;
  return stat;
}

#define MAXIMUM(a,b) ((a)>(b)?a:b)

bool Lib::BaseSocket::is_there_input(bool& a_is) {
  a_is = false;
  if(fSocket==(-1)) return false;

  fd_set mask;
  FD_ZERO(&mask);
  FD_SET(fSocket,&mask);

  static struct timeval timeout;
  timeout.tv_sec = 0;
  timeout.tv_usec = 10; //microsec

  int nfds = 0;
  nfds = MAXIMUM(nfds,fSocket);
  nfds++;
  if(::select(nfds,&mask,0,0,&timeout)==(-1)) {
    a_is = false;
    return false;
  }

  a_is = FD_ISSET(fSocket,&mask)?true:false;
  return true;
}

