// this :
#include <Lib/Cloner.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>

#include <stdio.h>

namespace Lib {

class Dump : public Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& aString,bool&) {
    Lib::Out out(fPrinter);
    out << "dir : " << aString << Lib::endl;
    return true;
  }
  virtual bool file(const std::string& aString) {
    Lib::Out out(fPrinter);
    out << "file : " << aString << Lib::endl;
    return true;
  }
public:
  Dump(Slash::Core::IWriter& aPrinter):fPrinter(aPrinter){}
  virtual ~Dump() {};
private:
  Slash::Core::IWriter& fPrinter;
};

class Clone : public virtual Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& aDir,bool& aProcess) {
    aProcess = true;
    unsigned int l = fDir.size();
    std::string to = fTo + aDir.substr(l,aDir.size()-l);
    replace(to);
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(to,p,n,s);
    if((n=="CVS")||(n==".svn")) {
      //Lib::Out out(fPrinter);
      //out << "Do not create CVS directory " << Lib::sout(to) << "." 
      //    << Lib::endl;
      aProcess = false;
    } else {
      if(!Lib::dirmanip::create(to)) {
        Lib::Out out(fPrinter);
        out << "Can't create dir " << Lib::sout(to) << "." << Lib::endl;
        fFailure = true;
        return false;
      } else {
        //Lib::Out out(fPrinter);
        //out << "dir " << Lib::sout(to) << " created." << Lib::endl;
      }
    }
    return true;
  }
  virtual bool file(const std::string& aFile) {
    if(fVerbose) {
      Lib::Out out(fPrinter);
      out << "Lib_clone : treat " << Lib::sout(aFile) << "" << Lib::endl;
      //out << "Lib_clone :  fDir " << Lib::sout(fDir) << "" << Lib::endl;
      //out << "Lib_clone :  fTo " << Lib::sout(fTo) << "" << Lib::endl;
    }
    unsigned int l = fDir.size();
    std::string newName = fTo + aFile.substr(l,aFile.size()-l);
    replace(newName);
    std::vector<std::string> text;
    if(!Lib::fmanip::read(aFile,text)) {
      Lib::Out out(fPrinter);
      out << "Can't read " << Lib::sout(aFile) << Lib::endl;
      fFailure = true;
      return false;
    }
    FILE* file = ::fopen(newName.c_str(),"wb");
    if(!file) {
      Lib::Out out(fPrinter);
      out << "Can't create " << Lib::sout(newName) << Lib::endl;
      fFailure = true;
      return false;
    }
    unsigned int textn = text.size();
    for(unsigned int index=0;index<textn;index++) {
      std::string s = text[index];
      replace(s);
      Lib::smanip::replace(s,"LIB_CLONE_PWD",fPWD);
      fprintf(file,"%s\n",s.c_str());
    }
    ::fclose(file);
    return true;
  }
public:
  Clone(Slash::Core::IWriter& aPrinter,
        const std::string& aPWD,
        const std::string& aDir,
        const std::string& aFrom,
        const std::string& aTo,
        bool aVerbose = false)
  :fPrinter(aPrinter)
  ,fPWD(aPWD)
  ,fDir(aDir)
  ,fFrom(aFrom)
  ,fTo(aTo)
  ,fVerbose(aVerbose)
  ,fFailure(false){
    fFROM = fFrom;
    fTO = fTo;
    Lib::smanip::touppercase(fFROM);
    Lib::smanip::touppercase(fTO);
    f_pack = fFrom;
    f_to = fTo;
    Lib::smanip::tolowercase(f_pack);
    Lib::smanip::tolowercase(f_to);
  }
  virtual ~Clone() {}
  bool isSuccess() const { return !fFailure;}
private:
  void replace(std::string& aString) {
    Lib::smanip::replace(aString,fFrom,fTo);
    Lib::smanip::replace(aString,fFROM,fTO);
    Lib::smanip::replace(aString,f_pack,f_to);
  }
private:
  Slash::Core::IWriter& fPrinter;
  std::string fPWD;
  std::string fDir;
  std::string fFrom;
  std::string fTo;
  std::string fFROM;
  std::string fTO;
  std::string f_pack;
  std::string f_to;
  bool fVerbose;
  bool fFailure;
};

}

//////////////////////////////////////////////////////////////////////////////
bool Lib::Cloner::clone(
 const std::string& aDirectory
,const std::string& aFrom
,const std::string& aTo
,Slash::Core::IWriter& aPrinter
,bool aVerbose
,bool aDebug
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aTo.find(' ')!=std::string::npos) {
    Lib::Out out(aPrinter);
    out << "Lib::Cloner::clone :"
        << " can't have spaces in new name (" << Lib::sout(aTo) << ")." 
        << Lib::endl;
    return false;
  }

  std::string spwd;
  if(!Lib::dirmanip::directory(spwd)) {
    Lib::Out out(aPrinter);
    out << "Lib::Cloner::clone :"
        << " can't get current directory."
        << Lib::endl;
    return false;
  }

  Lib::Directory tree;
  tree.setPath(aDirectory);
  if(!Lib::dirmanip::content(tree)) {
    Lib::Out out(aPrinter);
    out << "Lib::Cloner::clone :"
        << " can't get files of " << Lib::sout(aDirectory) 
        << "." << Lib::endl;
    return false;
  }

  if(aDebug) {
    Dump dump(aPrinter);
    tree.visit(dump);
  }

  Clone clone(aPrinter,spwd,aDirectory,aFrom,aTo,aVerbose);
  tree.visit(clone);

  return clone.isSuccess();
}
