// this :
#include <Lib/ExpFunc.h>

#include <Lib/Debug.h>

Lib::ExpFunc::ExpFunc(const std::string& aName,unsigned int aNumber)
:fName(aName)
,fNumArgs(aNumber)
{
  Debug::increment("Lib::ExpFunc");
}

Lib::ExpFunc::~ExpFunc(){ 
  Debug::decrement("Lib::ExpFunc");
}

Lib::ExpFunc::ExpFunc(const ExpFunc& aFrom) 
:IExpFunc()
,fName(aFrom.fName)
,fNumArgs(aFrom.fNumArgs)
{
  Debug::increment("Lib::ExpFunc");
}
Lib::ExpFunc& Lib::ExpFunc::operator=(const ExpFunc& aFrom) {
  fName = aFrom.fName;
  fNumArgs = aFrom.fNumArgs;
  return *this;
}
unsigned int Lib::ExpFunc::numberOfArguments() const { return fNumArgs;}
std::string Lib::ExpFunc::name() const { return fName;}

#define TO_D(aVariable,aD)\
 {if(!Lib::valuemanip::to_double(aVariable,aD)) {\
   aError = "Lib::Expression::execute : can't convert Variable to double.";\
   a_return.setNone();\
   return false;\
 }}

#include <Lib/Value.h>
#include <Lib/valuemanip.h>
#include <Lib/smanip.h>

bool Lib::ExpFunc::eval0(const Lib::Value& a_x,Lib::Value& a_return,std::string& aError,Func0 aFunc) {

  if(a_x.type()==Slash::Core::IValue::ARRAY_DOUBLE) {

   {std::vector<unsigned int> is;
    const std::vector<double>& stdv = a_x.get_array_double(is);
    a_return.set(is,stdv);}

    // In the below, we need a non const method.
    std::vector<double>& stdv = a_return.array_double();
    unsigned int number = stdv.size();
    for(unsigned int index=0;index<number;index++) { 
      stdv[index] = aFunc(stdv[index]);
    }
    return true;
  } else {
    double x;
    TO_D(a_x,x)
    a_return.set(aFunc(x));
    return true;
  }
}
bool Lib::ExpFunc::eval0(const Lib::Value& a_x,Lib::Value& a_return,std::string& aError,Func0 aFunc,InDomain aDomain) {

  if(a_x.type()==Slash::Core::IValue::ARRAY_DOUBLE) {

   {std::vector<unsigned int> is;
    const std::vector<double>& stdv = a_x.get_array_double(is);
    a_return.set(is,stdv);}

    // In the below, we need a non const method.
    std::vector<double>& stdv = a_return.array_double();
    unsigned int number = stdv.size();
    for(unsigned int index=0;index<number;index++) { 
      double x = stdv[index];
      if(!aDomain(x)) {
        aError = std::string("Expression::execute : ")
               + " " + fName + " : variable value "+ Lib::smanip::tostring(x)
               + " out of domain.";
        a_return.setNone();
        return false;
      }
      stdv[index] = aFunc(x);
    }
    return true;
  } else {
    double x;
    TO_D(a_x,x)

    if(!aDomain(x)) {
      aError = std::string("Expression::execute : ")
             + " " + fName + " : variable value "+ Lib::smanip::tostring(x)
             + " out of domain.";
      a_return.setNone();
      return false;
    }

    a_return.set(aFunc(x));
    return true;
  }
}
bool Lib::ExpFunc::eval1(const Lib::Value& a_x,Lib::Value& a_return,std::string& aError,const Lib::Value& a_1,Func1 aFunc) {

  double d_1;
  TO_D(a_1,d_1)

  if(a_x.type()==Slash::Core::IValue::ARRAY_DOUBLE) {

   {std::vector<unsigned int> is;
    const std::vector<double>& stdv = a_x.get_array_double(is);
    a_return.set(is,stdv);}

    // In the below, we need a non const method.
    std::vector<double>& stdv = a_return.array_double();
    unsigned int number = stdv.size();
    for(unsigned int index=0;index<number;index++) { 
      stdv[index] = aFunc(stdv[index],d_1);
    }
    return true;
  } else {
    double x;
    TO_D(a_x,x)
    a_return.set(aFunc(x,d_1));
    return true;
  }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Functions ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <math.h>
#include <stdlib.h>
#include <Lib/mmanip.h>

extern "C" {
static double Lib_RandomFlat_shoot(double /*aSeed*/) {
  // Shoot random numbers in [0,1] according a flat distribution.
  //FIXME srandom(aSeed);  
  double value  = (double)::rand();
  value /= (double)RAND_MAX;
  return value;
}
static double toint(double a_x) {return (double)int(a_x);}
}

#define LIB_EXPFUNC_CPCSTOR(a_func) \
  Lib::ExpFunc_##a_func::ExpFunc_##a_func(const Lib::ExpFunc_##a_func& aFrom):ExpFunc(aFrom){}\
  Lib::ExpFunc_##a_func& Lib::ExpFunc_##a_func::operator=(const Lib::ExpFunc_##a_func& aFrom){ ExpFunc::operator=(aFrom);return *this;}\
  Lib::IExpFunc* Lib::ExpFunc_##a_func::copy() const { return new Lib::ExpFunc_##a_func(*this);}

#define LIB_EXPFUNC_0(a_func,a_f) \
  Lib::ExpFunc_##a_func::ExpFunc_##a_func():ExpFunc(#a_func,1){}\
  LIB_EXPFUNC_CPCSTOR(a_func)\
  Lib::ExpFunc_##a_func::~ExpFunc_##a_func(){}\
  bool Lib::ExpFunc_##a_func::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {\
    if(a_args.size()!=fNumArgs) return false;\
    return ExpFunc::eval0(a_args[0],a_return,aError,a_f);\
  }

LIB_EXPFUNC_0(cos,::cos)
LIB_EXPFUNC_0(sin,::sin)
LIB_EXPFUNC_0(cosh,::cosh)
LIB_EXPFUNC_0(sinh,::sinh)
LIB_EXPFUNC_0(tanh,::tanh)
LIB_EXPFUNC_0(atan,::atan)
LIB_EXPFUNC_0(sqrt,::sqrt)
LIB_EXPFUNC_0(exp,::exp)
LIB_EXPFUNC_0(fabs,::fabs)
LIB_EXPFUNC_0(abs,::fabs)
LIB_EXPFUNC_0(rndm,Lib_RandomFlat_shoot)
LIB_EXPFUNC_0(toint,toint)
LIB_EXPFUNC_0(int,toint) //for OpenPAW/c_pawex20.kumac

#define LIB_EXPFUNC_DOMAIN_0(a_func,a_f,a_domain) \
  Lib::ExpFunc_##a_func::ExpFunc_##a_func():ExpFunc(#a_func,1){}\
  LIB_EXPFUNC_CPCSTOR(a_func)\
  Lib::ExpFunc_##a_func::~ExpFunc_##a_func(){}\
  bool Lib::ExpFunc_##a_func::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {\
    if(a_args.size()!=fNumArgs) return false;\
    return ExpFunc::eval0(a_args[0],a_return,aError,a_f,a_domain);\
  }

LIB_EXPFUNC_DOMAIN_0(tan,::tan,Lib::mmanip::in_domain_tan)
LIB_EXPFUNC_DOMAIN_0(acos,::acos,Lib::mmanip::in_domain_acos)
LIB_EXPFUNC_DOMAIN_0(asin,::asin,Lib::mmanip::in_domain_acos)
LIB_EXPFUNC_DOMAIN_0(log,::log,Lib::mmanip::in_domain_log)
LIB_EXPFUNC_DOMAIN_0(log10,::log10,Lib::mmanip::in_domain_log)

Lib::ExpFunc_nco::ExpFunc_nco():ExpFunc("nco",1){}
LIB_EXPFUNC_CPCSTOR(nco)
Lib::ExpFunc_nco::~ExpFunc_nco(){}
bool Lib::ExpFunc_nco::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
    std::vector<unsigned int> is;
    a_return.set((unsigned int)a_args[0].get_array_double(is).size());
    return true;
  }
  return false;
}

Lib::ExpFunc_vsum::ExpFunc_vsum():ExpFunc("vsum",1){}
LIB_EXPFUNC_CPCSTOR(vsum)
Lib::ExpFunc_vsum::~ExpFunc_vsum(){}
bool Lib::ExpFunc_vsum::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
    std::vector<unsigned int> is;
    const std::vector<double>& stdv = a_args[0].get_array_double(is);
    double dv = 0;
    unsigned int number = stdv.size();
    for(unsigned int index=0;index<number;index++) dv += stdv[index];
    a_return.set(dv);
    return true;
  } 
  return false;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#define LIB_EXPFUNC_1(a_func,a_f) \
  Lib::ExpFunc_##a_func::ExpFunc_##a_func():ExpFunc(#a_func,2){}\
  LIB_EXPFUNC_CPCSTOR(a_func)\
  Lib::ExpFunc_##a_func::~ExpFunc_##a_func(){}\
  bool Lib::ExpFunc_##a_func::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {\
    if(a_args.size()!=fNumArgs) return false;\
    return ExpFunc::eval1(a_args[0],a_return,aError,a_args[1],a_f);\
  }

LIB_EXPFUNC_1(pow,::pow)
LIB_EXPFUNC_1(mod,::fmod)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <Lib/Array.h>

Lib::ExpFunc_array::ExpFunc_array():ExpFunc("array",3){}
LIB_EXPFUNC_CPCSTOR(array)
Lib::ExpFunc_array::~ExpFunc_array(){}
bool Lib::ExpFunc_array::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)

    int n = (int)x;
    std::vector<unsigned int> orders(1);
    orders[0] = n;
    Lib::Array<double> v(orders);
    if(n>=2) {
      std::vector<double>& stdv = v.vector();
      double delta = (d_2 - d_1)/(n-1);
      for(int i=0;i<n;i++) stdv[i] = d_1 + i * delta;
    }
    a_return.set(v.orders(),v.vector());
    return true;

  }
  return false;
}

#include <Lib/Functions.h>

Lib::ExpFunc_expo::ExpFunc_expo():ExpFunc("expo",3){}
LIB_EXPFUNC_CPCSTOR(expo)
Lib::ExpFunc_expo::~ExpFunc_expo(){}
bool Lib::ExpFunc_expo::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)

    Lib::Function::Exponential func(d_1,d_2);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

Lib::ExpFunc_pol1::ExpFunc_pol1():ExpFunc("pol1",3){}
LIB_EXPFUNC_CPCSTOR(pol1)
Lib::ExpFunc_pol1::~ExpFunc_pol1(){}
bool Lib::ExpFunc_pol1::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)

    std::vector<double> params(2);
    params[0] = d_1;
    params[1] = d_2;
    Lib::Function::Polynomial func(params);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Lib::ExpFunc_gauss::ExpFunc_gauss():ExpFunc("gauss",4){}
LIB_EXPFUNC_CPCSTOR(gauss)
Lib::ExpFunc_gauss::~ExpFunc_gauss(){}
bool Lib::ExpFunc_gauss::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)

    Lib::Function::Gaussian func(d_1,d_2,d_3);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

Lib::ExpFunc_bw::ExpFunc_bw():ExpFunc("bw",4){}
LIB_EXPFUNC_CPCSTOR(bw)
Lib::ExpFunc_bw::~ExpFunc_bw(){}
bool Lib::ExpFunc_bw::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {
    double x;
    TO_D(a_args[0],x)
    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)
    Lib::Function::BreitWigner func(d_1,d_2,d_3);
    a_return.set(func.value(x));
    return true;
  }
  return false;
}

// For backward compatibility. Same as upper.
Lib::ExpFunc_breit::ExpFunc_breit():ExpFunc("breit",4){}
LIB_EXPFUNC_CPCSTOR(breit)
Lib::ExpFunc_breit::~ExpFunc_breit(){}
bool Lib::ExpFunc_breit::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {
    double x;
    TO_D(a_args[0],x)
    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)
    Lib::Function::BreitWigner func(d_1,d_2,d_3);
    a_return.set(func.value(x));
    return true;
  }
  return false;
}

Lib::ExpFunc_cauchy::ExpFunc_cauchy():ExpFunc("cauchy",4){}
LIB_EXPFUNC_CPCSTOR(cauchy)
Lib::ExpFunc_cauchy::~ExpFunc_cauchy(){}
bool Lib::ExpFunc_cauchy::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)

    Lib::Function::Cauchy func(d_1,d_2,d_3);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

Lib::ExpFunc_pol2::ExpFunc_pol2():ExpFunc("pol2",4){}
LIB_EXPFUNC_CPCSTOR(pol2)
Lib::ExpFunc_pol2::~ExpFunc_pol2(){}
bool Lib::ExpFunc_pol2::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)

    std::vector<double> params(3);
    params[0] = d_1;
    params[1] = d_2;
    params[2] = d_3;
    Lib::Function::Polynomial func(params);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Lib::ExpFunc_pol3::ExpFunc_pol3():ExpFunc("pol3",5){}
LIB_EXPFUNC_CPCSTOR(pol3)
Lib::ExpFunc_pol3::~ExpFunc_pol3(){}
bool Lib::ExpFunc_pol3::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)
    double d_4;
    TO_D(a_args[4],d_4)

    std::vector<double> params(4);
    params[0] = d_1;
    params[1] = d_2;
    params[2] = d_3;
    params[3] = d_4;
    Lib::Function::Polynomial func(params);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Lib::ExpFunc_pol4::ExpFunc_pol4():ExpFunc("pol4",6){}
LIB_EXPFUNC_CPCSTOR(pol4)
Lib::ExpFunc_pol4::~ExpFunc_pol4(){}
bool Lib::ExpFunc_pol4::eval(const Args& a_args,Lib::Value& a_return,std::string& aError) {
  if(a_args.size()!=fNumArgs) return false;
  if(a_args[0].type()==Slash::Core::IValue::ARRAY_DOUBLE) {
  } else {

    double x;
    TO_D(a_args[0],x)

    double d_1;
    TO_D(a_args[1],d_1)
    double d_2;
    TO_D(a_args[2],d_2)
    double d_3;
    TO_D(a_args[3],d_3)
    double d_4;
    TO_D(a_args[4],d_4)
    double d_5;
    TO_D(a_args[5],d_5)

    std::vector<double> params(5);
    params[0] = d_1;
    params[1] = d_2;
    params[2] = d_3;
    params[3] = d_4;
    params[4] = d_5;
    Lib::Function::Polynomial func(params);
    a_return.set(func.value(x));
    return true;

  }
  return false;
}


//////////////////////////////////////////////////////////////////////////////
/// Manager //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Lib::ExpFuncs::ExpFuncs() {
  Debug::increment("Lib::ExpFuncs");

#define LIB_EXPFUNC_ADD(a_func) \
  fFuncs.push_back(new ExpFunc_##a_func());

  LIB_EXPFUNC_ADD(cos)
  LIB_EXPFUNC_ADD(sin)
  LIB_EXPFUNC_ADD(cosh)
  LIB_EXPFUNC_ADD(sinh)
  LIB_EXPFUNC_ADD(tanh)
  LIB_EXPFUNC_ADD(atan)
  LIB_EXPFUNC_ADD(sqrt)
  LIB_EXPFUNC_ADD(exp)
  LIB_EXPFUNC_ADD(fabs)
  LIB_EXPFUNC_ADD(abs)
  LIB_EXPFUNC_ADD(rndm)
  LIB_EXPFUNC_ADD(toint)

  LIB_EXPFUNC_ADD(tan)
  LIB_EXPFUNC_ADD(acos)
  LIB_EXPFUNC_ADD(asin)
  LIB_EXPFUNC_ADD(log)
  LIB_EXPFUNC_ADD(log10)

  LIB_EXPFUNC_ADD(nco)
  LIB_EXPFUNC_ADD(vsum)

  LIB_EXPFUNC_ADD(pow)
  LIB_EXPFUNC_ADD(mod)
  LIB_EXPFUNC_ADD(int)

  LIB_EXPFUNC_ADD(array)
  LIB_EXPFUNC_ADD(expo)
  LIB_EXPFUNC_ADD(pol1)

  LIB_EXPFUNC_ADD(gauss)
  LIB_EXPFUNC_ADD(bw)
  LIB_EXPFUNC_ADD(cauchy)
  LIB_EXPFUNC_ADD(pol2)

  LIB_EXPFUNC_ADD(pol3)

  LIB_EXPFUNC_ADD(pol4)

  // For backward compatibility :
  LIB_EXPFUNC_ADD(breit)
}

Lib::ExpFuncs::~ExpFuncs(){ 
  std::vector<IExpFunc*>::iterator it;
  while(!fFuncs.empty()) {
    it = fFuncs.end();
    it--;
    IExpFunc* func = *it;
    fFuncs.erase(it);
    delete func;
  }
  Debug::decrement("Lib::ExpFuncs");
}

Lib::ExpFuncs::ExpFuncs(const ExpFuncs& aFrom) {
  for(std::vector<IExpFunc*>::const_iterator it = aFrom.fFuncs.begin();
      it!=aFrom.fFuncs.end();++it) {
    fFuncs.push_back((*it)->copy());
  }  
  Debug::increment("Lib::ExpFuncs");
}
Lib::ExpFuncs& Lib::ExpFuncs::operator=(const ExpFuncs& aFrom) {
  for(std::vector<IExpFunc*>::const_iterator it = aFrom.fFuncs.begin();
      it!=aFrom.fFuncs.end();++it) {
    fFuncs.push_back((*it)->copy());
  }  
  return *this;
}

void Lib::ExpFuncs::add(Lib::IExpFunc* aFunc) {fFuncs.push_back(aFunc);}

Lib::IExpFunc* Lib::ExpFuncs::find(const std::string& aString) const {  
  for(std::vector<IExpFunc*>::const_iterator it = fFuncs.begin();
      it!=fFuncs.end();++it) {
    if((*it)->name()==aString) return (*it);
  }  
  return 0;
}

