
// For Lab/tests/Lab_hbook_test.f in order
// to use the same generator that for HCL/examples/HCL_example.cxx
// and Lab/tests/Lab_ROOT_test.cxx

#include <Lib/F77.h>

#include <Lib/mmanip.h>

#include <math.h>
#include <stdlib.h>

#ifdef WIN32
#define librflat  LIBRFLAT
#define librgauss LIBRGAUSS
#define librbw    LIBRBW
#else
#define librflat  librflat_
#define librgauss librgauss_
#define librbw    librbw_
#endif

extern "C" {
  float Lib_stdcall librflat();
  float Lib_stdcall librgauss(float*,float*);
  float Lib_stdcall librbw(float*,float*);
}

//////////////////////////////////////////////////////////////////////////////
static double Lib_RandomFlat_shoot(
)
//////////////////////////////////////////////////////////////////////////////
// Shoot random numbers according a flat distribution.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value  = (double)::rand();
  value /= (double)RAND_MAX;
  return value;
}
//////////////////////////////////////////////////////////////////////////////
float Lib_stdcall librflat(
)
//////////////////////////////////////////////////////////////////////////////
// Shoot random numbers according a flat distribution.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)Lib_RandomFlat_shoot();
}
//////////////////////////////////////////////////////////////////////////////
float Lib_stdcall librgauss(
 float* aMean
,float* aStdDev
)
//////////////////////////////////////////////////////////////////////////////
//  Shoot random numbers according a 
// gaussian distribution of mean 0 and sigma 1.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double v1,v2,r,fac;
  do {
    v1 = 2.0 * Lib_RandomFlat_shoot() - 1.0;
    v2 = 2.0 * Lib_RandomFlat_shoot() - 1.0;
    r = v1*v1 + v2*v2;
  } while ( r > 1.0 );
  fac = ::sqrt(-2.0*::log(r)/r);
  return (float)((v2 * fac) * (*aStdDev) + (*aMean));
}
//////////////////////////////////////////////////////////////////////////////
float Lib_stdcall librbw(
 float* aMean
,float* aGamma
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double rval = 2.0 * Lib_RandomFlat_shoot() - 1.0;
  double displ = 0.5 * (*aGamma) * ::tan(rval * Lib::mmanip::half_pi());
  return (float)((*aMean) + displ);
}
