// this :
#include <Lib/FortranOut.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/FortranFormat.h>

/*
   When there is too few data compared to what the format describe,
  we stop (without error) at the end of data (like FORTRAN). For example :
    FormatOut out(<someWriter>,"(2(F10.2,1X,I5))");   
    out << 3.14 << 314 << Lib::endl;
  will write the two numbers and stop here.

   When the format is "exhausted" (too much data), 
  we go to next line and start again the format.
    FormatOut out(<someWriter>,"(F10.2,1X,I5)");   
    out << 3.14 << << 314 << 3.14 << 314 << Lib::endl;
  will write :
    3.14 314
    3.14 314
*/

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::endl(
 FortranOut& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aOut.status()) return aOut;
  aOut.writer().write("\n");
  return aOut;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut::FortranOut(
 Slash::Core::IWriter& aWriter
,const std::string& aFormat
,int aVerboseLevel
)
:fWriter(aWriter)
,fVerboseLevel(aVerboseLevel)
,fParser(aFormat,aWriter)
,fIterator(0)
,fNext(0)
,fStatus(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fParser.parse()) {
    Lib::Out out(aWriter);
    out << "Lib::FortranOut::FortranOut : " 
        << " parse of " << Lib::sout(aFormat) << " failed." << Lib::endl;
    return;
  } 
  if(fVerboseLevel) fParser.result().dump(fWriter);
  fIterator = fParser.result().createIterator();
  reset();
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut::~FortranOut(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fIterator;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut::FortranOut(
 const FortranOut& aFrom
)
:fWriter(aFrom.fWriter)
,fParser("",fWriter)
//////////////////////////////////////////////////////////////////////////////
// private.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator =(
 const Lib::FortranOut&
)
//////////////////////////////////////////////////////////////////////////////
// private.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::FortranOut::writer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fWriter;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::FortranOut::reset(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fIterator) return;
  // If we have an iterator, the given format had been parsed and is ok.
  fIterator->reset();
  fNext = 0;
  fStatus = true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::FortranOut::status(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStatus;
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor* Lib::FortranOut::next(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  while(true) {
    Lib::DataDescriptor* desc = fIterator->next();
    if(!desc) return 0;
    if(desc->fType==DataDescriptor_X) {
      fWriter.write(" ");
    } else if(desc->fType==DataDescriptor_SLASH) {
      fWriter.write("\n");
    } else {
      return desc;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 FortranOutFunc aFunc
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aFunc(*this);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fStatus) return *this;
  if(!fNext) fNext = next();
  if(fIterator->finished()) {
    fWriter.write("\n");
    fIterator->reset();
    fNext = next();
  }

  DataDescriptor* desc = fNext;
  if(!desc) return *this; //No data desc in format !

  if(desc->fType!=DataDescriptor_A) {
    fWriter.write("Lib::FortranOut::operator<<(const std::string&) :");
    fWriter.write(" format mismatch.\n");
    desc->dump(fWriter);
    fStatus = false;
    return *this;
  }
  //desc->dump(fWriter);
  if(desc->fWidth>0) { 
    std::string s(aValue);
    f_smanip.justify(s,desc->fWidth,smanip::right);
    fWriter.write(s);
  } else {
    fWriter.write(aValue);
  }
  fNext = next();
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 const char* aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << std::string(aValue);
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 bool aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fStatus) return *this;
  if(!fNext) fNext = next();
  if(fIterator->finished()) {
    fWriter.write("\n");
    fIterator->reset();
    fNext = next();
  }

  DataDescriptor* desc = fNext;
  if(!desc) return *this; //No data desc in format !

  if(desc->fType!=DataDescriptor_L) {
    fWriter.write("Lib::FortranOut::operator<<(bool) :");
    fWriter.write(" format mismatch.\n");
    desc->dump(fWriter);
    fStatus = false;
    return *this;
  }
  if(desc->fWidth<=0) {
    fWriter.write("Lib::FortranOut::operator<<(bool) :");
    fWriter.write(" null width.\n");
    desc->dump(fWriter);
    return *this;
  }
  //desc->dump(fWriter);
  std::string s;
  if(aValue) s = "T";
  else       s = "F";
  f_smanip.justify(s,desc->fWidth,smanip::right);
  fWriter.write(s);
  fNext = next();
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 double aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fStatus) return *this;
  if(!fNext) fNext = next();
  if(fIterator->finished()) {
    fWriter.write("\n");
    fIterator->reset();
    fNext = next();
  }
  DataDescriptor* desc = fNext;
  if(!desc) return *this; //No data desc in format !

  if( (desc->fType!=DataDescriptor_F) &&
      (desc->fType!=DataDescriptor_E) &&
      (desc->fType!=DataDescriptor_G) ){
    fWriter.write("Lib::FortranOut::operator<<(double) :");
    fWriter.write(" format mismatch.\n");
    desc->dump(fWriter);
    fStatus = false;
    return *this;
  }
  if(desc->fWidth<=0) {
    fWriter.write("Lib::FortranOut::operator<<(double) :");
    fWriter.write(" null width.\n");
    desc->dump(fWriter);
    return *this;
  }
  //desc->dump(fWriter);
  std::string s;
  if(!f_smanip.printf(s,desc->fWidth,desc->cformat().c_str(),aValue)) {
    fWriter.write("Lib::FortranOut::operator<<(double) :");
    fWriter.write(" something wrong in sprintf.\n");
    fStatus = false;
    return *this;
  }
  fWriter.write(s);
  fNext = next();
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 float aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << (double)aValue;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 long aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fStatus) return *this;
  if(!fNext) fNext = next();
  if(fIterator->finished()) {
    fWriter.write("\n");
    fIterator->reset();
    fNext = next();
  }

  DataDescriptor* desc = fNext;
  if(!desc) return *this; //No data desc in format !

  if(desc->fType!=DataDescriptor_I) {
    fWriter.write("Lib::FortranOut::operator<<(long) :");
    fWriter.write(" format mismatch.\n");
    desc->dump(fWriter);
    fStatus = false;
    return *this;
  }
  if(desc->fWidth<=0) {
    fWriter.write("Lib::FortranOut::operator<<(long) :");
    fWriter.write(" null width.\n");
    desc->dump(fWriter);
    return *this;
  }
  //desc->dump(fWriter);
  std::string s;
  if(!f_smanip.printf(s,desc->fWidth,desc->cformat().c_str(),aValue)) {
    fWriter.write("Lib::FortranOut::operator<<(long) :");
    fWriter.write(" something wrong in sprintf.\n");
    fStatus = false;
    return *this;
  }
  fWriter.write(s);
  fNext = next();
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 int aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << (long)aValue;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 unsigned int aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << (long)aValue;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 unsigned long aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << (long)aValue;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 const void* aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (*this) << (long)aValue;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 char aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  s += aValue;
  return (*this) << s;
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranOut& Lib::FortranOut::operator<<(
 const std::vector<double>& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fStatus) return *this;
  unsigned int number = aValue.size();
  for(unsigned int index=0;index<number;index++) {
    (*this) << (aValue[index]);
    if(!fStatus) break;
  }
  return *this;
}
