// this :
#include <Lib/InternetSocket.h>

#ifdef WIN32
#include <io.h>
#include <winsock.h>
#define CLIENT_ADDR_LENGTH int
#else
#include<sys/socket.h>
#include<netinet/in.h>
#include<netdb.h>
#include <errno.h>
#include <unistd.h>  //close,sleep
#ifdef __alpha
#define CLIENT_ADDR_LENGTH int
#else
#define CLIENT_ADDR_LENGTH socklen_t
#endif
#endif

#include <string.h> //strerror

// #include <sys/systeminfo.h>
// #define  GET_HOSTNAME( hostname, length )  sysinfo( SI_HOSTNAME, hostname, length ) 
#define  GET_HOSTNAME( hostname, length )  gethostname( hostname, length ) 


const int  MAX_BINDING_TRIAL = 10;
const int  BINDING_TRIAL_INTERVAL = 5;
const int  MAX_CONNECT_TRIAL = 10;

namespace Lib {

InternetSocket::InternetSocket(std::ostream& a_out,const std::string& a_host,int a_port,int aVerboseLevel)
:BaseSocket(a_out,aVerboseLevel)
,fHost(a_host)
,fPort(a_port)
,fAfterBindingFunc(0)
,fAfterBindingTag(0)
{ 
#ifdef WIN32
  WSADATA WSAData;
  int initwinsock = 0;
  if (initwinsock = ::WSAStartup(MAKEWORD(2, 0), &WSAData)) {
    m_out << "Lib::InternetSocket::InternetSocket :"
          << " starting sockets failed." 
          << std::endl;
  }
#endif
}

//private:
InternetSocket::InternetSocket(const InternetSocket& aFrom) 
:BaseSocket(aFrom)
,fHost(aFrom.fHost)
,fPort(aFrom.fPort)
,fAfterBindingFunc(aFrom.fAfterBindingFunc)
,fAfterBindingTag(aFrom.fAfterBindingTag)
{}

InternetSocket::~InternetSocket() {
#ifdef WIN32
  ::WSACleanup();
#endif
}

//private
InternetSocket& InternetSocket::operator=(const InternetSocket& aFrom){ return *this;}

void InternetSocket::setPort(int aPort) {fPort = aPort;}

int InternetSocket::port() const {return fPort;}

void InternetSocket::setHost(const std::string& aHost) {fHost = aHost;}
 
void InternetSocket::setAfterBindingFunc(AfterBindingFunc aFunc,void* aTag){
  fAfterBindingFunc = aFunc;
  fAfterBindingTag = aTag;
}

bool InternetSocket::accept(){
  if(fSocket!=(-1)) return true; //Done.

  int fd_socket_waiting;

  int num_binding_trial = 0;
  while(true) {
    // controlling retry
    num_binding_trial++ ;
    if( ( num_binding_trial % BINDING_TRIAL_INTERVAL ) == 0 ) { 
      m_out << "Lib::InternetSocket::accept :"
            << " trying to bind soket. Please wait." 
            << std::endl;
    }
    if( num_binding_trial > MAX_BINDING_TRIAL ) { 
      m_out << "Lib::InternetSocket::accept :"
            << " cannot use port " << fPort << "." 
            << std::endl;
      return false;
    }

    // make socket
    fd_socket_waiting = ::socket(AF_INET,SOCK_STREAM,0);
    if( fd_socket_waiting < 0 ) { 
      m_out << "Lib::InternetSocket::accept :"
            << " socket : " << std::string(strerror(errno))
            << std::endl;
      return false;
    }

    // Force reuse of the server socket (i.e. do not wait for the time
    // out to pass).
    int val = 1;
    if(::setsockopt(fd_socket_waiting,
                    SOL_SOCKET,SO_REUSEADDR,(char*)&val,sizeof(val)) == -1) {
      m_out << "Lib::InternetSocket::accept :"
            << " setsockopt : " << std::string(strerror(errno))
            << std::endl;
      return false;
    }

    // get server name
    char server_hostname[256];
    if( GET_HOSTNAME( server_hostname, sizeof( server_hostname ) ) < 0 ) { 
      m_out << "Lib::InternetSocket::accept :"
            << " gethostname : " << std::string(strerror(errno))
            << std::endl;
      return false;
    }
    if(fVerboseLevel) {
      m_out << "Lib::InternetSocket::accept :"
            << " hostname : \"" << server_hostname << "\"."
            << std::endl;
    }

    // get IP address of server from its name
    hostent* server_host_p;
    if( !(server_host_p = ::gethostbyname( server_hostname ) ) ) { 
      m_out << "Lib::InternetSocket::accept :"
            << " gethostbyname : " << std::string(strerror(errno))
            << std::endl;
      return false;
    }

    // binding
    sockaddr_in server_address;
    ::memset((char*)&server_address,'\0',sizeof(server_address)) ; 

    // clear server_address
    server_address.sin_family = AF_INET ;
    server_address.sin_port = htons(fPort);
    ::memcpy((char*)(&server_address.sin_addr), 
             (char*)(server_host_p->h_addr), 
             server_host_p->h_length);

    if(::bind(fd_socket_waiting,
              (struct sockaddr*)(&server_address),sizeof(server_address))<0 ) {
      if(fVerboseLevel) {
        m_out << "Lib::InternetSocket::accept :"
              << " bind : " << std::string(strerror(errno))
              << " : retry..." 
              << std::endl;
      }
      ::close(fd_socket_waiting);

    } else {
      if(fVerboseLevel) {
        m_out << "Lib::InternetSocket::accept :"
              << " socket is bound on port " << fPort << "." 
              << std::endl;
      }
      break ;
    }
  }

  if(fAfterBindingFunc)  {
    if(!fAfterBindingFunc(fAfterBindingTag)) return false;
  }

  // listen (wait connection)
  if(::listen(fd_socket_waiting,1) == -1 ) { 
    m_out << "Lib::InternetSocket::accept :"
          << " listen : " << std::string(strerror(errno))
          << std::endl;
    return false;
  }

  // accept connection
  sockaddr_in client_address;
  CLIENT_ADDR_LENGTH client_addr_length = sizeof(client_address);
  fSocket = ::accept(fd_socket_waiting, 
                     (struct sockaddr*)(&client_address), 
                     &client_addr_length ) ;
  ::close(fd_socket_waiting);
  if(fSocket<0) { 
    m_out << "Lib::InternetSocket::accept :"
          << " accept : " << std::string(strerror(errno))
          << std::endl;
    fSocket = -1;
    return false;
  }

  if(fVerboseLevel) {
    m_out << "Lib::InternetSocket::accept :"
          << " accept ok."
         << std::endl;
  }

  return true;
}

bool InternetSocket::connect(){
  if(fSocket!=(-1)) return true; //Done.

  // make socket :
  fSocket = ::socket(AF_INET, SOCK_STREAM,0);
  if( fSocket < 0 ) { 
    m_out << "Lib::InternetSocket::accept :"
          << " socket : " << std::string(strerror(errno))
          << std::endl;
    fSocket = -1;
    return false;
  }

  // get IP address of server from its name :
  hostent* server_host_p = ::gethostbyname(fHost.c_str()) ;
  if(!server_host_p) {
    m_out << "Lib::InternetSocket::accept :"
          << " gethostbyname : " << std::string(strerror(errno))
          << std::endl;
    ::close(fSocket);
    fSocket = -1;
    return false;
  }

  // connection and binding 
  sockaddr_in server_address;
  ::memset((char*)&server_address,'\0',sizeof(server_address)); 
  // clear server_address
  server_address.sin_family = AF_INET ;
  server_address.sin_port = htons(fPort);
  ::memcpy((char*)(&server_address.sin_addr), 
           (char*)(server_host_p->h_addr), 
           server_host_p->h_length);

  int connection_status = -1;
  int num_trial = 0;
  while( (connection_status<0) && (num_trial<=MAX_CONNECT_TRIAL) ) {
    num_trial++ ; 
    connection_status = ::connect 
      (fSocket,(struct sockaddr*)(&server_address),sizeof(server_address));
    if( connection_status  <0 ) {
      if(fVerboseLevel) {
        m_out << "Lib::InternetSocket::connect :"
              << " connect : " << std::string(strerror(errno))
              << " : retry..."
              << std::endl;
      }
    } else {
      if(fVerboseLevel) {
        m_out << "Lib::InternetSocket::connect :"
              << " connect to " << fHost << " ok."
              << std::endl;
      }
      return true;
    }
#ifdef WIN32
    ::Sleep(1000);                
#else
    ::sleep(1);                
#endif
  }

  m_out << "Lib::InternetSocket::connect :"
        << " can't connect."
        << std::endl;

  return false;
}

void InternetSocket::disconnect(){closeSocket();}

}
