%{

// this :
#include <Lib/Processor.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/ProcessorLexer.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/ExpFunc.h>

static Lib::Processor* sParser = 0;

extern int yylex();
int yyparse();
static void yyerror(const char*); 

#define NotFound (-1)

//#define DEBUG

%}

%union {
  unsigned int fIndex;
  unsigned int fInteger;
  double fNumber;
  Lib::Expression* fExpression;
}

%token <fIndex> STRG
%token <fIndex> T_NAME
%token <fInteger> NUMBER_INTEGER
%token <fNumber> NUMBER_REAL 

%token T_PI T_TRUE T_FALSE
%token T_AND T_OR

%type <fExpression> comparisons comparison expression 

%left '-' '+'
%left '/' '*'
%nonassoc UMINUS

%%

result 
: error {
  // To reset lex and yacc in case of error.
} | comparisons {
  if(sParser->findNames()||sParser->findFunctions()) {
    sParser->setResult(0);
  } else {
    sParser->setResult($1);
    sParser->stack().pop_back();
  }
} | expression {
  if(sParser->findNames()||sParser->findFunctions()) {
    sParser->setResult(0);
  } else {
    sParser->setResult($1);
    sParser->stack().pop_back();
  }
};

comparisons 
: '!' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : ! cps" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::NOT,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons '&' '&' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps && cps" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::AND,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons T_AND comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps && cps" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::AND,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons '|' '|' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps || cps" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::OR,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons T_OR comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps || cps" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::OR,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '(' comparisons ')'  {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : (cps)" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparison {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$1);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
};

comparison 
: expression '>' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp > exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::GT,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '>' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp >= exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::GE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '<' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp < exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::LT,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '<' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp <= exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::LE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '=' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp == exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::EQ,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '!' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp != exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::NE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
};

expression 
: expression '+' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp + exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ADD,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '-' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp - exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::SUB,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '*' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp * exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::MUL,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '*' '*' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp ** exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction("pow"),$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '/' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp / exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::DIV,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '-' expression %prec UMINUS { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : - exp" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::MINUS,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : (exp)" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }

} | T_NAME '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_COS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NAME '(' expression ',' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POW" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);    
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3,$5);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NAME '(' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_EXPO" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);    
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3,$5,$7);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NAME '(' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_GAUSS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);    
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3,$5,$7,$9);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NAME '(' expression ',' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL3" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3,$5,$7,$9,$11);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NAME '(' expression ',' expression ',' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL4" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else if(sParser->findFunctions()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);    
    $$ = new Lib::Expression
      (Lib::Expression::FUNC,sParser->findFunction(var),
       $3,$5,$7,$9,$11,$13);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }

} | T_PI { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_PI" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::PI);
    sParser->stack().push_back($$);
  }
} | T_TRUE { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_TRUE" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::BOOL_TRUE);
    sParser->stack().push_back($$);
  }
} | T_FALSE { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_FALSE" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::BOOL_FALSE);
    sParser->stack().push_back($$);
  }
} | NUMBER_INTEGER { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : NUMBER_INTEGER" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression
      (Lib::Expression::UNSIGNED_INTEGER,(unsigned int)$1);
    sParser->stack().push_back($$);
  }
} | NUMBER_REAL { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : NUMBER_REAL" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::REAL,$1);
    sParser->stack().push_back($$);
  }
} | T_NAME {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_NAME" << Lib::endl;
#endif
  if(sParser->findNames()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else if(sParser->findFunctions()) {
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);
    unsigned int index;
    int ivar = NotFound;
    if(sParser->findVariable(var,index)) ivar = int(index);
    $$ = new Lib::Expression(Lib::Expression::NAME,var,ivar);
    sParser->stack().push_back($$);
  }
} | STRG {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : STRG" << Lib::endl;
#endif
  if(sParser->findNames()||sParser->findFunctions()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::STRING,
                             sParser->lexer().variable($1));
    sParser->stack().push_back($$);
  }
};

%%

Lib::Processor::Processor(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
) 
:fString(aString)
,fPrinter(aPrinter)
,fFindNames(false)
,fFindFunctions(false)
,fLexer(aPrinter)
,fResult(0)
,fSuccess(false)
,fFuncs(0)
{
  fFuncs = new ExpFuncs();
}

Lib::Processor::~Processor() { 
  clear();
  delete fFuncs;
}

Lib::Processor::Processor(const Processor& aFrom)
:fString("")
,fPrinter(aFrom.fPrinter)
,fFindNames(false)
,fFindFunctions(false)
,fLexer(aFrom.fLexer)
,fResult(0)
,fSuccess(false)
,fFuncs(0)
{
  fFuncs = new ExpFuncs(*(aFrom.fFuncs));
}

Lib::Processor& Lib::Processor::operator=(const Processor& aFrom) {
  clear();
  fLexer = aFrom.fLexer;
  delete fFuncs;
  fFuncs = new ExpFuncs(*(aFrom.fFuncs));
  // Reset other things :
  fString = "";
  fFindNames = false;
  fFindFunctions = false;
  fResult = 0;
  return *this;
}

void Lib::Processor::setScript(const std::string& aString) {
  clear();
  fString = aString;
  fFindNames = false;
  fFindFunctions = false;
  fResult = 0;
}

void Lib::Processor::clear() {
  for(unsigned int index=0;index<fTmpExps.size();index++) 
    delete fTmpExps[index];
  fTmpExps.clear();
  fNames.clear();
  delete fResult;
  fResult = 0;
}

bool Lib::Processor::findVariables(std::vector<std::string>& aNames) {
  aNames.clear();
  clear();
  if(!fString.size()) return true;
  fLexer.set(fString);
  fFindNames = true;
  fFindFunctions = false;
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
    return false;
  }
  if(fTmpExps.size()) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::findVariables :"
        << " stack anomaly " << (unsigned int)fTmpExps.size()
        << Lib::endl;
  }
  smanip::unique(fNames);
  aNames = fNames;
  return true;
}

bool Lib::Processor::findFunctions(std::vector<std::string>& aNames) {
  aNames.clear();
  clear();
  if(!fString.size()) return true;
  fLexer.set(fString);
  fFindNames = false;
  fFindFunctions = true;
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
    return false;
  }
  if(fTmpExps.size()) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::findVariables :"
        << " stack anomaly " << (unsigned int)fTmpExps.size()
        << Lib::endl;
  }
  smanip::unique(fNames);
  aNames = fNames;
  return true;
}

bool Lib::Processor::compile() {
  std::vector<std::string> vars;
  findVariables(vars);
  return compile(vars);
}

bool Lib::Processor::compile(const std::vector<std::string>& aNames) {
  clear();
  if(!fString.size()) return true;
  fNames = aNames;
  fLexer.set(fString);
  fFindNames = false;
  fFindFunctions = false;
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
    return false;
  }
  if(fTmpExps.size()) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::compile :"
        << " stack anomaly " << (unsigned int)fTmpExps.size()
        << Lib::endl;
  }
  return true;
}

bool Lib::Processor::execute(const std::vector<Lib::Value>& aInput,Lib::Value& aResult,std::string& aError) {
  //printf("debug : Lib::Processor::execute : \"%s\" %lu\n",
  //  fString.c_str(),fResult);
  if(!fString.size()) {
    aResult.setNone();
    return true;
  }
  if(!fResult) {
    aResult.setNone();
    aError = "Lib::Processor::execute : not ready : fReturn = 0.";
    return false;
  }
  if(!fResult->execute(aInput,aError)) {
    aResult.setNone();
    return false;
  }
  if(!fSuccess) {
    aResult.setNone();
    aError = "Lib::Processor::execute : failed.";
    return false;
  }
  aResult = fResult->ret();
  return true;
}

bool Lib::Processor::cut(const std::vector<Lib::Value>& aInput,bool& aResult,std::string& aError) {
  Lib::Value ret; //WARNING : this allocation can take time.
  if(!execute(aInput,ret,aError)) {
    aResult = true;
    return false;
  }
  if(ret.type()==Slash::Core::IValue::BOOL) {
    aResult = ret.get_bool();
    return true;
  }
  if(ret.type()==Slash::Core::IValue::NONE) { 
    //This may happen if fString is empty.
    aResult = true;
    return true;
  }
  aResult = true;
  aError = "Lib::Processor::cut : failed.";
  return false;
}

void Lib::Processor::addName(const std::string& aString) { fNames.push_back(aString);}

Slash::Core::IWriter& Lib::Processor::printer() const { return fPrinter;}

bool Lib::Processor::findNames() const{ return fFindNames;}

bool Lib::Processor::findFunctions() const{ return fFindFunctions;}

const Lib::ProcessorLexer& Lib::Processor::lexer() const{return fLexer;}

bool Lib::Processor::findVariable(const std::string& aString,unsigned int& aIndex){
  if(!smanip::lineIndex(fNames,aString,aIndex)) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::findVariable :" 
        << " " << aString << " not a variable." 
        << Lib::endl;
    return false;
  }
  return true;
}

std::vector<Lib::Expression*>& Lib::Processor::stack(){ return fTmpExps;}

void Lib::Processor::setResult(Lib::Expression* aValue){ fResult = aValue;}

void Lib::Processor::setStatus(bool aValue){fSuccess = aValue;}

Lib::IExpFunc* Lib::Processor::findFunction(const std::string& aString,bool a_warn) const {

  if(!fFuncs) return 0;
  Lib::IExpFunc* func = fFuncs->find(aString);
  if(!func && a_warn) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::findFunction :" 
        << " " << aString << " not a function."
        << Lib::endl;
    return 0;
  }
  return func;
}

void Lib::Processor::addFunction(Lib::IExpFunc* aFunc) {
  if(!fFuncs) return;
  return fFuncs->add(aFunc);
}

static void yyerror(const char* aString) {
  if(sParser) {
    Lib::Out out(sParser->printer());
    out << "Processor : " << aString << Lib::endl;
    sParser->setStatus(false);
  }
}
