
// this :
#include <Lib/Property.h>

#include <Lib/Debug.h>
#include <Lib/smanip.h>

#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
Lib::Property::Property(
)
:fType(NONE)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Debug::increment();
}
//////////////////////////////////////////////////////////////////////////////
Lib::Property::Property(
 const std::string& aName
,Slash::Data::IProperty::Type aType
,int aHead
,const std::string& aLabel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Debug::increment();
  set(aName,aType,aHead,aLabel);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Property::Property(
 const Property& aProperty
)
:IProperty()
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Debug::increment();
  fName = aProperty.fName;
  fType = aProperty.fType;
  fLength = aProperty.fLength;
  fLabel = aProperty.fLabel;
  fHeader = aProperty.fHeader;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Property::~Property(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Debug::decrement();
}
//////////////////////////////////////////////////////////////////////////////
Lib::Property& Lib::Property::operator=(
 const Property& aProperty
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fName = aProperty.fName;
  fType = aProperty.fType;
  fLength = aProperty.fLength;
  fLabel = aProperty.fLabel;
  fHeader = aProperty.fHeader;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Property::set(
 const std::string& aName
,Slash::Data::IProperty::Type aType
,int aHead
,const std::string& aLabel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fName = aName;
  fType = aType;
  switch(fType) {
  case INTEGER:fLength = aHead ? aHead : 8;break;
  case  DOUBLE:fLength = aHead ? aHead : 12;break;
  case  STRING:fLength = aHead ? aHead : 12;break;
  case POINTER:fLength = aHead ? aHead : 12;break;
  case BOOLEAN:fLength = aHead ? aHead : 5;break;
  case   INT64:fLength = aHead ? aHead : 12;break;
  case NONE:
  case VECTOR_DOUBLE:
  case VECTOR_INTEGER:
  case VECTOR_BOOLEAN:
  case VECTOR_STRING:
  default:     fLength = aHead ? aHead : 8;break;
  }
  fLabel = aLabel;
  // Set header string used by "dump" :
  int ltitle = fName.size() + fLabel.size();
  int length = MAXIMUM(ltitle,fLength);
  int lspace = ltitle < length ? length - ltitle : 0;
  std::string spaces(lspace,' ');
  fHeader = spaces;
  fHeader += fName;
  fHeader += fLabel;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Property::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IProperty::Type Lib::Property::type(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fType;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Property::typeString(
) const
//////////////////////////////////////////////////////////////////////////////
// Same logic as Variable::type().
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  switch(fType) {
  case INTEGER:
    return "INTEGER";
  case DOUBLE:
    return "DOUBLE";
  case STRING:
    return "STRING";
  case POINTER:
    return "POINTER";
//case UNSIGNED_LONG:
//  return "UNSIGNED_LONG";
  case BOOLEAN:
    return "BOOLEAN";
//case SHORT:
//  return "SHORT";
//case LONG:
//  return "LONG";
//case FLOAT:
//  return "FLOAT";
//case BYTE:
//  return "BYTE";
//case CHAR:
//  return "CHAR";
  case INT64:
    return "INT64";
  case VECTOR_DOUBLE:
    return "VECTOR_DOUBLE";
  case VECTOR_INTEGER:
    return "VECTOR_INTEGER";
  case VECTOR_BOOLEAN:
    return "VECTOR_BOOLEAN";
  case VECTOR_STRING:
    return "VECTOR_STRING";
  case NONE:
  default:
    return "NONE";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Property::label(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLabel;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Property::header(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHeader;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Property::dress(
 std::string& aString
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  smanip::strip(aString);
  int lstring = aString.length();
  int lheader = fHeader.length();
  if(lstring < lheader) {
    int lspace = lheader - lstring;
    std::string s(lspace,' ');
    s += aString;
    s.resize(lheader);
    aString = s;
  } else {
    aString.resize(lheader);
  }
}
