// this :
#include <Lib/Randoms.h>

#include <Lib/mmanip.h>

#include <stdlib.h>   
#include <math.h>   

double Lib::RandomFlat::shoot(){
  // Shoot random numbers in [0,1] according a flat distribution.
  double value  = (double)::rand();
  value /= (double)RAND_MAX;
  return value;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::RandomGauss::RandomGauss(double aMean,double aStdDev)
:fMean(aMean)
,fStdDev(aStdDev)
{}

double Lib::RandomGauss::shoot(){
  //  Shoot random numbers according a 
  // gaussian distribution of mean 0 and sigma 1.
  double v1,v2,r,fac;
  do {
    v1 = 2.0 * fFlat.shoot() - 1.0;
    v2 = 2.0 * fFlat.shoot() - 1.0;
    r = v1*v1 + v2*v2;
  } while ( r > 1.0 );
  fac = ::sqrt(-2.0*::log(r)/r);
  return (v2 * fac) * fStdDev + fMean;
}

Lib::RandomBreitWigner::RandomBreitWigner(double aMean,double aGamma)
:fMean(aMean)
,fGamma(aGamma)
{}

double Lib::RandomBreitWigner::shoot(){
  double rval = 2.0 * fFlat.shoot() - 1.0;
  double displ = 0.5 * fGamma * ::tan(rval * Lib::mmanip::half_pi());
  return fMean + displ;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::RandomExponential::RandomExponential(double aRate)
:fRate(aRate)
{
  // aRate should be > 0
  // aRate * exp(-aRate * aX)
  if(fRate<0) fRate = 1; //FIXME : throw
}
double Lib::RandomExponential::shoot(){
  double v;
  do {
    v = fFlat.shoot();
  } while(v<=0);
  return -::log(v)/fRate;
}
