// this :
#include <Lib/Time.h>

#ifdef WIN32
#include <winsock.h>
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif

#include <sys/types.h> // For struct timeval, time_t.

//////////////////////////////////////////////////////////////////////////////
Lib::Time::Time(
)
:fSeconds(0)
,fMicroSeconds(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time::Time(
 const Time& aFrom
)
:fSeconds(aFrom.fSeconds)
,fMicroSeconds(aFrom.fMicroSeconds)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time::Time(
 double aSeconds
)
//////////////////////////////////////////////////////////////////////////////
// aSeconds should be >=0.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setSeconds(aSeconds);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time::Time(
 unsigned long aSeconds
,unsigned long aMicroSeconds
)
:fSeconds(aSeconds)
,fMicroSeconds(aMicroSeconds)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time::~Time(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time& Lib::Time::operator=(
 const Time& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSeconds = aFrom.fSeconds;
  fMicroSeconds = aFrom.fMicroSeconds; 
  return *this;  
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Time::copy(
 const Time& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSeconds = aFrom.fSeconds;
  fMicroSeconds = aFrom.fMicroSeconds; 
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Time::seconds(
) const
//////////////////////////////////////////////////////////////////////////////
// Get time in seconds as a double.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return ((double)fSeconds) + ((double)fMicroSeconds)/1000000.0; 
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Time::setSeconds(
 double aSeconds
)
//////////////////////////////////////////////////////////////////////////////
// aSeconds should be >=0.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aSeconds<0) return false;
  fSeconds = (unsigned long)aSeconds; 
  fMicroSeconds = (unsigned long)((aSeconds - fSeconds) * 1000000.0);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Time::subtract(
 const Time& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setSeconds(seconds() - aFrom.seconds());
}
/*
//////////////////////////////////////////////////////////////////////////////
long Lib::Time::elapsed(
 const Time& aFrom
) const
//////////////////////////////////////////////////////////////////////////////
// Return elapsed in microseconds.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fSeconds - aFrom.fSeconds) * 1000000 + 
          fMicroSeconds - aFrom.fMicroSeconds;
}
*/
//////////////////////////////////////////////////////////////////////////////
Lib::Time Lib::Time::now(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  struct timeb tb;
  ::ftime(&tb);
  return Time((unsigned long)tb.time,(unsigned long)(tb.millitm * 1000));
#else
  struct timeval tval;
  ::gettimeofday(&tval,0);
  return Time((unsigned long)tval.tv_sec,(unsigned long)tval.tv_usec);
#endif
}
//////////////////////////////////////////////////////////////////////////////
Lib::Time Lib::Time::elapsed(
 const Time& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Time elapsed = now();
  elapsed.subtract(aFrom);
  return elapsed;
}
