// this :
#include <Lib/UnixSocket.h>

#ifdef WIN32
#else
#include<sys/socket.h>
#include<sys/un.h>
#include <errno.h>
#include <unistd.h>  //close,sleep
#include <string.h> //strerror
#ifdef __alpha
#define CLIENT_ADDR_LENGTH int
#else
#define CLIENT_ADDR_LENGTH socklen_t
#endif
#endif

const int MAX_CONNECT_TRIAL = 10;

namespace Lib {

UnixSocket::UnixSocket(std::ostream& a_out,int aVerboseLevel)
:BaseSocket(a_out,aVerboseLevel)
,fSUN_PATH("LIB_TMP3")
{}

//private:
UnixSocket::UnixSocket(const UnixSocket& aFrom)
:BaseSocket(aFrom)
,fSUN_PATH("LIB_TMP3") 
{}

UnixSocket::~UnixSocket(){
#ifdef WIN32
#else
  ::unlink(fSUN_PATH.c_str()) ;
#endif
}

//private:
UnixSocket& UnixSocket::operator=(const UnixSocket& aFrom){ return *this;}

void UnixSocket::setSunPath(const std::string& sun_path) {  
  fSUN_PATH = sun_path;
}

bool UnixSocket::accept(){
#ifdef WIN32 
  m_out << "Lib::UnixSocket::accept :"
        << " unavailable on WIN32."
        << std::endl;
  return false;
#else
  if(fSocket!=(-1)) return true; //done.

  // clear PIPE
  ::unlink(fSUN_PATH.c_str());

  // make socket
  int fd_socket_waiting = ::socket(AF_UNIX,SOCK_STREAM,0);
  if( fd_socket_waiting < 0 ) { 
    m_out << "Lib::UnixSocket::accept :"
          << " socket : " << std::string(strerror(errno))
          << std::endl;
    return false;
  }

  // binding
  sockaddr_un server_address ;
  ::memset((char*)&server_address,'\0',sizeof(server_address)) ; 

  // clear server_address
  server_address.sun_family = AF_UNIX ;
  ::strcpy(server_address.sun_path,fSUN_PATH.c_str());
  if(::bind(fd_socket_waiting,
            (struct sockaddr*)(&server_address),sizeof(server_address))<0) {
    m_out << "Lib::UnixSocket::accept :"
          << " bind : " << std::string(strerror(errno))
          << std::endl;
    return false;
  }

  // listen (wait connection)
  if(::listen(fd_socket_waiting,1) == -1 ) { 
    m_out << "Lib::UnixSocket::accept :"
          << " listen : " << std::string(strerror(errno))
          << std::endl;
    return false;
  }

  // accept connection
  sockaddr_un client_address ;
  CLIENT_ADDR_LENGTH client_addr_length = sizeof(client_address);
  fSocket = ::accept(fd_socket_waiting, 
                     (struct sockaddr*)(&client_address), 
                     &client_addr_length ) ;
  ::close( fd_socket_waiting );
  if(fSocket<0) { 
    m_out << "Lib::UnixSocket::accept :"
          << " accept : " << std::string(strerror(errno))
          << std::endl;
    fSocket = -1;
    return false;
  }
  return true;
#endif
}

bool UnixSocket::connect(){
#ifdef WIN32 
  m_out << "Lib::UnixSocket::connect :"
        << " unavailable on WIN32."
        << std::endl;
  return false;
#else
  if(fSocket!=(-1)) return true; //Done.

  // make socket
  fSocket = ::socket( AF_UNIX, SOCK_STREAM, 0 );
  if(fSocket<0) { 
    m_out << "Lib::UnixSocket::connect :"
          << " socket : " << std::string(strerror(errno))
          << std::endl;
    fSocket = -1;
    return false;
  }

  // set server address
  struct sockaddr_un server_address ;
  ::memset((char*)&server_address,'\0',sizeof(server_address)) ;
  server_address.sun_family = AF_UNIX ;
  ::strcpy(server_address.sun_path,fSUN_PATH.c_str());

  // connection
  int connection_status = -1;
  int num_trial = 0;
  while( connection_status < 0 && num_trial <= MAX_CONNECT_TRIAL ) {
    num_trial++ ; 
    connection_status = ::connect
      (fSocket,(struct sockaddr*)(&server_address),sizeof(server_address)) ;
    if( connection_status  <0 ) {
      if(fVerboseLevel) {
        m_out << "Lib::UnixSocket::connect :"
              << " connect : " << std::string(strerror(errno))
              << " : retry..."
              << std::endl;
      }
    } else {
      return true;
    }
    ::sleep(1);                
  }

  m_out << "Lib::UnixSocket::connect :"
        << " can't connect."
        << std::endl;

  return false;
#endif
}

void UnixSocket::disconnect(){
  closeSocket();
#ifdef WIN32
#else
  ::unlink(fSUN_PATH.c_str()) ;
#endif
}

}
