// this :
#include <Lib/Upgrader.h>

#include <Slash/Tools/smanip.h>

#include <Lib/fmanip.h>
#include <Lib/smanip.h>
#include <Lib/dirmanip.h>

Lib::Upgrader::Upgrader(
 std::ostream& aOut
,int aVerboseLevel
,const std::string& aDataFile
,const std::string& aFilter
)
:fOut(aOut)
,fVerboseLevel(aVerboseLevel)
,fFilter(aFilter)
{
  if(fVerboseLevel) {
    fOut << "Lib::Upgrader::Upgrader :"
         << " upgrade file is " << Slash::sout(aDataFile) 
         << std::endl;
  }
  if(!fmanip::read(aDataFile,fOrders)) {
    fOut << "Lib::Upgrader::Upgrader :"
         << " can't read upgrade file." 
         << std::endl;
  }

  // include files :
  bool hasIncs = true;
  while(hasIncs) {
    hasIncs = false;
    std::vector<std::string> orders;
    unsigned int ordern = fOrders.size();
    for(unsigned int index=0;index<ordern;index++) {
      std::vector<std::string> words = smanip::words(fOrders[index],"\\n");
      if( (words.size()==2) && (words[0]=="include")) {
        const std::string& file = words[1];
        if(fVerboseLevel) {
          fOut << "Lib::Upgrader::upgradeFile :"
               << " include " << Slash::sout(file) 
               << std::endl;
        }
        std::vector<std::string> incs;
        if(!fmanip::read(file,incs)) {
          fOut << "Lib::Upgrader::Upgrader :"
               << " can't read include upgrade file " << Slash::sout(file) << "."
               << std::endl;
          fOrders.clear();
          return;
        }
        unsigned int incn = incs.size();
        for(unsigned int inci=0;inci<incn;inci++) {
          hasIncs = true;
          orders.push_back(incs[inci]);
        }
      } else {
        orders.push_back(fOrders[index]);
      }
    }
    fOrders = orders;
  }
}

Lib::Upgrader::Upgrader(const Upgrader& aUpgrader)
:fOut(aUpgrader.fOut)
,fVerboseLevel(aUpgrader.fVerboseLevel)
,fOrders(aUpgrader.fOrders)
,fFilter(aUpgrader.fFilter)
{}

Lib::Upgrader::~Upgrader(){}

Lib::Upgrader& Lib::Upgrader::operator =(const Lib::Upgrader& aUpgrader){
  fVerboseLevel = aUpgrader.fVerboseLevel;  
  fOrders = aUpgrader.fOrders;
  fFilter = aUpgrader.fFilter;
  return *this;
}

int Lib::Upgrader::verboseLevel() const {return fVerboseLevel;}

bool Lib::Upgrader::upgrade(const std::string& aString) {
  if(!fOrders.size()) return true; //Nothing to do.
  bool value;
  if(!dirmanip::isDirectory(aString,value)) return false;
  if(value) return upgradeDirectory(aString);
  else return upgradeFile(aString);
}

bool Lib::Upgrader::upgradeFile(const std::string& aFile) {
  if(fVerboseLevel) {
    fOut << "Lib::Upgrader::upgradeFile :"
        << " upgrade file " << Slash::sout(aFile) << "..."
        << std::endl;
  }
  if(fFilter.size()) {
    if(!Slash::match(aFile,fFilter)) return true;
  }
  unsigned int ordern = fOrders.size();
  for(unsigned int index=0;index<ordern;index++) {
    std::vector<std::string> words =
      smanip::words(fOrders[index],"\\n",true); //true = take empty.
    if(fOrders[index].size()<=0) {
    } else if( (fOrders[index].size()>=1) && (fOrders[index][0]=='#') ) {
    } else if( (words.size()==3) && (words[0]=="replace")) {
      if(fVerboseLevel) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " replace " << Slash::sout(words[1]) 
            << " by " << Slash::sout(words[2]) 
            << std::endl;
      }
      std::vector<std::string> changes;
      if(!fmanip::replace(aFile,words[1],words[2],changes)) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Slash::sout(aFile) << " fails."
            << std::endl;
        return false;
      }
      unsigned int changen = changes.size();
      for(unsigned int count=0;count<changen;count+=2) {
        fOut << ">>>>>>>>>>>>> In file " << Slash::sout(aFile) 
            << ", line changed : " << std::endl;
        fOut << changes[count] << std::endl;
        fOut << changes[count+1] << std::endl;
      }
    } else if( (words.size()==2) && (words[0]=="remove_line")) {
      if(fVerboseLevel) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " remove_line " << Slash::sout(words[1]) 
            << std::endl;
      }
      if(!fmanip::removeLines(aFile,words[1])) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Slash::sout(aFile) << " fails."
            << std::endl;
        return false;
      }
    } else if( (words.size()==3) && (words[0]=="if_rm_eol")) {
      if(fVerboseLevel) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " if_rm_eol " << Slash::sout(words[1]) 
            << std::endl;
      }
      std::vector<std::string> changes;
      if(!fmanip::if_rm_eol(aFile,words[1],words[2],changes)) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Slash::sout(aFile) << " fails."
            << std::endl;
        return false;
      }
      unsigned int changen = changes.size();
      for(unsigned int count=0;count<changen;count+=2) {
        fOut << ">>>>>>>>>>>>> In file " << Slash::sout(aFile) 
            << ", line changed : " << std::endl;
        fOut << changes[count] << std::endl;
        fOut << changes[count+1] << std::endl;
      }
    } else if( (words.size()==3) && (words[0]=="warn")) {
      std::vector<std::string> found;
      if(!fmanip::found(aFile,words[1],found)) {
        fOut << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Slash::sout(aFile) << " fails."
            << std::endl;
        return false;
      }
      unsigned int foundn = found.size();
      for(unsigned int count=0;count<foundn;count++) {
        fOut << ">>>>>>>>>>>>> In file " << Slash::sout(aFile) 
            << ", have to upgrade by hand line :" 
            << std::endl;
        fOut << found[count] << std::endl;
        fOut << "Action being :" << std::endl;
        fOut << words[2] << std::endl;
      }
    } else {
      fOut << "Lib::Upgrader::upgradeFile :"
          << " bad formed or unknown request " << Slash::sout(fOrders[index])
          << std::endl;
    }
  }
  return true;
}

namespace Lib {

class UpgradeVisitor : public Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& aString,bool& aProcess) {
    aProcess = true;
    std::string p,n,s;
    smanip::pathNameSuffix(aString,p,n,s);
    if((n=="CVS")||(n==".svn")) aProcess = false;
    return true;
  }
  virtual bool file(const std::string& aString) {
    return fUpgrader.upgradeFile(aString);
  }
public:
  UpgradeVisitor(Upgrader& aUpgrader):fUpgrader(aUpgrader){}
  virtual ~UpgradeVisitor() {};
private:
  Upgrader& fUpgrader;
};

}

bool Lib::Upgrader::upgradeDirectory(const std::string& aPath) {
  if(!fOrders.size()) return true; //Nothing to do.
  if(fVerboseLevel) {
    fOut << "Lib::Upgrader::upgradeDirectory :"
        << " upgrade directory " << Slash::sout(aPath) << "..."
        << std::endl;
  }
  Lib::Directory tree;
  tree.setPath(aPath);
  if(!dirmanip::content(tree)) {
    fOut << "Lib::Upgrader::upgradeDirectory :"
        << "Can't get files of " << Slash::sout(aPath) << std::endl;
    return false;
  }
  UpgradeVisitor visitor(*this);
  return tree.visit(visitor);
}
