// this :
#include <Lib/calc.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/Processor.h>
#include <Lib/valuemanip.h>

//////////////////////////////////////////////////////////////////////////////
bool Lib::calc(
 Slash::Core::IWriter& aPrinter
,const std::string& aFormula
,const std::vector<double>& aVals
,double& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Processor proc(aFormula,aPrinter);
  std::vector<std::string> vars;
  if(!proc.findVariables(vars)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " can't find variables in " << Lib::sout(aFormula) << "."
        << Lib::endl;
    return false;
  }
  //Lib::smanip::write(vars,printer);
  if(!proc.compile(vars)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " can't compile " << Lib::sout(aFormula) << "."
        << Lib::endl;
    return false;
  }
  unsigned int number = vars.size();
  if(aVals.size()!=number) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " not enough values. " 
        << (int)number << " required."
        << Lib::endl;
    return false;
  }
  std::vector<Lib::Value> vals;
  vals.resize(number,Lib::Value(0.));

 {std::vector<double>::const_iterator it1 = aVals.begin();
  std::vector<Lib::Value>::iterator it2 = vals.begin();
  for(;it1!=aVals.end();++it1,++it2) (*it2).set(*it1);}

  Lib::Value res;
  std::string serror;
  if(!proc.execute(vals,res,serror)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " execution failure of " << Lib::sout(aFormula)
        << ". Error message " << Lib::sout(serror)
        << Lib::endl;
    return false;
  }
  return Lib::valuemanip::to_double(res,aValue);
}

//////////////////////////////////////////////////////////////////////////////
bool Lib::calc(
 Slash::Core::IWriter& aPrinter
,const std::string& aFormula
,const std::vector<std::string>& aVals
,std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Processor proc(aFormula,aPrinter);
  std::vector<std::string> vars;
  if(!proc.findVariables(vars)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " can't find variables in " << Lib::sout(aFormula) << "."
        << Lib::endl;
    return false;
  }
  //Lib::smanip::write(vars,printer);
  if(!proc.compile(vars)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " can't compile " << Lib::sout(aFormula) << "."
        << Lib::endl;
    return false;
  }
  unsigned int number = vars.size();
  if(aVals.size()!=number) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " not enough values. " 
        << (int)number << " required."
        << Lib::endl;
    return false;
  }
  std::vector<Lib::Value> vals;
  vals.resize(number,Lib::Value(0.));

 {std::vector<std::string>::const_iterator it1 = aVals.begin();
  std::vector<Lib::Value>::iterator it2 = vals.begin();
  for(;it1!=aVals.end();++it1,++it2) {
    double d;
    if(!Lib::smanip::todouble(*it1,d)) {
      Lib::Out out(aPrinter);
      out << "Lib::calc :"
          << " argument " << Lib::sout(*it1) << " not a double."
          << Lib::endl;
      return false;
    }
    (*it2).set(d);
  }}

  Lib::Value res;
  std::string serror;
  if(!proc.execute(vals,res,serror)) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " exection failure of " << Lib::sout(aFormula)
        << ". Error message " << Lib::sout(serror)
        << Lib::endl;
    return false;
  }

  if(res.type()!=Slash::Core::IValue::DOUBLE) {
    Lib::Out out(aPrinter);
    out << "Lib::calc :"
        << " result not a double."
        << Lib::endl;
    return false;
  }

  aValue = Lib::valuemanip::toString(res);
  return true;
}

