
#include <Lib/Columns.h>

#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>
#include <Lib/valuemanip.h>

void printCols(const std::list<Lib::Value>& aVars,
               Slash::Core::IWriter& aPrinter) {
  std::list<Lib::Value>::const_iterator it;
  for(it=aVars.begin();it!=aVars.end();++it) {
    Lib::Out out(aPrinter);
    if((*it).type()==Lib::Value::VOID_STAR) {
      out << " ---- ITuple : " << (*it).label() << " : begin " << Lib::endl;
      std::list<Lib::Value>* vars = 
        (std::list<Lib::Value>*)(*it).get_void_star();
      printCols(*vars,aPrinter);
      out << " ---- ITuple : " << (*it).label() << " : end " << Lib::endl;
    } else {
      //(*it).print(aPrinter);
      out << (*it).stype() << " : " 
          << Lib::valuemanip::toString(*it)
          << Lib::endl;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::Debug::checkByClass(true);

 {Lib::Arguments args(aArgc,aArgv);

  int verbose = 0;
  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose.empty()) verbose = 1;
  }


 {Lib::Columns proc
    ("int i,int j = 2, float f = -3.2,double d,string s = \"xxx\"",
     printer,verbose);
  out << "Lib_columns : " 
      << " parse : " << proc.script() << Lib::endl;
  if(!proc.findVariables()) {
    out << "Lib_columns : " 
        << " parse failed."<< Lib::endl;
  }
  std::list<Lib::Value> vars = proc.result();
  printCols(vars,printer);
  Lib::Columns::deleteColumns(vars);}

 {Lib::Columns proc
    ("int[] i, float[] f,double[] d,string[] s",
     printer,verbose);
  out << "Lib_columns : " 
      << " parse : " << proc.script() << Lib::endl;
  if(!proc.findVariables()) {
    out << "Lib_columns : " 
        << " parse failed."<< Lib::endl;
  }
  std::list<Lib::Value> vars = proc.result();
  printCols(vars,printer);
  Lib::Columns::deleteColumns(vars);}

  // AIDA/TestTuple.cpp :
 {Lib::Columns proc(
    //    "ITuple col11_ITuple={ double px = 2,double py = 3., double pz = 0., ITuple xyz = {int x,int y,int z}, float ipx, float ipy, float ipz, int track=-1}",

    "\
int col1_Int= 1\
,short col2_Short=2\
,float col4_Float=0.3\
,double col5_Double=-0.5\
,boolean col6_Boolean=true\
,byte col7_Byte=23\
,char col8_Character=f\
,java.lang.String col9_string=de\
,java.lang.String col10_Object\
,ITuple col11_ITuple={ \
  double px = .2, py = 3., pz = 0., \
  ITuple xyz = {int x,y,z}, \
  float ipx, ipy,ipz, int track=-1\
}\
",
     printer,verbose);
  out << "Lib_columns : " 
      << " parse : " << proc.script() << Lib::endl;
  if(!proc.findVariables()) {
    out << "Lib_columns : " 
        << " parse failed."<< Lib::endl;
  }
  std::list<Lib::Value> vars = proc.result();
  printCols(vars,printer);
  Lib::Columns::deleteColumns(vars);}

 {Lib::Columns proc
    ("boolean bool_col=false,byte byte_col=0,short short_col=0,int int_col=0,long long_col=0,float float_col=0,double double_col=0",
     printer,verbose);
  out << "Lib_columns : " 
      << " parse : " << proc.script() << Lib::endl;
  if(!proc.findVariables()) {
    out << "Lib_columns : " 
        << " parse failed."<< Lib::endl;
  }
  std::list<Lib::Value> vars = proc.result();
  printCols(vars,printer);
  Lib::Columns::deleteColumns(vars);}

 }

  Lib::Debug::balance(printer);
 
  return 0;
}
