
// To test the function manager :

#include <Lib/Printer.h>
#include <Lib/Session.h>
#include <Lib/LibraryManager.h>
#include <Lib/FunctionManager.h>
#include <Lib/Debug.h>
#include <Lib/Arguments.h>
#include <Lib/Out.h>

//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;

 {Lib::Arguments args(aArgc,aArgv);

  int verbose = 0;
  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose.empty()) verbose = 1;
  }

  Lib::Session session(printer);
  session.setVerboseLevel(verbose);
  session.addManager(new Lib::LibraryManager(session));

  Lib::FunctionManager functionManager(session);

  Lib::Out out(printer);
  out << "Lib_functionManager :" 
      << " load $LIBROOT/data/htfun1.f"
      << Lib::endl;
  
  functionManager.load("$LIBROOT/data/htfun1.f");

  Slash::Data::IFunction* htfun1 = functionManager.findFunction("HTFUN1");
  if(htfun1) {
    std::vector<double> args(1);
    args[0] = 0.3;
    double value;
    htfun1->value(args,value);
    out << "Lib_functionManager :" 
        << " HTFUN1(0.3) = " << value << " (1.00193 expected)." 
        << Lib::endl;
  } else {
    out << "Lib_functionManager :" 
        << " function HTFUN1 not found."
        << Lib::endl;
  }

  Slash::Data::IFunction* htfun2 = functionManager.findFunction("HTFUN2");
  if(htfun2) {
    std::vector<double> args(2);
    args[0] = 0.3;
    args[1] = 0.3;
    double value;
    htfun2->value(args,value);
    out << "Lib_functionManager :" 
        << " HTFUN2(0.3,0.3) = " << value << " (100.387 expected)." 
        << Lib::endl;
  } else {
    out << "Lib_functionManager :" 
        << " function HTFUN2 not found."
        << Lib::endl;
  }

  out << "Lib_functionManager :" 
      << " load $LIBROOT/data/chtfun1.c"
      << Lib::endl;
  
  functionManager.load("$LIBROOT/data/chtfun1.c");
  Slash::Data::IFunction* chtfun1 = functionManager.findFunction("chtfun1");
  if(chtfun1) {
    std::vector<double> args(1);
    args[0] = 0.3;
    double value;
    chtfun1->value(args,value);
    out << "Lib_functionManager :" 
        << " chtfun1(0.3) = " << value << " (1.00193 expected)." 
        << Lib::endl;
  } else {
    out << "Lib_functionManager :" 
        << " function chtfun1 not found."
        << Lib::endl;
  }

  Slash::Data::IFunction* CHTFUN2 = functionManager.findFunction("CHTFUN2");
  if(CHTFUN2) {
    std::vector<double> args(2);
    args[0] = 0.3;
    args[1] = 0.3;
    double value;
    CHTFUN2->value(args,value);
    out << "Lib_functionManager :" 
        << " CHTFUN2(0.3,0.3) = " << value << " (100.387 expected)." 
        << Lib::endl;
  } else {
    out << "Lib_functionManager :" 
        << " function CHTFUN2 not found."
        << Lib::endl;
  }


 }

  Lib::Debug::balance(printer);

  return 0;
}
