#ifndef Slash_file_h
#define Slash_file_h

#include <string>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cstring>

namespace Slash {
namespace file {

inline bool exists(const std::string& a_string) {
  FILE* file = ::fopen(a_string.c_str(),"rb");
  if(!file) return false;
  ::fclose(file);
  return true;
}

inline bool write(const std::string& a_file,const std::vector<std::string>& a_text){
  FILE* file = ::fopen(a_file.c_str(),"wb");
  if(!file) return false;
  std::vector<std::string>::const_iterator it;
  for(it=a_text.begin();it!=a_text.end();++it) {
    if(::fprintf(file,"%s\n",(*it).c_str())<0) {
      ::fclose(file);
      return false;
    }
  }
  ::fclose(file);
  return true;
}

inline bool read(const std::string& a_file,std::vector<std::string>& a_text){
  a_text.clear();
  //FIXME : should read without a limited buffer.
  FILE* file = ::fopen(a_file.c_str(),"rb");
  if(!file) return false;
  unsigned int BUFSIZE = 65536;
  char* buffer = new char[BUFSIZE+1];
  if(!buffer) {::fclose(file);return false;}
  while(true) {
    if(::fgets(buffer,BUFSIZE,(FILE*)file)==NULL) break; // EOF.
    int l = ::strlen(buffer);
    //  On Windows, editors when saving binary files,
    // put \r\n at place of \n ; we then look for \r\n.
    if( (l>=2) && (buffer[l-2]=='\r') && (buffer[l-1]=='\n') ) {
      buffer[l-2] = '\0';
    } else if( (l>=1) && (buffer[l-1]=='\n') ) {
      buffer[l-1] = '\0';
    }
    a_text.push_back(buffer);
  }
  delete [] buffer;
  ::fclose(file);
  return true;
}

inline bool read_num(const std::string& a_file,std::vector<std::string>::size_type a_num,std::vector<std::string>& a_text){
  a_text.clear();
  //FIXME : should read without a limited buffer.
  FILE* file = ::fopen(a_file.c_str(),"rb");
  if(!file) return false;
  unsigned int BUFSIZE = 65536;
  char* buffer = new char[BUFSIZE+1];
  if(!buffer) {::fclose(file);return false;}
  while(true) {
    if(::fgets(buffer,BUFSIZE,(FILE*)file)==NULL) break; // EOF.
    int l = ::strlen(buffer);
    //  On Windows, editors when saving binary files,
    // put \r\n at place of \n ; we then look for \r\n.
    if( (l>=2) && (buffer[l-2]=='\r') && (buffer[l-1]=='\n') ) {
      buffer[l-2] = '\0';
    } else if( (l>=1) && (buffer[l-1]=='\n') ) {
      buffer[l-1] = '\0';
    }
    if(a_text.size()<a_num) {
      a_text.push_back(buffer);
    } else {
      break;
    }
  }
  delete [] buffer;
  ::fclose(file);
  return true;
}

// used in Lib/BaseSocket :
inline bool read_bytes(const std::string& a_file,char*& a_buffer,long& a_length){
  // Returned buffer should be deleted with delete [].
  FILE* file = ::fopen(a_file.c_str(),"rb");
  if(!file) {
    a_buffer = 0;
    a_length = 0L;
    return false;
  }
  // Get file size :
  ::fseek(file,0L,SEEK_END);
  long filesize = ::ftell(file);
  if(!filesize) {
    ::fclose(file);
    a_buffer = new char[1];
    a_length = 0L;
    return true; //empty file.
  }
  // Add one byte to be able to add \n if file contain lines.
  a_buffer = new char[filesize+1];
  if(!a_buffer) {
    ::fclose(file);
    a_buffer = 0;
    a_length = 0L;
    return false;
  }
  ::rewind(file);
  size_t nitem = ::fread(a_buffer,(size_t)filesize,(size_t)1,file);
  if(nitem!=1){
    ::fclose(file);
    delete [] a_buffer;
    a_buffer = 0;
    a_length = 0L;
    return false;
  }
  ::fclose(file);
  a_buffer[filesize] = 0;
  a_length = filesize;
  return true;
}  
// used in Lib/BaseSocket :
inline bool write_bytes(const std::string& a_file,const char* a_buffer,size_t a_length){
  FILE* file = ::fopen(a_file.c_str(),"wb");
  if(!file) return false;
  if(!a_length) {
    ::fclose(file);
    return true;
  }
  size_t nitem = ::fwrite((char*)a_buffer,a_length,(size_t)1,file);
  ::fclose(file);
  return (nitem!=1?false:true);
}  

inline bool copy(const std::string& a_from,const std::string& a_to){
  if(a_to==a_from) return true; //done
  std::vector<std::string> text;
  if(!Slash::file::read(a_from,text)) return false;
  return Slash::file::write(a_to,text);
}


inline bool found(const std::string& a_file,const std::string& a_what,std::vector<std::string>& a_found){
  a_found.clear();
  std::vector<std::string> text;
  if(!Slash::file::read(a_file,text)) return false;
  std::vector<std::string>::const_iterator it;
  for(it=text.begin();it!=text.end();++it) {
    if((*it).find(a_what)!=std::string::npos) {
      a_found.push_back(*it);
    }
  }
  return true;
}

inline bool remove(const std::string& a_file) {
  return (::remove(a_file.c_str()) ==0 ? true : false);
}

inline bool rename(const std::string& a_from,const std::string& a_to) {
  //NOTE : a_from must not be a path !
  //       Darwin is ok with a path but not Linux !
  //       For example : 
  //         ::rename("/tmp/tmp01"."x");
  //       return -1 on Linux.
  //       To do the upper then someone must use move. 
  //       But there is no move in the standard lib C !
  return (::rename(a_from.c_str(),a_to.c_str()) == 0 ? true : false);
}

/*
inline bool replace(const std::string& a_file,const std::string& a_old,const std::string& a_new,std::vector<std::string>& a_changes){
  a_changes.clear();
  std::vector<std::string> text;
  if(!read(a_file,text)) return false;
  // Replace :
  bool touched = false;
 {std::vector<std::string>::const_iterator it;
  for(it=text.begin();it!=text.end();++it) {
    std::string old = *it;
    if(smanip::replace(*it,a_old,a_new)) {
      a_changes.push_back(old);
      a_changes.push_back(*it);
      touched = true;
    }
  }}
  if(!touched) return true;
  // Create temporary file :
  //NOTE : have a tmpName in current directory so that the below 
  //       rename works. (Else we should do a move).
  std::string tmp_name;
  if(!tmpname(tmp_name,".","Lib_")) return false;
  FILE* file = ::fopen(tmp_name.c_str(),"wb");
  if(!file) return false;
 {unsigned int textn = text.size();
  for(unsigned index=0;index<textn;index++) {
    if(::fprintf(file,"%s\n",text[index].c_str())<0) {
      ::fclose(file);
      remove(tmp_name);
      return false;
    }
  }}
  ::fclose(file);
  if(!remove(a_file)) return false;
  if(!rename(tmp_name,a_file)) return false;
  return true;
}
*/

enum e_chmod_who { USER, GROUP, OTHER, ALL };
enum e_chmod_operation { ADD, SUBTRACT };
enum e_chmod_what { READ, WRITE, EXECUTE };

inline bool chmod(const std::string& a_file,e_chmod_who a_who,e_chmod_operation a_op,e_chmod_what a_what){
#ifdef WIN32
  //FIXME
  return false;
#else
  std::string cmd = "chmod ";
  switch(a_who){
  case USER: cmd += "u";break;
  case GROUP: cmd += "g";break;
  case OTHER: cmd += "o";break;
  case ALL: cmd += "a";break;
  //default:break;
  }
  switch(a_op){
  case ADD: cmd += "+";break;
  case SUBTRACT: cmd += "-";break;
  //default:break;
  }
  switch(a_what){
  case READ: cmd += "r";break;
  case WRITE: cmd += "w";break;
  case EXECUTE: cmd += "x";break;
  //default:break;
  }
  cmd += " "+a_file;
  int ret = ::system(cmd.c_str());
  return (ret==(-1)?false:true);
#endif  
}

}}

#endif
