#ifndef amanip_smanip_h
#define amanip_smanip_h

#include <string>
#include <sstream>
#include <vector>
#include <cstring>

#include "typedefs.h"

namespace amanip {

inline std::string sout(const std::string& a_string) {
  return std::string("\"")+a_string+"\"";
}

inline std::string suffix(const std::string& a_string,bool a_back = true) {
  // If a_string = dir0/dir1/dir2/dir3/name.xxx
  //   return xxx
  std::string::size_type pos = a_back?a_string.rfind('.'):a_string.find('.');
  if(pos==std::string::npos) return "";
  pos++;
  return a_string.substr(pos,a_string.size()-pos);
}

inline std::string nosuffix(const std::string& a_string,bool a_back = true){
  // If a_string = dir0/dir1/dir2/dir3/name.xxx
  //   return name
  // Start searching after the last / (or last \ for Windows).
  std::string::size_type pos = a_string.rfind('/');
  if(pos==std::string::npos) pos = a_string.rfind('\\');
  if(pos==std::string::npos) pos = 0;
  else pos++;
  std::string s = a_string.substr(pos,a_string.size()-pos);
  std::string::size_type dot_pos = a_back?s.rfind('.'):s.find('.');
  if(dot_pos==std::string::npos) return s;
  return s.substr(0,dot_pos);
}

//////////////////////////////////////////////////////////////////////////////
/// to ///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
template <class T>
inline bool to(const std::string& a_s,T& a_v) {
  if(a_s.empty()) {a_v = T();return false;} //for INLIB_STL istringstream.
  std::istringstream strm(a_s.c_str());
  strm >> a_v;
  if(strm.fail()) {a_v = T();return false;}
  return strm.eof();
}

template <class T>
inline std::string to(const T& a_v) {
  std::ostringstream strm;
  strm << a_v;
  return strm.str();
}

inline bool to(const std::string& a_string,bool& a_value){
  if(  (a_string=="1")
     ||(a_string=="true")||(a_string=="TRUE")||(a_string=="True")
     ||(a_string=="yes")||(a_string=="YES")||(a_string=="Yes")
     ||(a_string=="on")||(a_string=="ON")||(a_string=="On")
     ){
    a_value = true;
    return true;
  } else if((a_string=="0")
          ||(a_string=="false")||(a_string=="FALSE")||(a_string=="False")
          ||(a_string=="no")||(a_string=="NO")||(a_string=="No")
          ||(a_string=="off")||(a_string=="OFF")||(a_string=="Off")
          ){
    a_value = false;
    return true;
  } else {
    a_value = false;
    return false;
  }
}

inline std::string to(bool a_value){return a_value?"true":"false";}

inline std::string d2s(double a_value){
  std::ostringstream strm;
  strm.precision(25);
  strm << a_value;
  return strm.str();
}

//do it with to<amanip::int64>
//#include <cstring> //sprintf
//inline std::string to(amanip::int64 a_value){
//  char s[64];
//  ::sprintf(s,AMANIP_INT64_FORMAT,a_value);
//  return std::string(s);
//}

inline std::string tostring(unsigned int a_linen,const char* a_lines[]) {
  std::string s;
  for(unsigned int index=0;index<a_linen;index++) {
    s += std::string(a_lines[index]);
    s += "\n";
  }
  return s;
}

//////////////////////////////////////////////////////////////////////////////
/// words ////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

inline void words(const std::string& a_string,const std::string& a_sep,bool a_take_empty,std::vector<std::string>& a_words){
  //  If a_sep is for exa "|" and for "xxx||xxx" :
  //  - a_take_empty false : {"xxx","xxx"} will be created 
  //    (and NOT {"xxx","","xxx"}).
  //  - a_take_empty true : {"xxx","","xxx"} will be created.
  a_words.clear();
  if(a_string.empty()) return;
  std::string::size_type lim = (a_take_empty?0:1);
  if(a_sep.empty()) {
    a_words.push_back(a_string);
  } else {
    std::string::size_type l = a_string.length();
    std::string::size_type llimiter = a_sep.length();
    std::string::size_type pos = 0;
    while(true) {
      std::string::size_type index = a_string.find(a_sep,pos);
      if(index==std::string::npos){ // Last word.
        if((l-pos)>=lim) a_words.push_back(a_string.substr(pos,l-pos));
        break;
      } else {
        //     abcxxxef
        //     0  3  67
        if((index-pos)>=lim) a_words.push_back(a_string.substr(pos,index-pos));
        pos = index + llimiter;
      }
    }
  }
}

inline bool match(const std::string& a_string,const std::string& a_pattern){
  std::string::size_type lpattern = a_pattern.length();
  std::string::size_type lstring  = a_string.length();
  if ((lpattern==0)&&(lstring==0)) return true;
  if ((lpattern==0)&&(lstring!=0)) return true;
  if ((lpattern!=0)&&(lstring==0)) return false;

  if(a_pattern=="*") return true; // pattern is * :

 {bool some_star = false;
  for(unsigned int count=0;count<lpattern;count++) {
    if(a_pattern[count]=='*') {some_star = true;break;}
  }
  if(!some_star) {  // no wildcard :
    return (a_pattern==a_string ? true : false );
  }}

  // complex pattern :
  std::vector<std::string> ws;
  words(a_pattern,"*",false,ws);  
  if(ws.empty()) return true; // only wildcards :

  // tricky case :
  unsigned int wnumber = ws.size();
  char* token = (char*)a_string.c_str();
 {for(unsigned int count=0;count<wnumber;count++) { 
    unsigned int lword = ws[count].size(); 
    if(!lword) continue;//should never happen !
    if(count==0) {
      if(a_pattern[0]!='*') {
        // Begin of pattern (ws[0]) and a_string must match :
        if(::strncmp(token,ws[count].c_str(),lword)) return false;
        token = token + lword;
        continue;
      }
    }
    char* pos = ::strstr(token,ws[count].c_str());
    if(!pos) return false;
    if((count==(wnumber-1)) && (a_pattern[lpattern-1]!='*') ) { // Last word.
      // Compare last word and end of a_string.
      if(::strcmp(a_string.c_str()+lstring-lword,ws[count].c_str())) 
        return false;
      break;
    } else {
      token = pos + lword;
    }
  }}
  return true;
}

inline void tolowercase(std::string& a_string){
  for(std::string::iterator it=a_string.begin();it!=a_string.end();++it) {
    char c = *it;
    *it = ((c) >= 'A' && (c) <= 'Z' ?  c - 'A' + 'a' : c);
  }
}

inline void touppercase(std::string& a_string){
  for(std::string::iterator it=a_string.begin();it!=a_string.end();++it) {
    char c = *it;
    *it = ((c) >= 'a' && (c) <= 'z' ?  c - 'a' + 'A' : c);
  }
}

inline bool todoubles_one(const std::string& a_string,std::vector<double>& a_vals){
  std::string::size_type pos = a_string.find('*');
  if(pos==std::string::npos){
    double value;
    if(!to<double>(a_string,value)) return false;
    a_vals.push_back(value);
  } else {
    //3*2.4
    //01234
    std::string sn = a_string.substr(0,pos);
    std::string sv = a_string.substr(pos+1,a_string.size()-(pos+1));
    int multiplier;
    if(!to<int>(sn,multiplier)) return false;
    double value;
    if(!to<double>(sv,value)) return false;
    for(int index=0;index<multiplier;index++) a_vals.push_back(value);
  }
  return true;
}

inline bool todoubles(const std::string& a_string,const char a_sep,std::vector<double>& a_vals){
  // Support PAW syntax : 3*2.4 is understood as three times 2.4.
  a_vals.clear();
  if(a_string.empty()) return true;
  std::string::size_type lim = 1;
  std::string::size_type l = a_string.length();
  std::string::size_type pos = 0;
  while(true) {
    std::string::size_type index = a_string.find(a_sep,pos);
    if(index==std::string::npos){ // Last word.
      if((l-pos)>=lim) {
        if(!todoubles_one(a_string.substr(pos,l-pos),a_vals)) {
          a_vals.clear();
          return false;
        }
      }
      break;
    } else {
      //     abcxxxef
      //     0  3  67
      if((index-pos)>=lim) {
        if(!todoubles_one(a_string.substr(pos,index-pos),a_vals)) {
          a_vals.clear();
          return false;
        }
      }
      pos = index + 1;
    }
  }
  return true;
}

inline bool remove(std::vector<std::string>& a_strings,const std::string& a_string){
  bool found_some = false;
  std::vector<std::string>::iterator it;
  for(it=a_strings.begin();it!=a_strings.end();) {
    if(*it==a_string) {
      it = a_strings.erase(it);
      found_some = true;
    } else {
      ++it;
    }
  }
  return found_some;
}

inline std::string to_string(const std::vector<std::string>& a_strings,const std::string& a_sep){
  unsigned int number = a_strings.size();
  if(number<=0) return "";
  std::string result;
  number--;
  for(unsigned int index=0;index<number;index++) {
    result += a_strings[index];
    result += a_sep;
  }
  result += a_strings[number];
  return result;
}

//////////////////////////////////////////////////////////////////////////////
/// justify //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
enum what { leading, trailing, both };

inline void strip(std::string& a_string,what a_type = both,char a_char = ' '){
  std::string::size_type l = a_string.length();;
  if(l==0) return;

  switch ( a_type ) {
  case leading:{
    std::string::size_type i;
    char* pos = (char*)a_string.c_str();
    for(i=0;i<l;i++,pos++) {
      if(*pos!=a_char) {
        a_string = a_string.substr(i,l-i);
        return;
      }
    }
    }break;
  case trailing:{
    std::string::size_type i;
    char* pos = (char*)a_string.c_str();
    pos += (l-1);
    for(i=l-1;;i--,pos--) {
      if(*pos!=a_char) {
        a_string = a_string.substr(0,i+1);
        return;
      }
    }
    }break;
  case both:
    strip(a_string,leading,a_char);
    strip(a_string,trailing,a_char);
    break;
  //default:break;
  }
}

enum side { left, right, middle };

inline bool justify(std::string& a_string,unsigned int a_size,side a_side = left){
  // a_size is the final string length.
  strip(a_string);
  if(a_size<=a_string.size()) {
    a_string.resize(a_size);
    return false;
  } else {
    if(a_side==left) {
      a_string = a_string + std::string(a_size-a_string.size(),' ');
    } else if(a_side==right) {
      a_string = std::string(a_size-a_string.size(),' ') + a_string;
    } else if(a_side==middle) {
      int l = a_size - a_string.size();
      int h = l/2;
      if(h*2==l) { //even number of spaces :
        a_string = std::string(h,' ') + a_string + std::string(h,' ');
      } else { // odd number of spaces :
        a_string = std::string(h,' ') + a_string + std::string(h+1,' ');
      }
    }
    return false;
  }
}

inline std::string sjust(const std::string& a_string,unsigned int a_size,side a_side = left){
  std::string s(a_string); 
  justify(s,a_size,a_side);
  return s;
}

template <class aT>
inline std::string sjust(aT a_value,unsigned int a_size,side a_side = left) {
  std::string s = to<aT>(a_value);
  justify(s,a_size,a_side);
  return s;
}

}

#endif
