// this :
#include <obuild/Shell.h>

// Lib :
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>
#include <Lib/FileWriter.h>
#include <Lib/System.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/Time.h>

#include <cstring> //strlen
#include <cstdio> //strlen

#define BUFSIZE 65536

obuild::Shell::Shell(Slash::Core::IWriter& aPrinter)
:fPrinter(aPrinter)
,fShowTime(false)
,fOut(0)
{}

obuild::Shell::~Shell(){
  closeOut();
  removeOut();
}

bool obuild::Shell::setup(
 const std::vector<std::string>& aReps
,const std::vector<std::string>& aPVs
){
  fRepositories = aReps;
  fPackages = aPVs;
  return true;
}

bool obuild::Shell::setupFromFile(const std::string& aFile){
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(aFile,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::setFromFile :"
        << " can't read " << Lib::sout(aFile)
        << Lib::endl;
    fPackages.clear();
    fRepositories.clear();
    return false;
  }

  // FIXME : check that we have some ending pack/version directory ?
 {unsigned int linen = lines.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    std::string line = lines[linei];
    Lib::smanip::strip(line);
    if(line.size()&&(line[0]=='#')) continue;
    if(!line.size()) continue;
    std::vector<std::string> words;
    // line is of the form : <reposiory> <package/version>
    // If changing the " " separator, change obuild.cxx -oshell action.
    Lib::smanip::words(line," ",words);
    unsigned int wordn = words.size();
    if(wordn!=2) {
      Lib::Out out(fPrinter);
      out << "obuild::Shell::start :"
          << " no repository given in " << Lib::sout(line)
          << Lib::endl;
      fPackages.clear();
      fRepositories.clear();
      return false;
    }
    fRepositories.push_back(words[0]);      
    fPackages.push_back(words[1]); //pv
  }}

  return true;
}

bool obuild::Shell::start(){
  // Get packages to work with :
  std::string opath;
  if(!Lib::System::getenv("OBUILD_PATH",opath)) {
    Lib::Out out(fPrinter);
    out << " obuild::Shell::start :"
        << " environment variable OBUILD_PATH not defined."
        << Lib::endl;
    out << "OBUILD_PATH is a list of head directories" << Lib::endl;
    out << "containing packages. Directories are ':' separated" << Lib::endl;
    out << "on a UNIX and ';' separated on a Windows." << Lib::endl;
    return false;
  }
    
  fHistory.clear();
  fShowTime = false;

  while(true) {

    closeOut();

    ::printf("obuild>> ");

    char buffer[BUFSIZE+1];
   {if(::fgets(buffer,BUFSIZE,stdin)==NULL) break; // EOF.
    int l = ::strlen(buffer);
    if( (l>=2) && (buffer[l-2]=='\r') && (buffer[l-1]=='\n') ) {
      buffer[l-2] = '\0';
    } else if( (l>=1) && (buffer[l-1]=='\n') ) {
      buffer[l-1] = '\0';
    }}
    std::string sbuffer(buffer);

    //printf("debug : %s\n",sbuffer.c_str());

    if(sbuffer=="exit") return true;

    dispatch(sbuffer);
  }

  return true;
}

void obuild::Shell::dispatch(const std::string& aCommand){
  if(aCommand=="help") {
    Lib::Out out(fPrinter);
    out << "obuild>> help" << Lib::endl;
    out << "obuild>> exit" << Lib::endl;
    out << "obuild>> quit" << Lib::endl;
    out << "obuild>> erase" << Lib::endl;
    out << "obuild>> build" << Lib::endl;
    out << "obuild>> build -v" << Lib::endl;
    out << "obuild>> build log" << Lib::endl;
    out << "obuild>> build Qt" << Lib::endl;
    out << "obuild>> build Python" << Lib::endl;
    out << "obuild>> build SDL" << Lib::endl;
    out << "obuild>> build NextStep" << Lib::endl;
    out << "obuild>> install" << Lib::endl;
    out << "obuild>> cvsu" << Lib::endl;
//  out << "obuild>> commit" << Lib::endl;
    out << "obuild>> co" << Lib::endl;
    out << "obuild>> check versions" << Lib::endl;
    out << "obuild>> checks" << Lib::endl;
    out << "obuild>> time" << Lib::endl;
    out << "obuild>> [sh command]" << Lib::endl;
    out << "obuild>> ![begin of command in history]" << Lib::endl;

  } else if(aCommand=="!!") {
    if(!fHistory.size()) return;
    std::string cmd = fHistory[fHistory.size()-1]; //copy the string.
    dispatch(cmd);

  } else if((aCommand.size()>=1)&&(aCommand.substr(0,1)=="!")) {
    // Executed a command saved in the history.
    std::string s = aCommand.substr(1,aCommand.size()-1);
    int index;
    if(Lib::smanip::toint(s,index)) {
      index--;
      if((index<0)||(index>=(int)fHistory.size())) {
        Lib::Out out(fPrinter);
        out << "obuild::Shell::dispatch :"
            << " numbered command not found."
            << Lib::endl;
      } else {
        std::string cmd = fHistory[index]; //copy the string.
        dispatch(cmd);
      }

    } else {
      bool found = false;
      std::vector<std::string>::reverse_iterator it;
      for(it = fHistory.rbegin();it!=fHistory.rend();++it) {
        std::string beg = (*it).substr(0,s.size());
        if(beg==s) {
          found = true;
          std::string cmd = *it; //copy the string.
          dispatch(cmd);
          break;
        }
      }      
      if(!found) {
        Lib::Out out(fPrinter);
        out << "obuild::Shell::dispatch :"
            << " command not found."
            << Lib::endl;
      }
    }

  } else if(aCommand=="history") {
    // Show the history :
    unsigned int index = 1;
    std::vector<std::string>::iterator it;
    for(it = fHistory.begin();it!=fHistory.end();++it,index++) {
      Lib::Out out(fPrinter);
      std::string s;
      Lib::smanip::printf(s,32,"%d",index);  
      Lib::smanip::justify(s,6,Lib::smanip::right);
      out << s << "  " << (*it) << Lib::endl;
    }      

  } else if(aCommand=="time") {
    fHistory.push_back(aCommand);
    fShowTime = !fShowTime;
    Lib::Out out(fPrinter);
    out << "show time " << Lib::sout(fShowTime) << Lib::endl;

  } else if(aCommand=="build") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build());

  } else if(aCommand=="build -v") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-v"));

  } else if(aCommand=="build log") {
    fHistory.push_back(aCommand);
    del("obuild.log");
    process_packages(cmd_build()+cmd_to_log());

  } else if(aCommand=="build Qt") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-group Qt"));

  } else if(aCommand=="build Python") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-group Python"));

  } else if(aCommand=="build SDL") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-group SDL"));

  } else if(aCommand=="build NextStep") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-group NextStep"));

  } else if(aCommand=="build Java") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-group Java"));

  } else if(aCommand=="build darwin_app") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("darwin_app"));

  } else if(aCommand=="erase") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-e"));

  } else if(aCommand=="install") {
    fHistory.push_back(aCommand);
    process_packages(cmd_build("-v -bci"));

  /////////////////////////////////////
  /// CVS : ///////////////////////////
  /////////////////////////////////////
  } else if(aCommand=="cvsu") {
    fHistory.push_back(aCommand);
    process_packages("cvs -n update");

  } else if(aCommand=="tag") {
    fHistory.push_back(aCommand);
    process_packages("tag");

//} else if(aCommand=="commit") {
//  fHistory.push_back(aCommand);
//  process_packages("cvs commit \"\"");

  } else if(aCommand=="check versions") {
    fHistory.push_back(aCommand);
    process_packages("check versions");

  } else if(aCommand=="check setups") {
    fHistory.push_back(aCommand);
    process_packages("check setups");

  } else if(aCommand=="checks") {
    fHistory.push_back(aCommand);
    process_packages("check versions");
    process_packages("check setups");

  } else if(aCommand=="co") {

    std::string user = "anonymous";

    unsigned int packn = fPackages.size();
    for(unsigned int packi=0;packi<packn;packi++) {
      std::string rep = fRepositories[packi];
      std::string pv = fPackages[packi];

      Lib::smanip::replace(rep,"@@user@@",user);

      std::string r;
      std::string p;
      std::string v;
      if(!r_p_v(pv,r,p,v)) return;
      if(rep.empty()) {
        Lib::Out out(fPrinter);
        out << "package " << Lib::sout(pv) << " has no repository." 
        << Lib::endl;
        continue;
      }

      std::string sys_cmd;
      sys_cmd += "CVSROOT="+rep;
      sys_cmd += cmd_and();
      sys_cmd += "export CVSROOT";
      sys_cmd += cmd_and();
      if(r.empty()) {
        sys_cmd += "cvs checkout -r "+v+" "+p;
      } else {  
        //used in osc_source/projects.oshell
        sys_cmd += "cvs checkout -d "+p+" -r "+v+" "+r+"/"+p;
      }

      if(Lib::System::isWindows_NT()) {
        //printf("debug : cmd : %s\n",sys_cmd.c_str());
        /*int status =*/ Lib::System::execute(sys_cmd);
      } else {
        //printf("debug : cmd : %s\n",sys_cmd.c_str());
        /*int status =*/ Lib::System::execute(sys_cmd);
      }

    }

  /////////////////////////////////////
  /////////////////////////////////////
  /////////////////////////////////////

  } else if((aCommand.size()>=3)&&(aCommand.substr(0,3)=="no ")) {
    fHistory.push_back(aCommand);

    std::vector<std::string> words;
    Lib::smanip::words(aCommand," ",words);
    unsigned int wordn = words.size();
    if(wordn!=2) {
      Lib::Out out(fPrinter);
      out << "obuild::Shell::dispatch :"
          << " bad syntax in " << Lib::sout(aCommand)
          << Lib::endl;
    } else {
      std::string what = words[1];
      Lib::smanip::removeLinesWith(fPackages,words[1]);
    }

  } else if(aCommand.empty()) {
    process_packages();

  } else {
    fHistory.push_back(aCommand);
    process_packages(aCommand);

  }
}

void obuild::Shell::process_packages(const std::string& aCommand){
  Lib::Time begin = Lib::Time::now();
  openOut(); //Open the out file.

  unsigned int linen = fPackages.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    std::string pv = fPackages[linei];
    Lib::smanip::strip(pv);
    if(pv.size()&&(pv[0]=='#')) continue;
    if(!pv.size()) continue;

    Lib::Out out(fPrinter);
    out << "process : " << pv << Lib::endl;
    //out << "command : " << aCommand << Lib::endl;

    std::string r;
    std::string p;
    std::string v;
    if(!r_p_v(pv,r,p,v)) continue;

    if(aCommand.empty()) continue;

    Lib::Time proc_begin = Lib::Time::now();

    if(aCommand=="check versions") {
      check_versions(p);

    } else if(aCommand=="check setups") {
      check_obuild_setups(p);

    } else if(aCommand=="tag") {
      std::string top = ".."+sep()+"..";
      std::string dir_p = top+sep()+p;
      std::string dir_pv = top+sep()+p+sep()+v;
      std::string sys_cmd = 
        "if [ -d \""+dir_pv+"\" ] ; then cd "+dir_pv+" ;else cd "+dir_p+" ;fi";
      sys_cmd += cmd_and();

      std::string tag = v;
     {std::string::size_type dot = v.find('.');
      if(dot!=std::string::npos){
        std::vector<unsigned int> verss;
        if(!Lib::smanip::version(v,verss)) {
          out << v << " has not the syntax of a version." << Lib::endl;
          continue;
        }
        if(verss.size()==1) {
          tag = "v"+Lib::smanip::tostring(verss[0]);
        } else if(verss.size()==2) {
          tag  = "v"+Lib::smanip::tostring(verss[0]);
          tag += "r"+Lib::smanip::tostring(verss[1]);
        } else if(verss.size()==3) {
          tag  = "v"+Lib::smanip::tostring(verss[0]);
          tag += "r"+Lib::smanip::tostring(verss[1]);
          tag += "p"+Lib::smanip::tostring(verss[2]);
        } else {
          out << v << " has more than three items. Not treated." << Lib::endl;
          continue;
        }
      }}

      out << "tag package " << p << " with " << tag << " ..." << Lib::endl;

      sys_cmd += "cvs tag -F "+tag;
      //printf("debug : cmd : %s\n",sys_cmd.c_str());
      /*int status =*/ Lib::System::execute(sys_cmd);

    } else {
      // Else, assume a system shell command from the package root directory :

      std::string top = ".."+sep()+"..";
      std::string dir_p = top+sep()+p;
      std::string dir_pv = top+sep()+p+sep()+v;
      std::string sys_cmd = 
        "if [ -d \""+dir_pv+"\" ] ; then cd "+dir_pv+" ;else cd "+dir_p+" ;fi";
      sys_cmd += cmd_and();

      std::string s = aCommand;
      Lib::smanip::replace(s,"@@package@@",p);
      Lib::smanip::replace(s,"@@version@@",v);
      sys_cmd += s;
      //printf("debug : cmd : %s\n",sys_cmd.c_str());
      /*int status =*/ Lib::System::execute(sys_cmd);
    }
  
    if(fShowTime) {
      Lib::Time proc_end = Lib::Time::elapsed(proc_begin);
      Lib::Out out(fPrinter);
      unsigned int secs = (unsigned int)proc_end.seconds();
      unsigned int mns = secs/60;
      secs = secs-mns*60;
      out << "time : " << mns << " minutes " << secs << " seconds." 
          << Lib::endl;
    }
  }

 {Lib::Time end = Lib::Time::elapsed(begin);
  Lib::Out out(fPrinter);
  unsigned int secs = (unsigned int)end.seconds();
  unsigned int mns = secs/60;
  secs = secs-mns*60;
  out << "total time : " << mns << " minutes " << secs << " seconds." 
      << Lib::endl;}
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Shell::get_cvs_version(const std::string& aPV,std::string& aTag){
  aTag = "";

  std::string pwd;
  Lib::dirmanip::directory(pwd);
  std::string cvs_log = pwd+sep()+"cvs.log";

  std::string top = ".."+sep()+"..";
  std::string dir_pack = top+sep()+aPV;
  
 {std::string dir_obuild = dir_pack+sep()+"obuild";
  if(!Lib::dirmanip::exists(dir_obuild)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::get_cvs_version :"
        << " directory " << Lib::sout(dir_obuild) 
        << " not found."
        << Lib::endl;
    return false;
  }
  std::string sys_cmd;
  sys_cmd  = "cd "+dir_obuild;
  sys_cmd += cmd_and();
  sys_cmd += "cvs log -h this.obuild";
  sys_cmd += " > " + quote(cvs_log);
  Lib::System::execute(sys_cmd);}

  // Find last tag :
  bool found = false;
  std::string cvs_tag;
 {std::vector<std::string> lines;
  if(!Lib::fmanip::read(cvs_log,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::get_cvs_version :"
        << " can't read " << Lib::sout(cvs_log)
        << Lib::endl;
    return false;
  }
  unsigned int linen = lines.size();
  bool start_search = false;
  for(unsigned int linei=0;linei<linen;linei++) {
    const std::string& line = lines[linei];
    if(!start_search) {
      if(line=="symbolic names:") start_search = true;
      continue;
    }

    cvs_tag = lines[linei]; //Candidate for a version tag.
    Lib::smanip::strip(cvs_tag,Lib::smanip::both,'\t');
    Lib::smanip::strip(cvs_tag);

    std::string::size_type double_dot = cvs_tag.find(':');
    if(double_dot==std::string::npos){
      Lib::Out out(fPrinter);
      out << "obuild::Shell::get_cvs_version :"
          << " bad syntax in " << Lib::sout(cvs_tag) 
          << " for package " << Lib::sout(aPV) << "."
          << Lib::endl;
      return false;
    }
    cvs_tag = cvs_tag.substr(0,double_dot);
    // Look for a version tag :
    if(!cvs_tag.size()) continue;
    if((cvs_tag[0]!='v')&&(cvs_tag[0]!='V')) continue;
    std::vector<unsigned int> verss;
    if(!Lib::smanip::version(cvs_tag,verss)) continue;
    found = true;
    break;
  }}
  if(!found) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::get_cvs_version :"
        << " no tag found for package " << Lib::sout(aPV) << "."
        << Lib::endl;
    return false;
  }
  Lib::Out out(fPrinter);
  out << "tag : " << cvs_tag << Lib::endl;    

  del(quote(cvs_log));

  aTag = cvs_tag;

  return true;
}

bool obuild::Shell::has_changed(
 const std::string& aPV
,const std::string& aTag
,bool& aChanged
){
  aChanged = false;

  std::string pwd;
  Lib::dirmanip::directory(pwd);
  std::string cvs_log = pwd+sep()+"cvs.log";

  std::string top = ".."+sep()+"..";
  std::string dir_pack = top+sep()+aPV;

 {std::string sys_cmd;
  sys_cmd  = "cd "+dir_pack;
  sys_cmd += cmd_and();
  sys_cmd += "cvs -n update -r " + aTag;
  sys_cmd += cmd_redirect() + cvs_log;
  Lib::System::execute(sys_cmd);}

  // Search for some leading "U " :
  bool found = false;
  std::string sdiff;
 {std::vector<std::string> lines;
  if(!Lib::fmanip::read(cvs_log,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::has_changed :"
        << " can't read " << Lib::sout(cvs_log)
        << Lib::endl;
    return false;
  }
  unsigned int linen = lines.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    const std::string& line = lines[linei];
    if(line.size()<2) continue;
    if(line.substr(0,2)!="U ") continue;
    sdiff = line;
    found = true;
    break;
  }}
  aChanged = found;
  return true;
}

bool obuild::Shell::check_versions(const std::string& aPV){
  // Check consistency of :
  // - version in the last version tag of obuild/this.obuild.
  std::string p,n,s;
  Lib::smanip::pathNameSuffix(aPV,p,n,s);
  std::string path_version = n;
 {std::string cvs_version;        
  if(!get_cvs_version(aPV,cvs_version)) return false;
  if(path_version!=cvs_version) {
    Lib::Out out(fPrinter);
    out << "For " << aPV << ", tag/path version mismatch." << Lib::endl;
    out << " path version is " << path_version << Lib::endl;
    out << " version tag is " << cvs_version << Lib::endl;
    return false;    
  }}
  Lib::Out out(fPrinter);
  out << "Versions ok for " << aPV << "." << Lib::endl;
  return true;
}
/*
bool obuild::Shell::check_version_consistency(const std::string& aPV){
  // Check consistency of :
  // - version in the path.
  // - version in cmt/requirements
  // - version in the last version tag of cmt/requirements.
  std::string tag;        
  if(!get_cvs_tag(line,tag)) return false;
  bool changed;
  has_changed(line,tag,changed);
  if(!changed) {
    Lib::Out out(fPrinter);
    out << "package : " << line << " not changed." << Lib::endl;
  } else {
    Lib::Out out(fPrinter);
    out << "package : " << line << " changed." << Lib::endl;
      // Save changed packages :
    if(fOut) {
      Lib::Out obuild_out(*fOut);
      obuild_out << "package : " << line << " changed." << Lib::endl;
      fOut->flush();
    }
  }
  return true;
}
*/
bool obuild::Shell::check_obuild_setups(const std::string& aPV){
  // Check that the obuild/setup[.sh,.csh] files are not the installed ones.
  std::string top = ".."+sep()+"..";
  std::string dir_pack = top+sep()+aPV;

  std::string p;
  if(aPV.find('/')==std::string::npos) {
    p = aPV;
  } else {
    std::string r;
    std::string v;
    if(!r_p_v(aPV,r,p,v)) return false;
  }

 {std::string file = dir_pack+sep()+"obuild"+sep()+"setup.sh";
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(file,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::check_obuild_setups :"
        << " can't read " << Lib::sout(file)
        << Lib::endl;
    return false;
  }
  bool found = false;
  std::string search = "obuild_"+p+"_path=\"@@absolute_dir@@\"";
  unsigned int linen = lines.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    const std::string& line = lines[linei];
    if(line==search) {
      found = true;
      break;
    }
  }
  if(!found) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::check_obuild_setup :"
        << " installed setup.sh for package "
        << aPV << "."
        << Lib::endl;
  }}

 {std::string file = dir_pack+sep()+"obuild"+sep()+"setup.csh";
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(file,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::check_obuild_setups :"
        << " can't read " << Lib::sout(file)
        << Lib::endl;
    return false;
  }
  bool found = false;
  std::string search = "set obuild_"+p+"_path=\"@@absolute_dir@@\"";
  unsigned int linen = lines.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    const std::string& line = lines[linei];
    if(line==search) {
      found = true;
      break;
    }
  }
  if(!found) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::check_obuild_setup :"
        << " installed setup.csh for package "
        << aPV << "."
        << Lib::endl;
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Shell::sep() {
  if(Lib::System::isWindows_NT()) {
    return "\\";
  } else {
    return "/";
  }
}
std::string obuild::Shell::cmd_and() {
  if(Lib::System::isWindows_NT()) {
    return " & ";
  } else {
    return " ; ";
  }
}
std::string obuild::Shell::cmd_redirect() {
  if(Lib::System::isWindows_NT()) {
    return " > ";
  } else {
    return " > "; //FIXME : correct ?
  }
}
std::string obuild::Shell::cmd_build(const std::string& aOptions){
  if(Lib::System::isWindows_NT()) {
    return "CD obuild & DOS\\build "+aOptions;
  } else {
    return "cd obuild ; sh/build "+aOptions;
  }
}
void obuild::Shell::del(const std::string& aFile) {
  if(Lib::System::isWindows_NT()) {
    std::string sys_cmd;
    sys_cmd = "IF EXIST "+aFile+" DEL "+aFile;
    Lib::System::execute(sys_cmd);
  } else {
    std::string sys_cmd;
    sys_cmd = "/bin/rm -f "+aFile;
  }
}
std::string obuild::Shell::cmd_to_log() {
  std::string cmd;
  std::string pwd;
  Lib::dirmanip::directory(pwd);
  if(Lib::System::isWindows_NT()) {
    cmd = " -v >> " + pwd + "\\obuild.log";
  } else {
    cmd = " -v 2>> " + pwd + "/obuild.log";
  }
  return cmd;
}
void obuild::Shell::removeOut(){
  std::string out_file = "obuild.out";
  Lib::fmanip::remove(out_file);
}
bool obuild::Shell::openOut(){
  if(fOut) return true; //Done.
  // Open the out file :
  std::string out_file = "obuild.out";
  Lib::fmanip::remove(out_file);
  fOut = new Lib::FileWriter(out_file,fPrinter);
  if(!fOut->enabled()) {
    Lib::Out out(fPrinter);
    out << " obuild::Shell::opeOut :"
        << " can't open file " << Lib::sout(out_file) 
        << Lib::endl;
    delete fOut;
    fOut = 0;
    return false;
  }
  return true;
}
bool obuild::Shell::closeOut(){
  if(!fOut) return true;
  fOut->disable();
  delete fOut;
  fOut = 0;
  return true;
}
bool obuild::Shell::r_p_v(
 const std::string& aPV
,std::string& aRepository
,std::string& aPackage
,std::string& aVersion
){
  std::vector<std::string> words;
  Lib::smanip::words(aPV,"/",words);
  unsigned int wordn = words.size();
  if((wordn!=2)&&(wordn!=3)) {
    Lib::Out out(fPrinter);
    out << "obuild::Shell::r_p_v :"
        << " bad syntax in " << Lib::sout(aPV)
        << Lib::endl;
    aRepository = "";
    aPackage = "";
    aVersion = "";
    return false;
  }
  if(wordn==2) {
    aRepository = "";
    aPackage = words[0];
    aVersion = words[1];
  } else { //wordn==3
    aRepository = words[0];
    aPackage = words[1];
    aVersion = words[2];
  }
  return true;
}
std::string obuild::Shell::quote(const std::string& aString){
  return std::string("\"")+aString+"\"";
}
